@@ -37,6 +37,8 @@
 #include "account.h"
 #include "prefs.h"
 #include "prefs_account.h"
+#include "procmsg.h"
+#include "procheader.h"
 #include "compose.h"
 #include "manage_window.h"
 #include "stock_pixmap.h"
@@ -226,6 +228,90 @@ PrefsAccount *account_find_from_id(gint id)
 	return NULL;
 }
 
+PrefsAccount *account_find_from_item(FolderItem *item)
+{
+	PrefsAccount *ac;
+
+	g_return_val_if_fail(item != NULL, NULL);
+
+	ac = item->account;
+	if (!ac) {
+		FolderItem *cur_item = item->parent;
+		while (cur_item != NULL) {
+			if (cur_item->account && cur_item->ac_apply_sub) {
+				ac = cur_item->account;
+				break;
+			}
+			cur_item = cur_item->parent;
+		}
+	}
+	if (!ac)
+		ac = item->folder->account;
+
+	return ac;
+}
+
+PrefsAccount *account_find_from_message_file(const gchar *file)
+{
+	static HeaderEntry hentry[] = {{"From:",		  NULL, FALSE},
+				       {"X-Sylpheed-Account-Id:", NULL, FALSE},
+				       {"AID:",			  NULL, FALSE}};
+
+	enum
+	{
+		H_FROM			= 0,
+		H_X_SYLPHEED_ACCOUNT_ID = 1,
+		H_AID			= 2
+	};
+
+	PrefsAccount *ac = NULL;
+	FILE *fp;
+	gchar *str;
+	gchar buf[BUFFSIZE];
+	gint hnum;
+
+	g_return_val_if_fail(file != NULL, NULL);
+
+	if ((fp = fopen(file, "rb")) == NULL) {
+		FILE_OP_ERROR(file, "fopen");
+		return NULL;
+	}
+
+	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
+	       != -1) {
+		str = buf + strlen(hentry[hnum].name);
+		if (hnum == H_FROM)
+			ac = account_find_from_address(str);
+		else if (hnum == H_X_SYLPHEED_ACCOUNT_ID || hnum == H_AID) {
+			PrefsAccount *tmp_ac;
+
+			tmp_ac = account_find_from_id(atoi(str));
+			if (tmp_ac) {
+				ac = tmp_ac;
+				break;
+			}
+		}
+	}
+
+	fclose(fp);
+	return ac;
+}
+
+PrefsAccount *account_find_from_msginfo(MsgInfo *msginfo)
+{
+	gchar *file;
+	PrefsAccount *ac;
+
+	file = procmsg_get_message_file(msginfo);
+	ac = account_find_from_message_file(file);
+	g_free(file);
+
+	if (!ac && msginfo->folder)
+		ac = account_find_from_item(msginfo->folder);
+
+	return ac;
+}
+
 void account_set_menu(void)
 {
 	main_window_set_account_menu(account_list);
@@ -300,11 +386,38 @@ void account_add(void)
 		ac_prefs->folder = REMOTE_FOLDER(folder);
 		folder_add(folder);
 		if (ac_prefs->protocol == A_IMAP4)
-			folder->create_tree(folder);
+			folder->klass->create_tree(folder);
 		folderview_set_all();
 	}
 }
 
+void account_open(PrefsAccount *ac_prefs)
+{
+	gboolean prev_default;
+	gchar *ac_name;
+
+	g_return_if_fail(ac_prefs != NULL);
+
+	prev_default = ac_prefs->is_default;
+	Xstrdup_a(ac_name, ac_prefs->account_name ? ac_prefs->account_name : "",
+		  return);
+
+	prefs_account_open(ac_prefs);
+
+	if (!prev_default && ac_prefs->is_default)
+		account_set_as_default(ac_prefs);
+
+	if (ac_prefs->folder && strcmp2(ac_name, ac_prefs->account_name) != 0) {
+		folder_set_name(FOLDER(ac_prefs->folder),
+				ac_prefs->account_name);
+		folderview_set_all();
+	}
+
+	account_save_config_all();
+	account_set_menu();
+	main_window_reflect_prefs_all();
+}
+
 void account_set_as_default(PrefsAccount *ac_prefs)
 {
 	PrefsAccount *ap;
@@ -356,7 +469,7 @@ void account_set_missing_folder(void)
 			ap->folder = REMOTE_FOLDER(folder);
 			folder_add(folder);
 			if (ap->protocol == A_IMAP4)
-				folder->create_tree(folder);
+				folder->klass->create_tree(folder);
 		}
 	}
 }
@@ -368,7 +481,14 @@ FolderItem *account_get_special_folder(PrefsAccount *ac_prefs,
 
 	g_return_val_if_fail(ac_prefs != NULL, NULL);
 
-	if (type == F_OUTBOX) {
+	switch (type) {
+	case F_INBOX:
+		if (ac_prefs->folder)
+			item = FOLDER(ac_prefs->folder)->inbox;
+		if (!item)
+			item = folder_get_default_inbox();
+		break;
+	case F_OUTBOX:
 		if (ac_prefs->set_sent_folder && ac_prefs->sent_folder) {
 			item = folder_find_item_from_identifier
 				(ac_prefs->sent_folder);
@@ -379,7 +499,8 @@ FolderItem *account_get_special_folder(PrefsAccount *ac_prefs,
 			if (!item)
 				item = folder_get_default_outbox();
 		}
-	} else if (type == F_DRAFT) {
+		break;
+	case F_DRAFT:
 		if (ac_prefs->set_draft_folder && ac_prefs->draft_folder) {
 			item = folder_find_item_from_identifier
 				(ac_prefs->draft_folder);
@@ -390,12 +511,14 @@ FolderItem *account_get_special_folder(PrefsAccount *ac_prefs,
 			if (!item)
 				item = folder_get_default_draft();
 		}
-	} else if (type == F_QUEUE) {
+		break;
+	case F_QUEUE:
 		if (ac_prefs->folder)
 			item = FOLDER(ac_prefs->folder)->queue;
 		if (!item)
 			item = folder_get_default_queue();
-	} else if (type == F_TRASH) {
+		break;
+	case F_TRASH:
 		if (ac_prefs->set_trash_folder && ac_prefs->trash_folder) {
 			item = folder_find_item_from_identifier
 				(ac_prefs->trash_folder);
@@ -406,6 +529,9 @@ FolderItem *account_get_special_folder(PrefsAccount *ac_prefs,
 			if (!item)
 				item = folder_get_default_trash();
 		}
+		break;
+	default:
+		break;
 	}
 
 	return item;
@@ -415,6 +541,8 @@ void account_destroy(PrefsAccount *ac_prefs)
 {
 	g_return_if_fail(ac_prefs != NULL);
 
+	folder_unref_account_all(ac_prefs);
+
 	prefs_account_free(ac_prefs);
 	account_list = g_list_remove(account_list, ac_prefs);
 
@@ -520,8 +648,8 @@ static void account_edit_create(void)
 
 	gtk_signal_connect (GTK_OBJECT (clist), "select_row",
 			    GTK_SIGNAL_FUNC (account_selected), NULL);
-	gtk_signal_connect (GTK_OBJECT (clist), "row_move",
-			    GTK_SIGNAL_FUNC (account_row_moved), NULL);
+	gtk_signal_connect_after (GTK_OBJECT (clist), "row_move",
+				  GTK_SIGNAL_FUNC (account_row_moved), NULL);
 
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
@@ -597,28 +725,12 @@ static void account_edit_prefs(void)
 	GtkCList *clist = GTK_CLIST(edit_account.clist);
 	PrefsAccount *ac_prefs;
 	gint row;
-	gboolean prev_default;
-	gchar *ac_name;
 
 	if (!clist->selection) return;
 
 	row = GPOINTER_TO_INT(clist->selection->data);
 	ac_prefs = gtk_clist_get_row_data(clist, row);
-	prev_default = ac_prefs->is_default;
-	Xstrdup_a(ac_name, ac_prefs->account_name ? ac_prefs->account_name : "",
-		  return);
-
-	prefs_account_open(ac_prefs);
-
-	if (!prev_default && ac_prefs->is_default)
-		account_set_as_default(ac_prefs);
-
-	if ((ac_prefs->protocol == A_IMAP4 || ac_prefs->protocol == A_NNTP) &&
-	    ac_prefs->folder && strcmp2(ac_name, ac_prefs->account_name) != 0) {
-		folder_set_name(FOLDER(ac_prefs->folder),
-				ac_prefs->account_name);
-		folderview_set_all();
-	}
+	account_open(ac_prefs);
 
 	account_clist_set();
 }
@@ -735,10 +847,8 @@ static void account_selected(GtkCList *clist, gint row, gint column,
 static void account_row_moved(GtkCList *clist, gint source_row, gint dest_row)
 {
 	account_list_set();
-	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL) {
-		gtk_clist_moveto(clist, dest_row, -1,
-				 source_row < dest_row ? 1.0 : 0.0, 0.0);
-	}
+	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL)
+		gtk_clist_moveto(clist, dest_row, -1, 0.5, 0.0);
 }
 
 static void account_key_pressed(GtkWidget *widget, GdkEventKey *event,
