@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,13 +60,13 @@ Folder *folder_new(FolderType type, const gchar *name, const gchar *path)
 	name = name ? name : path;
 	switch (type) {
 	case F_MH:
-		folder = mh_folder_new(name, path);
+		folder = mh_get_class()->folder_new(name, path);
 		break;
 	case F_IMAP:
-		folder = imap_folder_new(name, path);
+		folder = imap_get_class()->folder_new(name, path);
 		break;
 	case F_NEWS:
-		folder = news_folder_new(name, path);
+		folder = news_get_class()->folder_new(name, path);
 		break;
 	default:
 		return NULL;
@@ -82,7 +82,6 @@ static void folder_init(Folder *folder, const gchar *name)
 	g_return_if_fail(folder != NULL);
 
 	folder_set_name(folder, name);
-	folder->type = F_UNKNOWN;
 	folder->account = NULL;
 	folder->inbox = NULL;
 	folder->outbox = NULL;
@@ -93,7 +92,7 @@ static void folder_init(Folder *folder, const gchar *name)
 	folder->ui_func_data = NULL;
 	item = folder_item_new(name, NULL);
 	item->folder = folder;
-	folder->node = g_node_new(item);
+	folder->node = item->node = g_node_new(item);
 	folder->data = NULL;
 }
 
@@ -114,9 +113,9 @@ void folder_remote_folder_init(Folder *folder, const gchar *name,
 void folder_destroy(Folder *folder)
 {
 	g_return_if_fail(folder != NULL);
-	g_return_if_fail(folder->destroy != NULL);
+	g_return_if_fail(folder->klass->destroy != NULL);
 
-	folder->destroy(folder);
+	folder->klass->destroy(folder);
 
 	folder_list = g_list_remove(folder_list, folder);
 
@@ -163,7 +162,6 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->stype = F_NORMAL;
 	item->name = g_strdup(name);
 	item->path = g_strdup(path);
-	item->account = NULL;
 	item->mtime = 0;
 	item->new = 0;
 	item->unread = 0;
@@ -172,10 +170,18 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->no_sub = FALSE;
 	item->no_select = FALSE;
 	item->collapsed = FALSE;
-	item->threaded  = TRUE;
-	item->opened    = FALSE;
+	item->threaded = TRUE;
+	item->opened = FALSE;
+	item->node = NULL;
 	item->parent = NULL;
 	item->folder = NULL;
+	item->account = NULL;
+	item->ac_apply_sub = FALSE;
+	item->auto_to = NULL;
+	item->use_auto_to_on_reply = FALSE;
+	item->auto_cc = NULL;
+	item->auto_bcc = NULL;
+	item->auto_replyto = NULL;
 	item->mark_queue = NULL;
 	item->data = NULL;
 
@@ -184,19 +190,22 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 
 void folder_item_append(FolderItem *parent, FolderItem *item)
 {
-	GNode *node;
-
 	g_return_if_fail(parent != NULL);
 	g_return_if_fail(parent->folder != NULL);
+	g_return_if_fail(parent->node != NULL);
 	g_return_if_fail(item != NULL);
 
-	node = parent->folder->node;
-	node = g_node_find(node, G_PRE_ORDER, G_TRAVERSE_ALL, parent);
-	g_return_if_fail(node != NULL);
-
 	item->parent = parent;
 	item->folder = parent->folder;
-	g_node_append_data(node, item);
+	item->node = g_node_append_data(parent->node, item);
+}
+
+static gboolean folder_item_remove_func(GNode *node, gpointer data)
+{
+	FolderItem *item = FOLDER_ITEM(node->data);
+
+	folder_item_destroy(item);
+	return FALSE;
 }
 
 void folder_item_remove(FolderItem *item)
@@ -205,23 +214,60 @@ void folder_item_remove(FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->node != NULL);
 
-	node = item->folder->node;
-	node = g_node_find(node, G_PRE_ORDER, G_TRAVERSE_ALL, item);
-	g_return_if_fail(node != NULL);
+	node = item->node;
 
-	/* TODO: free all FolderItem's first */
 	if (item->folder->node == node)
 		item->folder->node = NULL;
+
+	g_node_traverse(node, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+			folder_item_remove_func, NULL);
 	g_node_destroy(node);
 }
 
+void folder_item_remove_children(FolderItem *item)
+{
+	GNode *node, *next;
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->node != NULL);
+
+	node = item->node->children;
+	while (node != NULL) {
+		next = node->next;
+		folder_item_remove(FOLDER_ITEM(node->data));
+		node = next;
+	}
+}
+
 void folder_item_destroy(FolderItem *item)
 {
+	Folder *folder;
+
 	g_return_if_fail(item != NULL);
 
+	folder = item->folder;
+	if (folder) {
+		if (folder->inbox == item)
+			folder->inbox = NULL;
+		else if (folder->outbox == item)
+			folder->outbox = NULL;
+		else if (folder->draft == item)
+			folder->draft = NULL;
+		else if (folder->queue == item)
+			folder->queue = NULL;
+		else if (folder->trash == item)
+			folder->trash = NULL;
+	}
+
 	g_free(item->name);
 	g_free(item->path);
+	g_free(item->auto_to);
+	g_free(item->auto_cc);
+	g_free(item->auto_bcc);
+	g_free(item->auto_replyto);
 	g_free(item);
 }
 
@@ -251,16 +297,8 @@ void folder_tree_destroy(Folder *folder)
 {
 	g_return_if_fail(folder != NULL);
 
-	/* TODO: destroy all FolderItem before */
 	if (folder->node)
-		g_node_destroy(folder->node);
-
-	folder->inbox = NULL;
-	folder->outbox = NULL;
-	folder->draft = NULL;
-	folder->queue = NULL;
-	folder->trash = NULL;
-	folder->node = NULL;
+		folder_item_remove(FOLDER_ITEM(folder->node->data));
 }
 
 void folder_add(Folder *folder)
@@ -273,15 +311,15 @@ void folder_add(Folder *folder)
 
 	for (i = 0, cur = folder_list; cur != NULL; cur = cur->next, i++) {
 		cur_folder = FOLDER(cur->data);
-		if (folder->type == F_MH) {
-			if (cur_folder->type != F_MH) break;
-		} else if (folder->type == F_IMAP) {
-			if (cur_folder->type != F_MH &&
-			    cur_folder->type != F_IMAP) break;
-		} else if (folder->type == F_NEWS) {
-			if (cur_folder->type != F_MH &&
-			    cur_folder->type != F_IMAP &&
-			    cur_folder->type != F_NEWS) break;
+		if (FOLDER_TYPE(folder) == F_MH) {
+			if (FOLDER_TYPE(cur_folder) != F_MH) break;
+		} else if (FOLDER_TYPE(folder) == F_IMAP) {
+			if (FOLDER_TYPE(cur_folder) != F_MH &&
+			    FOLDER_TYPE(cur_folder) != F_IMAP) break;
+		} else if (FOLDER_TYPE(folder) == F_NEWS) {
+			if (FOLDER_TYPE(cur_folder) != F_MH &&
+			    FOLDER_TYPE(cur_folder) != F_IMAP &&
+			    FOLDER_TYPE(cur_folder) != F_NEWS) break;
 		}
 	}
 
@@ -329,7 +367,7 @@ void folder_write_list(void)
 	PrefFile *pfile;
 
 	path = folder_get_list_path();
-	if ((pfile = prefs_write_open(path)) == NULL) return;
+	if ((pfile = prefs_file_open(path)) == NULL) return;
 
 	fprintf(pfile->fp, "<?xml version=\"1.0\" encoding=\"%s\"?>\n",
 		conv_get_current_charset_str());
@@ -342,56 +380,115 @@ void folder_write_list(void)
 
 	fputs("</folderlist>\n", pfile->fp);
 
-	if (prefs_write_close(pfile) < 0)
+	if (prefs_file_close(pfile) < 0)
 		g_warning("failed to write folder list.\n");
 }
 
-struct TotalMsgCount
+struct TotalMsgStatus
 {
 	guint new;
 	guint unread;
 	guint total;
+	GString *str;
 };
 
-static gboolean folder_count_total_msgs_func(GNode *node, gpointer data)
+static gboolean folder_get_status_full_all_func(GNode *node, gpointer data)
 {
 	FolderItem *item;
-	struct TotalMsgCount *count = (struct TotalMsgCount *)data;
+	struct TotalMsgStatus *status = (struct TotalMsgStatus *)data;
+	gchar *id;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
 
 	item = FOLDER_ITEM(node->data);
-	count->new += item->new;
-	count->unread += item->unread;
-	count->total += item->total;
+
+	if (!item->path) return FALSE;
+
+	status->new += item->new;
+	status->unread += item->unread;
+	status->total += item->total;
+
+	if (status->str) {
+		id = folder_item_get_identifier(item);
+		g_string_sprintfa(status->str, "%5d %5d %5d %s\n",
+				  item->new, item->unread,
+				  item->total, id);
+		g_free(id);
+	}
 
 	return FALSE;
 }
 
-void folder_count_total_msgs(guint *new, guint *unread, guint *total)
+static void folder_get_status_full_all(GString *str, guint *new, guint *unread,
+				       guint *total)
 {
 	GList *list;
 	Folder *folder;
-	struct TotalMsgCount count;
+	struct TotalMsgStatus status;
 
-	count.new = count.unread = count.total = 0;
+	status.new = status.unread = status.total = 0;
+	status.str = str;
 
-	debug_print(_("Counting total number of messages...\n"));
+	debug_print("Counting total number of messages...\n");
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = FOLDER(list->data);
 		if (folder->node)
 			g_node_traverse(folder->node, G_PRE_ORDER,
 					G_TRAVERSE_ALL, -1,
-					folder_count_total_msgs_func,
-					&count);
+					folder_get_status_full_all_func,
+					&status);
 	}
 
-	*new = count.new;
-	*unread = count.unread;
-	*total = count.total;
+	*new = status.new;
+	*unread = status.unread;
+	*total = status.total;
+}
 
-	return;
+gchar *folder_get_status(GPtrArray *folders, gboolean full)
+{
+	guint new, unread, total;
+	GString *str;
+	gint i;
+	gchar *ret;
+
+	new = unread = total = 0;
+
+	str = g_string_new(NULL);
+
+	if (folders) {
+		for (i = 0; i < folders->len; i++) {
+			FolderItem *item;
+
+			item = g_ptr_array_index(folders, i);
+			new += item->new;
+			unread += item->unread;
+			total += item->total;
+
+			if (full) {
+				gchar *id;
+
+				id = folder_item_get_identifier(item);
+				g_string_sprintfa(str, "%5d %5d %5d %s\n",
+						  item->new, item->unread,
+						  item->total, id);
+				g_free(id);
+			}
+		}
+	} else {
+		folder_get_status_full_all(full ? str : NULL,
+					   &new, &unread, &total);
+	}
+
+	if (full)
+		g_string_sprintfa(str, "%5d %5d %5d\n", new, unread, total);
+	else
+		g_string_sprintfa(str, "%d %d %d\n", new, unread, total);
+
+	ret = str->str;
+	g_string_free(str, FALSE);
+
+	return ret;
 }
 
 Folder *folder_find_from_path(const gchar *path)
@@ -401,7 +498,7 @@ Folder *folder_find_from_path(const gchar *path)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type == F_MH &&
+		if (FOLDER_TYPE(folder) == F_MH &&
 		    !path_cmp(LOCAL_FOLDER(folder)->rootpath, path))
 			return folder;
 	}
@@ -416,7 +513,8 @@ Folder *folder_find_from_name(const gchar *name, FolderType type)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type == type && strcmp2(name, folder->name) == 0)
+		if (FOLDER_TYPE(folder) == type &&
+		    strcmp2(name, folder->name) == 0)
 			return folder;
 	}
 
@@ -495,7 +593,7 @@ gchar *folder_get_identifier(Folder *folder)
 
 	g_return_val_if_fail(folder != NULL, NULL);
 
-	type_str = folder_get_type_string(folder->type);
+	type_str = folder_get_type_string(FOLDER_TYPE(folder));
 	return g_strconcat(type_str, "/", folder->name, NULL);
 }
 
@@ -634,7 +732,7 @@ void folder_set_missing_folders(void)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type != F_MH) continue;
+		if (FOLDER_TYPE(folder) != F_MH) continue;
 		rootitem = FOLDER_ITEM(folder->node->data);
 		g_return_if_fail(rootitem != NULL);
 
@@ -642,7 +740,7 @@ void folder_set_missing_folders(void)
 		    folder->queue && folder->trash)
 			continue;
 
-		if (folder->create_tree(folder) < 0) {
+		if (folder->klass->create_tree(folder) < 0) {
 			g_warning("%s: can't create the folder tree.\n",
 				  LOCAL_FOLDER(folder)->rootpath);
 			continue;
@@ -656,6 +754,33 @@ void folder_set_missing_folders(void)
 	}
 }
 
+static gboolean folder_unref_account_func(GNode *node, gpointer data)
+{
+	FolderItem *item = node->data;
+	PrefsAccount *account = data;
+
+	if (item->account == account)
+		item->account = NULL;
+
+	return FALSE;
+}
+
+void folder_unref_account_all(PrefsAccount *account)
+{
+	Folder *folder;
+	GList *list;
+
+	if (!account) return;
+
+	for (list = folder_list; list != NULL; list = list->next) {
+		folder = list->data;
+		if (folder->account == account)
+			folder->account = NULL;
+		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+				folder_unref_account_func, account);
+	}
+}
+
 #undef CREATE_FOLDER_IF_NOT_EXIST
 
 gchar *folder_get_path(Folder *folder)
@@ -723,7 +848,7 @@ gint folder_item_scan(FolderItem *item)
 	g_return_val_if_fail(item != NULL, -1);
 
 	folder = item->folder;
-	return folder->scan(folder, item);
+	return folder->klass->scan(folder, item);
 }
 
 static void folder_item_scan_foreach_func(gpointer key, gpointer val,
@@ -744,7 +869,7 @@ GSList *folder_item_get_msg_list(FolderItem *item, gboolean use_cache)
 	g_return_val_if_fail(item != NULL, NULL);
 
 	folder = item->folder;
-	return folder->get_msg_list(folder, item, use_cache);
+	return folder->klass->get_msg_list(folder, item, use_cache);
 }
 
 gchar *folder_item_fetch_msg(FolderItem *item, gint num)
@@ -754,125 +879,138 @@ gchar *folder_item_fetch_msg(FolderItem *item, gint num)
 	g_return_val_if_fail(item != NULL, NULL);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	return folder->fetch_msg(folder, item, num);
+	return folder->klass->fetch_msg(folder, item, num);
 }
 
-gint folder_item_add_msg(FolderItem *dest, const gchar *file,
+MsgInfo *folder_item_get_msginfo(FolderItem *item, gint num)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(item != NULL, NULL);
+
+	folder = item->folder;
+
+	return folder->klass->get_msginfo(folder, item, num);
+}
+
+gint folder_item_add_msg(FolderItem *dest, const gchar *file, MsgFlags *flags,
 			 gboolean remove_source)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(file != NULL, -1);
-	g_return_val_if_fail(dest->folder->add_msg != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->add_msg(folder, dest, file, remove_source);
-	if (num > 0) dest->last_num = num;
+	return folder->klass->add_msg(folder, dest, file, flags, remove_source);
+}
 
-	return num;
+gint folder_item_add_msgs(FolderItem *dest, GSList *file_list,
+			  gboolean remove_source, gint *first)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(file_list != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msgs != NULL, -1);
+
+	folder = dest->folder;
+
+	return folder->klass->add_msgs(folder, dest, file_list, remove_source,
+				       first);
 }
 
 gint folder_item_move_msg(FolderItem *dest, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->move_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->move_msg(folder, dest, msginfo);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->move_msg(folder, dest, msginfo);
 }
 
-gint folder_item_move_msgs_with_dest(FolderItem *dest, GSList *msglist)
+gint folder_item_move_msgs(FolderItem *dest, GSList *msglist)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->move_msgs != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->move_msgs_with_dest(folder, dest, msglist);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->move_msgs(folder, dest, msglist);
 }
 
 gint folder_item_copy_msg(FolderItem *dest, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->copy_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->copy_msg(folder, dest, msginfo);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->copy_msg(folder, dest, msginfo);
 }
 
-gint folder_item_copy_msgs_with_dest(FolderItem *dest, GSList *msglist)
+gint folder_item_copy_msgs(FolderItem *dest, GSList *msglist)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->copy_msgs != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->copy_msgs_with_dest(folder, dest, msglist);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->copy_msgs(folder, dest, msglist);
 }
 
-gint folder_item_remove_msg(FolderItem *item, gint num)
+gint folder_item_remove_msg(FolderItem *item, MsgInfo *msginfo)
 {
 	Folder *folder;
 	gint ret;
 
 	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder->klass->remove_msg != NULL, -1);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	ret = folder->remove_msg(folder, item, num);
-	if (ret == 0 && num == item->last_num)
-		folder->scan(folder, item);
+	ret = folder->klass->remove_msg(folder, item, msginfo);
+	if (ret == 0 && msginfo->msgnum == item->last_num)
+		folder->klass->scan(folder, item);
 
 	return ret;
 }
 
 gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
 {
+	Folder *folder;
 	gint ret = 0;
 
 	g_return_val_if_fail(item != NULL, -1);
 
+	folder = item->folder;
+	if (folder->klass->remove_msgs) {
+		ret = folder->klass->remove_msgs(folder, item, msglist);
+		if (ret == 0)
+			folder->klass->scan(folder, item);
+		return ret;
+	}
+
 	while (msglist != NULL) {
 		MsgInfo *msginfo = (MsgInfo *)msglist->data;
 
-		ret = folder_item_remove_msg(item, msginfo->msgnum);
+		ret = folder_item_remove_msg(item, msginfo);
 		if (ret != 0) break;
 		msglist = msglist->next;
 	}
@@ -885,11 +1023,11 @@ gint folder_item_remove_all_msg(FolderItem *item)
 	Folder *folder;
 
 	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder->klass->remove_all_msg != NULL, -1);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	return folder->remove_all_msg(folder, item);
+	return folder->klass->remove_all_msg(folder, item);
 }
 
 gboolean folder_item_is_msg_changed(FolderItem *item, MsgInfo *msginfo)
@@ -897,9 +1035,22 @@ gboolean folder_item_is_msg_changed(FolderItem *item, MsgInfo *msginfo)
 	Folder *folder;
 
 	g_return_val_if_fail(item != NULL, FALSE);
+	g_return_val_if_fail(item->folder->klass->is_msg_changed != NULL,
+			     FALSE);
 
 	folder = item->folder;
-	return folder->is_msg_changed(folder, item, msginfo);
+	return folder->klass->is_msg_changed(folder, item, msginfo);
+}
+
+gint folder_item_close(FolderItem *item)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	item->opened = FALSE;
+	folder = item->folder;
+	return folder->klass->close(folder, item);
 }
 
 gchar *folder_item_get_cache_file(FolderItem *item)
@@ -949,11 +1100,15 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
 	gboolean no_sub = FALSE, no_select = FALSE, collapsed = FALSE,
-		 threaded = TRUE;
+		 threaded = TRUE, ac_apply_sub = FALSE;
 	FolderSortKey sort_key = SORT_BY_NONE;
 	FolderSortType sort_type = SORT_ASCENDING;
 	gint new = 0, unread = 0, total = 0;
 	time_t mtime = 0;
+	gboolean use_auto_to_on_reply = FALSE;
+	gchar *auto_to = NULL, *auto_cc = NULL, *auto_bcc = NULL,
+	      *auto_replyto = NULL;
+	gboolean trim_summary_subject = FALSE, trim_compose_subject = FALSE;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
 	if (!node->parent) return FALSE;
@@ -986,11 +1141,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 			name = attr->value;
 		else if (!strcmp(attr->name, "path"))
 			path = attr->value;
-		else if (!strcmp(attr->name, "account_id")) {
-			account = account_find_from_id(atoi(attr->value));
-			if (!account) g_warning("account_id: %s not found\n",
-						attr->value);
-		} else if (!strcmp(attr->name, "mtime"))
+		else if (!strcmp(attr->name, "mtime"))
 			mtime = strtoul(attr->value, NULL, 10);
 		else if (!strcmp(attr->name, "new"))
 			new = atoi(attr->value);
@@ -1029,17 +1180,41 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				sort_key = SORT_BY_UNREAD;
 			else if (!strcmp(attr->value, "mime"))
 				sort_key = SORT_BY_MIME;
+			else if (!strcmp(attr->value, "to"))
+				sort_key = SORT_BY_TO;
 		} else if (!strcmp(attr->name, "sort_type")) {
 			if (!strcmp(attr->value, "ascending"))
 				sort_type = SORT_ASCENDING;
 			else
 				sort_type = SORT_DESCENDING;
+		} else if (!strcmp(attr->name, "account_id")) {
+			account = account_find_from_id(atoi(attr->value));
+			if (!account) g_warning("account_id: %s not found\n",
+						attr->value);
+		} else if (!strcmp(attr->name, "account_apply_sub"))
+			ac_apply_sub = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "to"))
+			auto_to = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "use_auto_to_on_reply"))
+			use_auto_to_on_reply =
+				*attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "cc"))
+			auto_cc = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "bcc"))
+			auto_bcc = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "replyto"))
+			auto_replyto = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "trim_summary_subject")) {
+			trim_summary_subject =
+				*attr->value == '1' ? TRUE : FALSE;
+		} else if (!strcmp(attr->name, "trim_compose_subject")) {
+			trim_compose_subject =
+				*attr->value = '1' ? TRUE : FALSE;
 		}
 	}
 
 	item = folder_item_new(name, path);
 	item->stype = stype;
-	item->account = account;
 	item->mtime = mtime;
 	item->new = new;
 	item->unread = unread;
@@ -1050,6 +1225,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	item->threaded  = threaded;
 	item->sort_key  = sort_key;
 	item->sort_type = sort_type;
+	item->node = node;
 	item->parent = FOLDER_ITEM(node->parent->data);
 	item->folder = folder;
 	switch (stype) {
@@ -1060,6 +1236,15 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	case F_TRASH:  folder->trash  = item; break;
 	default:       break;
 	}
+	item->account = account;
+	item->ac_apply_sub = ac_apply_sub;
+	item->auto_to = auto_to;
+	item->use_auto_to_on_reply = use_auto_to_on_reply;
+	item->auto_cc = auto_cc;
+	item->auto_bcc = auto_bcc;
+	item->auto_replyto = auto_replyto;
+	item->trim_summary_subject = trim_summary_subject;
+	item->trim_compose_subject = trim_compose_subject;
 	node->data = item;
 	xml_free_node(xmlnode);
 
@@ -1069,13 +1254,14 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 static gboolean folder_read_folder_func(GNode *node, gpointer data)
 {
 	Folder *folder;
+	FolderItem *item;
 	XMLNode *xmlnode;
 	GList *list;
 	FolderType type = F_UNKNOWN;
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
-	gboolean collapsed = FALSE, threaded = TRUE;
+	gboolean collapsed = FALSE, threaded = TRUE, ac_apply_sub = FALSE;
 
 	if (g_node_depth(node) != 2) return FALSE;
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -1106,14 +1292,16 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 			name = attr->value;
 		else if (!strcmp(attr->name, "path"))
 			path = attr->value;
+		else if (!strcmp(attr->name, "collapsed"))
+			collapsed = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "threaded"))
+			threaded = *attr->value == '1' ? TRUE : FALSE;
 		else if (!strcmp(attr->name, "account_id")) {
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
 						attr->value);
-		} else if (!strcmp(attr->name, "collapsed"))
-			collapsed = *attr->value == '1' ? TRUE : FALSE;
-		else if (!strcmp(attr->name, "threaded"))
-			threaded = *attr->value == '1' ? TRUE : FALSE;
+		} else if (!strcmp(attr->name, "account_apply_sub"))
+			ac_apply_sub = *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	folder = folder_new(type, name, path);
@@ -1121,12 +1309,16 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	folder->account = account;
 	if (account && (type == F_IMAP || type == F_NEWS))
 		account->folder = REMOTE_FOLDER(folder);
-	node->data = folder->node->data;
+	item = FOLDER_ITEM(folder->node->data);
+	node->data = item;
+	item->node = node;
 	g_node_destroy(folder->node);
 	folder->node = node;
 	folder_add(folder);
-	FOLDER_ITEM(node->data)->collapsed = collapsed;
-	FOLDER_ITEM(node->data)->threaded  = threaded;
+	item->collapsed = collapsed;
+	item->threaded  = threaded;
+	item->account   = account;
+	item->ac_apply_sub = ac_apply_sub;
 
 	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			folder_build_tree, folder);
@@ -1145,6 +1337,13 @@ static gchar *folder_get_list_path(void)
 	return filename;
 }
 
+#define PUT_ESCAPE_STR(fp, attr, str)			\
+{							\
+	fputs(" " attr "=\"", fp);			\
+	xml_file_put_escape_str(fp, str);		\
+	fputs("\"", fp);				\
+}
+
 static void folder_write_list_recursive(GNode *node, gpointer data)
 {
 	FILE *fp = (FILE *)data;
@@ -1156,7 +1355,7 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 						 "draft", "queue", "trash"};
 	static gchar *sort_key_str[] = {"none", "number", "size", "date",
 					"from", "subject", "score", "label",
-					"mark", "unread", "mime"};
+					"mark", "unread", "mime", "to"};
 
 	g_return_if_fail(node != NULL);
 	g_return_if_fail(fp != NULL);
@@ -1170,39 +1369,28 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 	if (depth == 1) {
 		Folder *folder = item->folder;
 
-		fprintf(fp, "<folder type=\"%s\"", folder_type_str[folder->type]);
-		if (folder->name) {
-			fputs(" name=\"", fp);
-			xml_file_put_escape_str(fp, folder->name);
-			fputs("\"", fp);
-		}
-		if (folder->type == F_MH) {
-			fputs(" path=\"", fp);
-			xml_file_put_escape_str
-				(fp, LOCAL_FOLDER(folder)->rootpath);
-			fputs("\"", fp);
-		}
+		fprintf(fp, "<folder type=\"%s\"",
+			folder_type_str[FOLDER_TYPE(folder)]);
+		if (folder->name)
+			PUT_ESCAPE_STR(fp, "name", folder->name);
+		if (FOLDER_TYPE(folder) == F_MH)
+			PUT_ESCAPE_STR(fp, "path",
+				       LOCAL_FOLDER(folder)->rootpath);
+		if (item->collapsed && node->children)
+			fputs(" collapsed=\"1\"", fp);
 		if (folder->account)
 			fprintf(fp, " account_id=\"%d\"",
 				folder->account->account_id);
-		if (item->collapsed && node->children)
-			fputs(" collapsed=\"1\"", fp);
+		if (item->ac_apply_sub)
+			fputs(" account_apply_sub=\"1\"", fp);
 	} else {
 		fprintf(fp, "<folderitem type=\"%s\"",
 			folder_item_stype_str[item->stype]);
-		if (item->name) {
-			fputs(" name=\"", fp);
-			xml_file_put_escape_str(fp, item->name);
-			fputs("\"", fp);
-		}
-		if (item->path) {
-			fputs(" path=\"", fp);
-			xml_file_put_escape_str(fp, item->path);
-			fputs("\"", fp);
-		}
-		if (item->account)
-			fprintf(fp, " account_id=\"%d\"",
-				item->account->account_id);
+		if (item->name)
+			PUT_ESCAPE_STR(fp, "name", item->name);
+		if (item->path)
+			PUT_ESCAPE_STR(fp, "path", item->path);
+
 		if (item->no_sub)
 			fputs(" no_sub=\"1\"", fp);
 		if (item->no_select)
@@ -1226,6 +1414,28 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 		fprintf(fp,
 			" mtime=\"%lu\" new=\"%d\" unread=\"%d\" total=\"%d\"",
 			item->mtime, item->new, item->unread, item->total);
+
+		if (item->account)
+			fprintf(fp, " account_id=\"%d\"",
+				item->account->account_id);
+		if (item->ac_apply_sub)
+			fputs(" account_apply_sub=\"1\"", fp);
+
+		if (item->auto_to)
+			PUT_ESCAPE_STR(fp, "to", item->auto_to);
+		if (item->use_auto_to_on_reply)
+			fputs(" use_auto_to_on_reply=\"1\"", fp);
+		if (item->auto_cc)
+			PUT_ESCAPE_STR(fp, "cc", item->auto_cc);
+		if (item->auto_bcc)
+			PUT_ESCAPE_STR(fp, "bcc", item->auto_bcc);
+		if (item->auto_replyto)
+			PUT_ESCAPE_STR(fp, "replyto", item->auto_replyto);
+
+		if (item->trim_summary_subject)
+			fputs(" trim_summary_subject=\"1\"", fp);
+		if (item->trim_compose_subject)
+			fputs(" trim_compose_subject=\"1\"", fp);
 	}
 
 	if (node->children) {
@@ -1247,3 +1457,5 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 	} else
 		fputs(" />\n", fp);
 }
+
+#undef PUT_ESCAPE_STR
