@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,6 +62,7 @@ static GtkWidget *cancel_button;
 static FolderItem *folder_item;
 
 static gboolean cancelled;
+static gboolean finished;
 
 static void foldersel_create	(void);
 static void foldersel_init	(void);
@@ -122,9 +123,10 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder,
 	gtk_widget_grab_focus(ok_button);
 	gtk_widget_grab_focus(ctree);
 
-	cancelled = FALSE;
+	cancelled = finished = FALSE;
 
-	gtk_main();
+	while (finished == FALSE)
+		gtk_main_iteration();
 
 	gtk_widget_hide(window);
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
@@ -238,7 +240,7 @@ static gboolean foldersel_gnode_func(GtkCTree *ctree, guint depth,
 		name = item->name;
 
 		if (!item->parent) {
-			switch (item->folder->type) {
+			switch (FOLDER_TYPE(item->folder)) {
 			case F_MH:
 				Xstrcat_a(name, name, " (MH)", ); break;
 			case F_IMAP:
@@ -309,7 +311,7 @@ static void foldersel_set_tree(Folder *cur_folder, FolderSelectionType type)
 		g_return_if_fail(folder != NULL);
 
 		if (type != FOLDER_SEL_ALL) {
-			if (folder->type == F_NEWS)
+			if (FOLDER_TYPE(folder) == F_NEWS)
 				continue;
 		}
 
@@ -354,13 +356,13 @@ static void foldersel_ok(GtkButton *button, gpointer data)
 		folder_item = gtk_ctree_node_get_row_data
 			(GTK_CTREE(ctree), GTK_CTREE_NODE(list->data));
 
-	gtk_main_quit();
+	finished = TRUE;
 }
 
 static void foldersel_cancel(GtkButton *button, gpointer data)
 {
 	cancelled = TRUE;
-	gtk_main_quit();
+	finished = TRUE;
 }
 
 static void foldersel_activated(void)
