@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,9 +87,6 @@ typedef enum
 
 static GList *folderview_list = NULL;
 
-static GdkFont *normalfont;
-static GdkFont *boldfont;
-
 static GtkStyle *bold_style;
 static GtkStyle *bold_color_style;
 
@@ -221,12 +218,12 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
-	{N_("/R_escan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
+	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 static GtkItemFactoryEntry folderview_imap_popup_entries[] =
@@ -237,12 +234,12 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
-	{N_("/R_escan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
+	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 static GtkItemFactoryEntry folderview_news_popup_entries[] =
@@ -257,7 +254,7 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 	{N_("/Remove _news account"),	NULL, folderview_rm_news_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
+	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 
@@ -407,8 +404,10 @@ FolderView *folderview_create(void)
 
 void folderview_init(FolderView *folderview)
 {
+	static GdkFont *boldfont = NULL;
 	GtkWidget *ctree = folderview->ctree;
 
+	gtk_widget_realize(ctree);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_INBOX, &inboxxpm, &inboxxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_OUTBOX,
 			 &outboxxpm, &outboxxpmmask);
@@ -418,14 +417,12 @@ void folderview_init(FolderView *folderview)
 			 &folderopenxpm, &folderopenxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_TRASH, &trashxpm, &trashxpmmask);
 
-	if (!normalfont)
-		normalfont = gdk_fontset_load(prefs_common.normalfont);
-	if (!boldfont)
-		boldfont = gdk_fontset_load(prefs_common.boldfont);
-
 	if (!bold_style) {
 		bold_style = gtk_style_copy(gtk_widget_get_style(ctree));
-		bold_style->font = boldfont;
+		if (!boldfont)
+			boldfont = gtkut_font_load(prefs_common.boldfont);
+		if (boldfont)
+			bold_style->font = boldfont;
 		bold_color_style = gtk_style_copy(bold_style);
 		bold_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
 	}
@@ -484,7 +481,8 @@ static void folderview_select_node(FolderView *folderview, GtkCTreeNode *node)
 	folderview->open_folder = TRUE;
 	gtkut_ctree_set_focus_row(ctree, node);
 	gtk_ctree_select(ctree, node);
-	if (folderview->summaryview->messages > 0)
+	if (folderview->summaryview->folder_item &&
+	    folderview->summaryview->folder_item->total > 0)
 		gtk_widget_grab_focus(folderview->summaryview->ctree);
 	else
 		gtk_widget_grab_focus(folderview->ctree);
@@ -588,10 +586,10 @@ static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
 
 	if (FOLDER_IS_LOCAL(folder))
 		rootpath = LOCAL_FOLDER(folder)->rootpath;
-	else if (folder->type == F_IMAP && folder->account &&
+	else if (FOLDER_TYPE(folder) == F_IMAP && folder->account &&
 		 folder->account->recv_server)
 		rootpath = folder->account->recv_server;
-	else if (folder->type == F_NEWS && folder->account &&
+	else if (FOLDER_TYPE(folder) == F_NEWS && folder->account &&
 		 folder->account->nntp_server)
 		rootpath = folder->account->nntp_server;
 	else
@@ -646,20 +644,20 @@ void folderview_rescan_tree(Folder *folder)
 
 	g_return_if_fail(folder != NULL);
 
-	if (!folder->scan_tree) return;
+	if (!folder->klass->scan_tree) return;
 
 	avalue = alertpanel
-		(_("Rescan folder tree"),
-		 _("All previous settings for each folders will be lost.\n"
-		   "Continue?"),
+		(_("Rebuild folder tree"),
+		 _("The folder tree will be rebuilt. Continue?"),
 		 _("Yes"), _("No"), NULL);
 	if (avalue != G_ALERTDEFAULT) return;
 
 	inc_lock();
-	window = label_window_create(_("Rescanning folder tree..."));
+	window = label_window_create(_("Rebuilding folder tree..."));
 
 	folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
-	folder->scan_tree(folder);
+	if (folder->klass->scan_tree(folder) < 0)
+		alertpanel_error(_("Rebuilding of the folder tree failed."));
 	folder_set_ui_func(folder, NULL, NULL);
 
 	folder_write_list();
@@ -676,15 +674,15 @@ void folderview_rescan_all(void)
 	GtkWidget *window;
 
 	inc_lock();
-	window = label_window_create(_("Rescanning all folder trees..."));
+	window = label_window_create(_("Rebuilding all folder trees..."));
 
 	list = folder_get_list();
 	for (; list != NULL; list = list->next) {
 		Folder *folder = list->data;
 
-		if (!folder->scan_tree) continue;
+		if (!folder->klass->scan_tree) continue;
 		folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
-		folder->scan_tree(folder);
+		folder->klass->scan_tree(folder);
 		folder_set_ui_func(folder, NULL, NULL);
 	}
 
@@ -911,7 +909,7 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 		openxpm = folderopenxpm;
 		openmask = folderopenxpmmask;
 		if (!item->parent) {
-			switch (item->folder->type) {
+			switch (FOLDER_TYPE(item->folder)) {
 			case F_MH:
 				name = " (MH)"; break;
 			case F_IMAP:
@@ -923,11 +921,12 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 			}
 			name = g_strconcat(item->name, name, NULL);
 		} else {
-			if (item->folder->type == F_NEWS &&
+			if (FOLDER_TYPE(item->folder) == F_NEWS &&
 			    item->path &&
-			    !strcmp2(item->name, item->path) &&
-			    prefs_common.ng_abbrev_len < strlen(item->path))
-				name = get_abbrev_newsgroup_name(item->path);
+			    !strcmp2(item->name, item->path))
+				name = get_abbrev_newsgroup_name
+					(item->path,
+					 prefs_common.ng_abbrev_len);
 			else
 				name = g_strdup(item->name);
 		}
@@ -1088,6 +1087,16 @@ static void folderview_expand_func(GtkCTree *ctree, GtkCTreeNode *node,
 				sibling = GTK_CTREE_ROW(prev)->sibling; \
 			else \
 				sibling = GTK_CTREE_ROW(parent)->children; \
+			while (sibling) { \
+				FolderItem *tmp; \
+ \
+				tmp = gtk_ctree_node_get_row_data \
+					(ctree, sibling); \
+				if (tmp->stype != F_NORMAL) \
+					sibling = GTK_CTREE_ROW(sibling)->sibling; \
+				else \
+					break; \
+			} \
 			if (node != sibling) \
 				gtk_ctree_move(ctree, node, parent, sibling); \
 		} \
@@ -1138,7 +1147,7 @@ void folderview_new_folder(FolderView *folderview)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	switch (item->folder->type) {
+	switch (FOLDER_TYPE(item->folder)) {
 	case F_MH:
 	case F_MBOX:
 	case F_MAILDIR:
@@ -1166,7 +1175,7 @@ void folderview_rename_folder(FolderView *folderview)
 	if (!item->path) return;
 	if (item->stype != F_NORMAL) return;
 
-	switch (item->folder->type) {
+	switch (FOLDER_TYPE(item->folder)) {
 	case F_MH:
 	case F_MBOX:
 	case F_MAILDIR:
@@ -1192,7 +1201,7 @@ void folderview_delete_folder(FolderView *folderview)
 	if (!item->path) return;
 	if (item->stype != F_NORMAL) return;
 
-	switch (item->folder->type) {
+	switch (FOLDER_TYPE(item->folder)) {
 	case F_MH:
 	case F_MBOX:
 	case F_MAILDIR:
@@ -1267,9 +1276,11 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 
 	if (folderview->mainwin->lock_count == 0) {
 		new_folder = TRUE;
-		if (item->parent == NULL)
+		if (item->parent == NULL) {
 			update_tree = remove_tree = TRUE;
-		else
+			if (folder->account)
+				folder_property = TRUE;
+		} else
 			search_folder = folder_property = TRUE;
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
@@ -1292,10 +1303,10 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(mail_factory, "/Rename folder...", rename_folder);
 		SET_SENS(mail_factory, "/Delete folder", delete_folder);
 		SET_SENS(mail_factory, "/Check for new messages", update_tree);
-		SET_SENS(mail_factory, "/Rescan folder tree", rescan_tree);
+		SET_SENS(mail_factory, "/Rebuild folder tree", rescan_tree);
 		SET_SENS(mail_factory, "/Remove mailbox", remove_tree);
 		SET_SENS(mail_factory, "/Search messages...", search_folder);
-		SET_SENS(mail_factory, "/Property...", folder_property);
+		SET_SENS(mail_factory, "/Properties...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
 		popup = folderview->imap_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
@@ -1303,10 +1314,10 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
 		SET_SENS(imap_factory, "/Delete folder", delete_folder);
 		SET_SENS(imap_factory, "/Check for new messages", update_tree);
-		SET_SENS(imap_factory, "/Rescan folder tree", rescan_tree);
+		SET_SENS(imap_factory, "/Rebuild folder tree", rescan_tree);
 		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
 		SET_SENS(imap_factory, "/Search messages...", search_folder);
-		SET_SENS(imap_factory, "/Property...", folder_property);
+		SET_SENS(imap_factory, "/Properties...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
 		popup = folderview->news_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
@@ -1315,7 +1326,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(news_factory, "/Check for new messages", update_tree);
 		SET_SENS(news_factory, "/Remove news account", remove_tree);
 		SET_SENS(news_factory, "/Search messages...", search_folder);
-		SET_SENS(news_factory, "/Property...", folder_property);
+		SET_SENS(news_factory, "/Properties...", folder_property);
 	} else
 		return;
 
@@ -1353,7 +1364,8 @@ static void folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_space:
 		if (folderview->selected) {
 			if (folderview->opened == folderview->selected &&
-			    folderview->summaryview->messages == 0)
+			    (!folderview->summaryview->folder_item ||
+			     folderview->summaryview->folder_item->total == 0))
 				folderview_select_next_unread(folderview);
 			else
 				folderview_select_node(folderview,
@@ -1560,7 +1572,8 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 		return;
 	}
 
-	new_item = item->folder->create_folder(item->folder, item, new_folder);
+	new_item = item->folder->klass->create_folder(item->folder, item,
+						      new_folder);
 	if (!new_item) {
 		alertpanel_error(_("Can't create the folder `%s'."), name);
 		g_free(new_folder);
@@ -1633,7 +1646,8 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	Xstrdup_a(old_path, item->path, {g_free(new_folder); return;});
 	old_id = folder_item_get_identifier(item);
 
-	if (item->folder->rename_folder(item->folder, item, new_folder) < 0) {
+	if (item->folder->klass->rename_folder(item->folder, item,
+					       new_folder) < 0) {
 		g_free(old_id);
 		g_free(new_folder);
 		return;
@@ -1671,6 +1685,7 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 					GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	Folder *folder;
 	FolderItem *item;
 	gchar *message, *name, *name_;
 	AlertValue avalue;
@@ -1684,6 +1699,8 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	folder = item->folder;
+
 	name_ = trim_string(item->name, 32);
 	Xstrdup_a(name, name_, return);
 	g_free(name_);
@@ -1698,29 +1715,24 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	Xstrdup_a(old_path, item->path, return);
 	old_id = folder_item_get_identifier(item);
 
-	if (item->folder->remove_folder(item->folder, item) < 0) {
+	if (folderview->opened == folderview->selected ||
+	    gtk_ctree_is_ancestor(ctree,
+				  folderview->selected, folderview->opened)) {
+		summary_clear_all(folderview->summaryview);
+		folderview->opened = NULL;
+	}
+
+	if (folder->klass->remove_folder(folder, item) < 0) {
 		alertpanel_error(_("Can't remove the folder `%s'."), name);
-		if (folderview->opened == folderview->selected)
-			summary_show(folderview->summaryview,
-				     folderview->summaryview->folder_item,
-				     FALSE);
 		g_free(old_id);
 		return;
 	}
 
-	if (folder_get_default_folder() == item->folder)
+	if (folder_get_default_folder() == folder)
 		prefs_filter_delete_path(old_path);
 	prefs_filter_delete_path(old_id);
 	g_free(old_id);
 
-	if (folderview->opened == folderview->selected ||
-	    gtk_ctree_is_ancestor(ctree,
-				  folderview->selected,
-				  folderview->opened)) {
-		summary_clear_all(folderview->summaryview);
-		folderview->opened = NULL;
-	}
-
 	gtk_ctree_remove_node(ctree, folderview->selected);
 	folder_write_list();
 }
@@ -1753,9 +1765,12 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	g_free(message);
 	if (avalue != G_ALERTDEFAULT) return;
 
+	if (folderview->summaryview->folder_item &&
+	    folderview->summaryview->folder_item->folder == item->folder) {
+		summary_clear_all(folderview->summaryview);
+		folderview->opened = NULL;
+	}
 	folder_destroy(item->folder);
-	summary_clear_all(folderview->summaryview);
-	folderview_unselect(folderview);
 	gtk_ctree_remove_node(ctree, node);
 	folder_write_list();
 }
@@ -1777,7 +1792,7 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_IMAP);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
 	new_folder = input_dialog
@@ -1807,7 +1822,8 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 		return;
 	}
 
-	new_item = item->folder->create_folder(item->folder, item, new_folder);
+	new_item = item->folder->klass->create_folder(item->folder, item,
+						      new_folder);
 	if (!new_item) {
 		alertpanel_error(_("Can't create the folder `%s'."), name);
 		g_free(new_folder);
@@ -1837,6 +1853,7 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
+	PrefsAccount *account;
 	gchar *name, *name_;
 	gchar *message;
 	AlertValue avalue;
@@ -1846,7 +1863,7 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_IMAP);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
 	name_ = trim_string(item->folder->name, 32);
@@ -1859,16 +1876,15 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 
 	if (avalue != G_ALERTDEFAULT) return;
 
-	if (folderview->opened == folderview->selected ||
-	    gtk_ctree_is_ancestor(ctree,
-				  folderview->selected,
-				  folderview->opened)) {
+	if (folderview->summaryview->folder_item &&
+	    folderview->summaryview->folder_item->folder == item->folder) {
 		summary_clear_all(folderview->summaryview);
 		folderview->opened = NULL;
 	}
 
-	account_destroy(item->folder->account);
+	account = item->folder->account;
 	folder_destroy(item->folder);
+	account_destroy(account);
 	gtk_ctree_remove_node(ctree, folderview->selected);
 	account_set_menu();
 	main_window_reflect_prefs_all();
@@ -1895,7 +1911,7 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item != NULL);
 	folder = item->folder;
 	g_return_if_fail(folder != NULL);
-	g_return_if_fail(folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(folder) == F_NEWS);
 	g_return_if_fail(folder->account != NULL);
 
 	if (GTK_CTREE_ROW(folderview->selected)->parent != NULL)
@@ -1980,7 +1996,7 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
 	name_ = trim_string(item->path, 32);
@@ -2007,6 +2023,7 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
+	PrefsAccount *account;
 	gchar *name, *name_;
 	gchar *message;
 	AlertValue avalue;
@@ -2016,7 +2033,7 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
 	name_ = trim_string(item->folder->name, 32);
@@ -2029,16 +2046,15 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 
 	if (avalue != G_ALERTDEFAULT) return;
 
-	if (folderview->opened == folderview->selected ||
-	    gtk_ctree_is_ancestor(ctree,
-				  folderview->selected,
-				  folderview->opened)) {
+	if (folderview->summaryview->folder_item &&
+	    folderview->summaryview->folder_item->folder == item->folder) {
 		summary_clear_all(folderview->summaryview);
 		folderview->opened = NULL;
 	}
 
-	account_destroy(item->folder->account);
+	account = item->folder->account;
 	folder_destroy(item->folder);
+	account_destroy(account);
 	gtk_ctree_remove_node(ctree, folderview->selected);
 	account_set_menu();
 	main_window_reflect_prefs_all();
@@ -2063,7 +2079,10 @@ static void folderview_property_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	prefs_folder_item_open(item);
+	if (item->parent == NULL && item->folder->account)
+		account_open(item->folder->account);
+	else
+		prefs_folder_item_open(item);
 }
 
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
@@ -2085,7 +2104,7 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		src_item = folderview->summaryview->folder_item;
 		if (item && item->folder && item->path &&
 		    src_item && src_item != item) {
-			switch (item->folder->type) {
+			switch (FOLDER_TYPE(item->folder)) {
 			case F_MH:
 			case F_IMAP:
 				acceptable = TRUE;
@@ -2142,8 +2161,9 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 	src_item = folderview->summaryview->folder_item;
 	if (item && src_item) {
-		if (src_item->folder->type != item->folder->type ||
-		    (item->folder->type == F_IMAP &&
+		if (FOLDER_TYPE(src_item->folder) !=
+		    FOLDER_TYPE(item->folder) ||
+		    (FOLDER_TYPE(item->folder) == F_IMAP &&
 		     src_item->folder != item->folder))
 			summary_copy_selected_to(folderview->summaryview, item);
 		else
