@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,6 +72,9 @@
 gint gtkut_get_font_width		(GdkFont	*font);
 gint gtkut_get_font_height		(GdkFont	*font);
 
+GdkFont *gtkut_font_load		(const gchar	*fontset_name);
+GdkFont *gtkut_font_load_from_fontset	(const gchar	*fontset_name);
+
 void gtkut_convert_int_to_gdk_color	(gint		 rgbvalue,
 					 GdkColor	*color);
 
@@ -90,6 +93,10 @@ gint gtkut_ctree_get_nth_from_node	(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
 GtkCTreeNode *gtkut_ctree_node_next	(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
+GtkCTreeNode *gtkut_ctree_node_prev	(GtkCTree	*ctree,
+					 GtkCTreeNode	*node);
+gboolean gtkut_ctree_node_is_selected	(GtkCTree	*ctree,
+					 GtkCTreeNode	*node);
 GtkCTreeNode *gtkut_ctree_find_collapsed_parent
 					(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
@@ -123,16 +130,22 @@ guint gtkut_stext_str_compare		(GtkSText	*text,
 					 guint		 start_pos,
 					 guint		 text_len,
 					 const gchar	*str);
+gint gtkut_stext_find			(GtkSText	*text,
+					 guint		 start_pos,
+					 const gchar	*str,
+					 gboolean	 case_sens);
 gboolean gtkut_stext_is_uri_string	(GtkSText	*text,
 					 guint		 start_pos,
 					 guint		 text_len);
-void gtk_stext_clear			(GtkSText	*text);
+void gtkut_stext_clear			(GtkSText	*text);
+
+void gtkut_window_popup			(GtkWidget	*window);
 
-void gtkut_widget_disable_theme_engine	(GtkWidget	*widget);
-void gtkut_widget_wait_for_draw		(GtkWidget	*widget);
 void gtkut_widget_get_uposition		(GtkWidget	*widget,
 					 gint		*px,
 					 gint		*py);
+void gtkut_widget_disable_theme_engine	(GtkWidget	*widget);
+void gtkut_widget_wait_for_draw		(GtkWidget	*widget);
 void gtkut_widget_init			(void);
 
 #endif /* __GTKUTILS_H__ */
