@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,7 +62,6 @@
 #include "inputdialog.h"
 #include "alertpanel.h"
 #include "filter.h"
-#include "automaton.h"
 #include "folder.h"
 
 static GList *inc_dialog_list = NULL;
@@ -83,22 +82,33 @@ static void inc_finished		(MainWindow		*mainwin,
 static gint inc_account_mail		(PrefsAccount		*account,
 					 MainWindow		*mainwin);
 
-static IncProgressDialog *inc_progress_dialog_create	(void);
+static IncProgressDialog *inc_progress_dialog_create
+					(gboolean		 autocheck);
+static void inc_progress_dialog_set_list(IncProgressDialog	*inc_dialog);
 static void inc_progress_dialog_destroy	(IncProgressDialog	*inc_dialog);
 
 static IncSession *inc_session_new	(PrefsAccount		*account);
 static void inc_session_destroy		(IncSession		*session);
 static gint inc_start			(IncProgressDialog	*inc_dialog);
 static IncState inc_pop3_session_do	(IncSession		*session);
-static gint pop3_automaton_terminate	(SockInfo		*source,
-					 Automaton		*atm);
 
-static gboolean inc_pop3_recv_func	(SockInfo	*sock,
-					 gint		 count,
-					 gint		 read_bytes,
+static void inc_progress_dialog_set_label
+					(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
+
+static gint inc_recv_data_progressive	(Session	*session,
+					 guint		 cur_len,
+					 guint		 total_len,
+					 gpointer	 data);
+static gint inc_recv_data_finished	(Session	*session,
+					 guint		 len,
+					 gpointer	 data);
+static gint inc_recv_message		(Session	*session,
+					 const gchar	*msg,
 					 gpointer	 data);
 
-static void inc_put_error		(IncState	 istate);
+static void inc_put_error		(IncState	 istate,
+					 const gchar	*msg);
 
 static void inc_cancel_cb		(GtkWidget	*widget,
 					 gpointer	 data);
@@ -185,7 +195,6 @@ static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
-	gchar *text[3];
 
 	if (account->protocol == A_IMAP4 || account->protocol == A_NNTP) {
 		FolderItem *item = mainwin->summaryview->folder_item;
@@ -200,15 +209,10 @@ static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 	session = inc_session_new(account);
 	if (!session) return 0;
 
-	inc_dialog = inc_progress_dialog_create();
+	inc_dialog = inc_progress_dialog_create(FALSE);
 	inc_dialog->queue_list = g_list_append(inc_dialog->queue_list, session);
 	inc_dialog->mainwin = mainwin;
-	session->data = inc_dialog;
-
-	text[0] = NULL;
-	text[1] = account->account_name;
-	text[2] = _("Standby");
-	gtk_clist_append(GTK_CLIST(inc_dialog->dialog->clist), text);
+	inc_progress_dialog_set_list(inc_dialog);
 
 	main_window_set_toolbar_sensitive(mainwin);
 	main_window_set_menu_sensitive(mainwin);
@@ -216,7 +220,7 @@ static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 	return inc_start(inc_dialog);
 }
 
-void inc_all_account_mail(MainWindow *mainwin)
+void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 {
 	GList *list, *queue_list = NULL;
 	IncProgressDialog *inc_dialog;
@@ -275,20 +279,10 @@ void inc_all_account_mail(MainWindow *mainwin)
 		return;
 	}
 
-	inc_dialog = inc_progress_dialog_create();
+	inc_dialog = inc_progress_dialog_create(autocheck);
 	inc_dialog->queue_list = queue_list;
 	inc_dialog->mainwin = mainwin;
-	for (list = queue_list; list != NULL; list = list->next) {
-		IncSession *session = list->data;
-		gchar *text[3];
-
-		session->data = inc_dialog;
-
-		text[0] = NULL;
-		text[1] = session->pop3_state->ac_prefs->account_name;
-		text[2] = _("Standby");
-		gtk_clist_append(GTK_CLIST(inc_dialog->dialog->clist), text);
-	}
+	inc_progress_dialog_set_list(inc_dialog);
 
 	main_window_set_toolbar_sensitive(mainwin);
 	main_window_set_menu_sensitive(mainwin);
@@ -300,7 +294,7 @@ void inc_all_account_mail(MainWindow *mainwin)
 	inc_autocheck_timer_set();
 }
 
-static IncProgressDialog *inc_progress_dialog_create(void)
+static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 {
 	IncProgressDialog *dialog;
 	ProgressDialog *progress;
@@ -326,24 +320,45 @@ static IncProgressDialog *inc_progress_dialog_create(void)
 			 &errorxpm, &errorxpmmask);
 
 	if (prefs_common.recv_dialog_mode == RECV_DIALOG_ALWAYS ||
-	    (prefs_common.recv_dialog_mode == RECV_DIALOG_ACTIVE &&
-	     manage_window_get_focus_window())) {
+	    (prefs_common.recv_dialog_mode == RECV_DIALOG_MANUAL &&
+	     !autocheck)) {
 		dialog->show_dialog = TRUE;
 		gtk_widget_show_now(progress->window);
 	}
 
 	dialog->dialog = progress;
 	dialog->queue_list = NULL;
+	dialog->cur_row = 0;
 
 	inc_dialog_list = g_list_append(inc_dialog_list, dialog);
 
 	return dialog;
 }
 
+static void inc_progress_dialog_set_list(IncProgressDialog *inc_dialog)
+{
+	GList *list;
+
+	for (list = inc_dialog->queue_list; list != NULL; list = list->next) {
+		IncSession *session = list->data;
+		Pop3Session *pop3_session = POP3_SESSION(session->session);
+		gchar *text[3];
+
+		session->data = inc_dialog;
+
+		text[0] = NULL;
+		text[1] = pop3_session->ac_prefs->account_name;
+		text[2] = _("Standby");
+		gtk_clist_append(GTK_CLIST(inc_dialog->dialog->clist), text);
+	}
+}
+
 static void inc_progress_dialog_clear(IncProgressDialog *inc_dialog)
 {
 	progress_dialog_set_value(inc_dialog->dialog, 0.0);
 	progress_dialog_set_label(inc_dialog->dialog, "");
+	gtk_progress_set_show_text
+		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), FALSE);
 	gtk_progress_bar_update
 		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
 }
@@ -354,6 +369,8 @@ static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
 
 	inc_dialog_list = g_list_remove(inc_dialog_list, inc_dialog);
 
+	gtk_progress_set_show_text
+		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), FALSE);
 	gtk_progress_bar_update
 		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
 	progress_dialog_destroy(inc_dialog->dialog);
@@ -373,8 +390,8 @@ static IncSession *inc_session_new(PrefsAccount *account)
 		return NULL;
 
 	session = g_new0(IncSession, 1);
-	session->pop3_state = pop3_state_new(account);
-	session->pop3_state->data = session;
+	session->session = pop3_session_new(account);
+	session->session->data = session;
 	session->folder_table = g_hash_table_new(NULL, NULL);
 
 	return session;
@@ -384,7 +401,7 @@ static void inc_session_destroy(IncSession *session)
 {
 	g_return_if_fail(session != NULL);
 
-	pop3_state_destroy(session->pop3_state);
+	session_destroy(session->session);
 	g_hash_table_destroy(session->folder_table);
 	g_free(session);
 }
@@ -393,90 +410,121 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 {
 	IncSession *session;
 	GtkCList *clist = GTK_CLIST(inc_dialog->dialog->clist);
-	Pop3State *pop3_state;
+	GList *qlist;
+	Pop3Session *pop3_session;
 	IncState inc_state;
-	gint num = 0;
 	gint error_num = 0;
 	gint new_msgs = 0;
 	gchar *msg;
 	gchar *fin_msg;
 
-	while (inc_dialog->queue_list != NULL) {
-		session = inc_dialog->queue_list->data;
-		pop3_state = session->pop3_state;
+	qlist = inc_dialog->queue_list;
+	while (qlist != NULL) {
+		GList *next = qlist->next;
 
-		inc_progress_dialog_clear(inc_dialog);
-
-		gtk_clist_moveto(clist, num, -1, 1.0, 0.0);
-
-		pop3_state->user = g_strdup(pop3_state->ac_prefs->userid);
-		if (pop3_state->ac_prefs->passwd)
-			pop3_state->pass =
-				g_strdup(pop3_state->ac_prefs->passwd);
-		else if (pop3_state->ac_prefs->tmp_pass)
-			pop3_state->pass =
-				g_strdup(pop3_state->ac_prefs->tmp_pass);
+		session = qlist->data;
+		pop3_session = POP3_SESSION(session->session); 
+		pop3_session->user = g_strdup(pop3_session->ac_prefs->userid);
+		if (pop3_session->ac_prefs->passwd)
+			pop3_session->pass =
+				g_strdup(pop3_session->ac_prefs->passwd);
+		else if (pop3_session->ac_prefs->tmp_pass)
+			pop3_session->pass =
+				g_strdup(pop3_session->ac_prefs->tmp_pass);
 		else {
 			gchar *pass;
 
-			pass = input_dialog_query_password
-				(pop3_state->ac_prefs->recv_server,
-				 pop3_state->user);
-
 			if (inc_dialog->show_dialog)
 				manage_window_focus_in
-					(inc_dialog->mainwin->window,
+					(inc_dialog->dialog->window,
 					 NULL, NULL);
+
+			pass = input_dialog_query_password
+				(pop3_session->ac_prefs->recv_server,
+				 pop3_session->user);
+
+			if (inc_dialog->show_dialog)
+				manage_window_focus_out
+					(inc_dialog->dialog->window,
+					 NULL, NULL);
+
 			if (pass) {
-				pop3_state->ac_prefs->tmp_pass = g_strdup(pass);
-				pop3_state->pass = pass;
-			} else {
-				inc_session_destroy(session);
-				inc_dialog->queue_list = g_list_remove
-					(inc_dialog->queue_list, session);
-				continue;
+				pop3_session->ac_prefs->tmp_pass =
+					g_strdup(pass);
+				pop3_session->pass = pass;
 			}
 		}
 
-		gtk_clist_set_pixmap(clist, num, 0, currentxpm, currentxpmmask);
-		gtk_clist_set_text(clist, num, 2, _("Retrieving"));
+		qlist = next;
+	}
+
+#define SET_PIXMAP_AND_TEXT(xpm, xpmmask, str)				   \
+{									   \
+	gtk_clist_set_pixmap(clist, inc_dialog->cur_row, 0, xpm, xpmmask); \
+	gtk_clist_set_text(clist, inc_dialog->cur_row, 2, str);		   \
+}
+
+	for (; inc_dialog->queue_list != NULL; inc_dialog->cur_row++) {
+		session = inc_dialog->queue_list->data;
+		pop3_session = POP3_SESSION(session->session);
+
+		if (pop3_session->pass == NULL) {
+			SET_PIXMAP_AND_TEXT(okxpm, okxpmmask, _("Cancelled"));
+			inc_session_destroy(session);
+			inc_dialog->queue_list =
+				g_list_remove(inc_dialog->queue_list, session);
+			continue;
+		}
+
+		inc_progress_dialog_clear(inc_dialog);
+		gtk_clist_moveto(clist, inc_dialog->cur_row, -1, 1.0, 0.0);
+
+		SET_PIXMAP_AND_TEXT(currentxpm, currentxpmmask,
+				    _("Retrieving"));
+
+		session_set_recv_message_notify(session->session,
+						inc_recv_message, session);
+		session_set_recv_data_progressive_notify
+			(session->session, inc_recv_data_progressive, session);
+		session_set_recv_data_notify(session->session,
+					     inc_recv_data_finished, session);
 
 		/* begin POP3 session */
 		inc_state = inc_pop3_session_do(session);
 
 		switch (inc_state) {
 		case INC_SUCCESS:
-			if (pop3_state->cur_total_num > 0)
+			if (pop3_session->cur_total_num > 0)
 				msg = g_strdup_printf
 					(_("Done (%d message(s) (%s) received)"),
-					 pop3_state->cur_total_num,
-					 to_human_readable(pop3_state->cur_total_recv_bytes));
+					 pop3_session->cur_total_num,
+					 to_human_readable(pop3_session->cur_total_recv_bytes));
 			else
 				msg = g_strdup_printf(_("Done (no new messages)"));
-			gtk_clist_set_pixmap(clist, num, 0, okxpm, okxpmmask);
-			gtk_clist_set_text(clist, num, 2, msg);
+			SET_PIXMAP_AND_TEXT(okxpm, okxpmmask, msg);
 			g_free(msg);
 			break;
 		case INC_CONNECT_ERROR:
-			gtk_clist_set_pixmap(clist, num, 0, errorxpm, errorxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Connection failed"));
+			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask,
+					    _("Connection failed"));
 			break;
 		case INC_AUTH_FAILED:
-			gtk_clist_set_pixmap(clist, num, 0, errorxpm, errorxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Auth failed"));
+			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask,
+					    _("Auth failed"));
 			break;
 		case INC_LOCKED:
-			gtk_clist_set_pixmap(clist, num, 0, errorxpm, errorxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Locked"));
+			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask,
+					    _("Locked"));
 			break;
 		case INC_ERROR:
-		case INC_NOSPACE:
-			gtk_clist_set_pixmap(clist, num, 0, errorxpm, errorxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Error"));
+		case INC_NO_SPACE:
+		case INC_IO_ERROR:
+		case INC_SOCKET_ERROR:
+		case INC_EOF:
+			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask, _("Error"));
 			break;
 		case INC_CANCEL:
-			gtk_clist_set_pixmap(clist, num, 0, okxpm, okxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Cancelled"));
+			SET_PIXMAP_AND_TEXT(okxpm, okxpmmask, _("Cancelled"));
 			break;
 		default:
 			break;
@@ -484,7 +532,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 
 		statusbar_pop_all();
 
-		new_msgs += pop3_state->cur_total_num;
+		new_msgs += pop3_session->cur_total_num;
 
 		if (!prefs_common.scan_all_after_inc) {
 			folder_item_scan_foreach(session->folder_table);
@@ -493,29 +541,37 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 				 !prefs_common.open_inbox_on_inc);
 		}
 
-		if (pop3_state->error_val == PS_AUTHFAIL &&
-		    pop3_state->ac_prefs->tmp_pass) {
-			g_free(pop3_state->ac_prefs->tmp_pass);
-			pop3_state->ac_prefs->tmp_pass = NULL;
+		if (pop3_session->error_val == PS_AUTHFAIL &&
+		    pop3_session->ac_prefs->tmp_pass) {
+			g_free(pop3_session->ac_prefs->tmp_pass);
+			pop3_session->ac_prefs->tmp_pass = NULL;
 		}
 
-		pop3_write_uidl_list(pop3_state);
+		pop3_write_uidl_list(pop3_session);
 
 		if (inc_state != INC_SUCCESS && inc_state != INC_CANCEL) {
 			error_num++;
-			if (inc_state == INC_NOSPACE) {
-				inc_put_error(inc_state);
+			if (inc_dialog->show_dialog)
+				manage_window_focus_in
+					(inc_dialog->dialog->window,
+					 NULL, NULL);
+			inc_put_error(inc_state, pop3_session->error_msg);
+			if (inc_dialog->show_dialog)
+				manage_window_focus_out
+					(inc_dialog->dialog->window,
+					 NULL, NULL);
+			if (inc_state == INC_NO_SPACE ||
+			    inc_state == INC_IO_ERROR)
 				break;
-			}
 		}
 
 		inc_session_destroy(session);
 		inc_dialog->queue_list =
 			g_list_remove(inc_dialog->queue_list, session);
-
-		num++;
 	}
 
+#undef SET_PIXMAP_AND_TEXT
+
 	if (new_msgs > 0)
 		fin_msg = g_strdup_printf(_("Finished (%d new message(s))"),
 					  new_msgs);
@@ -524,6 +580,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 
 	progress_dialog_set_label(inc_dialog->dialog, fin_msg);
 
+#if 0
 	if (error_num && !prefs_common.no_recv_err_panel) {
 		if (inc_dialog->show_dialog)
 			manage_window_focus_in(inc_dialog->dialog->window,
@@ -533,6 +590,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			manage_window_focus_out(inc_dialog->dialog->window,
 						NULL, NULL);
 	}
+#endif
 
 	while (inc_dialog->queue_list != NULL) {
 		session = inc_dialog->queue_list->data;
@@ -557,250 +615,145 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 
 static IncState inc_pop3_session_do(IncSession *session)
 {
-	Pop3State *pop3_state = session->pop3_state;
+	Pop3Session *pop3_session = POP3_SESSION(session->session);
 	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
-	Automaton *atm;
-	SockInfo *sockinfo;
-	gint i;
 	gchar *server;
 	gushort port;
 	gchar *buf;
-	static AtmHandler handlers[] = {
-		pop3_greeting_recv      ,
-#if USE_SSL
-		pop3_stls_send          , pop3_stls_recv,
-#endif
-		pop3_getauth_user_send  , pop3_getauth_user_recv,
-		pop3_getauth_pass_send  , pop3_getauth_pass_recv,
-		pop3_getauth_apop_send  , pop3_getauth_apop_recv,
-		pop3_getrange_stat_send , pop3_getrange_stat_recv,
-		pop3_getrange_last_send , pop3_getrange_last_recv,
-		pop3_getrange_uidl_send , pop3_getrange_uidl_recv,
-		pop3_getsize_list_send  , pop3_getsize_list_recv,
-		pop3_retr_send          , pop3_retr_recv,
-		pop3_delete_send        , pop3_delete_recv,
-		pop3_logout_send        , pop3_logout_recv
-	};
 
 	debug_print(_("getting new messages of account %s...\n"),
-		    pop3_state->ac_prefs->account_name);
-
-	atm = automaton_create(N_POP3_PHASE);
-
-	session->atm = atm;
-	atm->data = pop3_state;
+		    pop3_session->ac_prefs->account_name);
 
 	buf = g_strdup_printf(_("%s: Retrieving new messages"),
-			      pop3_state->ac_prefs->recv_server);
+			      pop3_session->ac_prefs->recv_server);
 	gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window), buf);
 	g_free(buf);
 
-	for (i = POP3_GREETING_RECV; i < N_POP3_PHASE; i++)
-		atm->state[i].handler = handlers[i];
-	atm->state[POP3_GREETING_RECV].condition = GDK_INPUT_READ;
-	for (i = POP3_GREETING_RECV + 1; i < N_POP3_PHASE; ) {
-		atm->state[i++].condition = GDK_INPUT_WRITE;
-		atm->state[i++].condition = GDK_INPUT_READ;
-	}
-
-	atm->terminate = (AtmHandler)pop3_automaton_terminate;
-	atm->ui_func = (AtmUIFunc)inc_progress_update;
-
-	atm->num = POP3_GREETING_RECV;
-
-	server = pop3_state->ac_prefs->recv_server;
+	server = pop3_session->ac_prefs->recv_server;
 #if USE_SSL
-	port = pop3_state->ac_prefs->set_popport ?
-		pop3_state->ac_prefs->popport :
-		pop3_state->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
+	port = pop3_session->ac_prefs->set_popport ?
+		pop3_session->ac_prefs->popport :
+		pop3_session->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
+	SESSION(pop3_session)->ssl_type = pop3_session->ac_prefs->ssl_pop;
+	if (pop3_session->ac_prefs->ssl_pop != SSL_NONE)
+		SESSION(pop3_session)->nonblocking =
+			pop3_session->ac_prefs->use_nonblocking_ssl;
 #else
-	port = pop3_state->ac_prefs->set_popport ?
-		pop3_state->ac_prefs->popport : 110;
+	port = pop3_session->ac_prefs->set_popport ?
+		pop3_session->ac_prefs->popport : 110;
 #endif
 
-	buf = g_strdup_printf(_("Connecting to POP3 server: %s ..."), server);
+	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."), server);
 	log_message("%s\n", buf);
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
-	GTK_EVENTS_FLUSH();
-	statusbar_pop_all();
 
-	if ((sockinfo = sock_connect(server, port)) == NULL) {
+	if (session_connect(SESSION(pop3_session), server, port) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    server, port);
-		pop3_automaton_terminate(NULL, atm);
-		automaton_destroy(atm);
 		session->inc_state = INC_CONNECT_ERROR;
+		statusbar_pop_all();
 		return INC_CONNECT_ERROR;
 	}
 
-#if USE_SSL
-	if (pop3_state->ac_prefs->ssl_pop == SSL_TUNNEL &&
-	    !ssl_init_socket(sockinfo)) {
-		pop3_automaton_terminate(NULL, atm);
-		automaton_destroy(atm);
-		session->inc_state = INC_CONNECT_ERROR;
-		return INC_CONNECT_ERROR;
-	}
-#endif
-
-	/* :WK: Hmmm, with the later sock_gdk_input, we have 2 references
-	 * to the sock structure - implement a reference counter?? */
-	pop3_state->sockinfo = sockinfo;
-	atm->help_sock = sockinfo;
-
-	log_verbosity_set(TRUE);
-	recv_set_ui_func(inc_pop3_recv_func, session);
-
-	atm->tag = sock_gdk_input_add(sockinfo,
-				      atm->state[atm->num].condition,
-				      automaton_input_cb, atm);
-
-	while (!atm->terminated)
+	while (session_is_connected(SESSION(pop3_session)) &&
+	       session->inc_state != INC_CANCEL)
 		gtk_main_iteration();
 
-	log_verbosity_set(FALSE);
-	recv_set_ui_func(NULL, NULL);
+	statusbar_pop_all();
 
-	automaton_destroy(atm);
-
-	if (session->inc_state != INC_SUCCESS)
-		return session->inc_state;
-
-	switch (pop3_state->error_val) {
-	case PS_SUCCESS:
-		session->inc_state = INC_SUCCESS;
-		break;
-	case PS_AUTHFAIL:
-		session->inc_state = INC_AUTH_FAILED;
-		break;
-	case PS_IOERR:
-		session->inc_state = INC_NOSPACE;
-		break;
-	case PS_LOCKBUSY:
-		session->inc_state = INC_LOCKED;
-		break;
-	default:
-		session->inc_state = INC_ERROR;
-		break;
+	if (session->inc_state == INC_SUCCESS) {
+		switch (pop3_session->error_val) {
+		case PS_SUCCESS:
+			if (SESSION(pop3_session)->state == SESSION_ERROR) {
+				if (pop3_session->state == POP3_READY)
+					session->inc_state = INC_CONNECT_ERROR;
+				else
+					session->inc_state = INC_ERROR;
+			} else if (SESSION(pop3_session)->state == SESSION_EOF)
+				session->inc_state = INC_EOF;
+			else
+				session->inc_state = INC_SUCCESS;
+			break;
+		case PS_AUTHFAIL:
+			session->inc_state = INC_AUTH_FAILED;
+			break;
+		case PS_IOERR:
+			session->inc_state = INC_IO_ERROR;
+			break;
+		case PS_SOCKET:
+			session->inc_state = INC_SOCKET_ERROR;
+			break;
+		case PS_LOCKBUSY:
+			session->inc_state = INC_LOCKED;
+			break;
+		default:
+			session->inc_state = INC_ERROR;
+			break;
+		}
 	}
 
 	return session->inc_state;
 }
 
-static gint pop3_automaton_terminate(SockInfo *source, Automaton *atm)
-{
-	if (atm->terminated) return 0;
-
-	if (atm->tag > 0) {
-		gdk_input_remove(atm->tag);
-		atm->tag = 0;
-	}
-	if (atm->timeout_tag > 0) {
-		gtk_timeout_remove(atm->timeout_tag);
-		atm->timeout_tag = 0;
-	}
-	if (source)
-		sock_close(source);
-
-	atm->terminated = TRUE;
-
-	return 0;
-}
-
-static gboolean inc_pop3_recv_func(SockInfo *sock, gint count, gint read_bytes,
-				   gpointer data)
+static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
+					  IncSession *inc_session)
 {
 	gchar buf[MSGBUFSIZE];
-	IncSession *session = (IncSession *)data;
-	Pop3State *state = session->pop3_state;
-	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
 	ProgressDialog *dialog = inc_dialog->dialog;
-	gint cur_total;
-	gchar *total_size;
+	Pop3Session *session;
 
-	cur_total = state->cur_total_bytes + read_bytes;
-	if (cur_total > state->total_bytes)
-		cur_total = state->total_bytes;
+	g_return_if_fail(inc_session != NULL);
 
-	Xstrdup_a(total_size, to_human_readable(state->total_bytes),
-		  return FALSE);
-	g_snprintf(buf, sizeof(buf),
-		   _("Retrieving message (%d / %d) (%s / %s)"),
-		   state->cur_msg, state->count,
-		   to_human_readable(cur_total), total_size);
-	progress_dialog_set_label(dialog, buf);
+	session = POP3_SESSION(inc_session->session);
 
-	progress_dialog_set_percentage
-		(dialog, (gfloat)cur_total / (gfloat)state->total_bytes);
-	gtk_progress_bar_update
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
-		 (gfloat)cur_total / (gfloat)state->total_bytes);
-	GTK_EVENTS_FLUSH();
-
-	if (session->inc_state == INC_CANCEL)
-		return FALSE;
-	else
-		return TRUE;
-}
-
-void inc_progress_update(Pop3State *state, Pop3Phase phase)
-{
-	gchar buf[MSGBUFSIZE];
-	IncSession *session = (IncSession *)state->data;
-	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
-	ProgressDialog *dialog = inc_dialog->dialog;
-	gchar *total_size;
-
-	switch (phase) {
-	case POP3_GREETING_RECV:
+	switch (session->state) {
+	case POP3_GREETING:
 		break;
-	case POP3_GETAUTH_USER_SEND:
-	case POP3_GETAUTH_PASS_SEND:
-	case POP3_GETAUTH_APOP_SEND:
+	case POP3_GETAUTH_USER:
+	case POP3_GETAUTH_PASS:
+	case POP3_GETAUTH_APOP:
 		progress_dialog_set_label(dialog, _("Authenticating..."));
+		statusbar_print_all(_("Retrieving messages from %s..."),
+				    SESSION(session)->server);
 		break;
-	case POP3_GETRANGE_STAT_SEND:
+	case POP3_GETRANGE_STAT:
 		progress_dialog_set_label
 			(dialog, _("Getting the number of new messages (STAT)..."));
 		break;
-	case POP3_GETRANGE_LAST_SEND:
+	case POP3_GETRANGE_LAST:
 		progress_dialog_set_label
 			(dialog, _("Getting the number of new messages (LAST)..."));
 		break;
-	case POP3_GETRANGE_UIDL_SEND:
+	case POP3_GETRANGE_UIDL:
 		progress_dialog_set_label
 			(dialog, _("Getting the number of new messages (UIDL)..."));
 		break;
-	case POP3_GETSIZE_LIST_SEND:
+	case POP3_GETSIZE_LIST:
 		progress_dialog_set_label
 			(dialog, _("Getting the size of messages (LIST)..."));
 		break;
-	case POP3_RETR_SEND:
-		Xstrdup_a(total_size, to_human_readable(state->total_bytes), return);
-		g_snprintf(buf, sizeof(buf),
-			   _("Retrieving message (%d / %d) (%s / %s)"),
-			   state->cur_msg, state->count,
-			   to_human_readable(state->cur_total_bytes),
-			   total_size);
-		progress_dialog_set_label(dialog, buf);
-		progress_dialog_set_percentage
-			(dialog,
-			 (gfloat)(state->cur_total_bytes) /
-			 (gfloat)(state->total_bytes));
-		gtk_progress_bar_update
-			(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
-			 (gfloat)(state->cur_total_bytes) /
-			 (gfloat)(state->total_bytes));
+	case POP3_RETR:
+		gtk_progress_set_show_text
+			(GTK_PROGRESS(inc_dialog->mainwin->progressbar), TRUE);
+		g_snprintf(buf, sizeof(buf), "%d / %d",
+			   session->cur_msg, session->count);
+		gtk_progress_set_format_string
+			(GTK_PROGRESS(inc_dialog->mainwin->progressbar), buf);
+		inc_recv_data_progressive
+			(SESSION(session), 0,
+			 session->msg[session->cur_msg].size,
+			 inc_session);
 		break;
-	case POP3_DELETE_SEND:
-		if (state->msg[state->cur_msg].recv_time < state->current_time) {
+	case POP3_DELETE:
+		if (session->msg[session->cur_msg].recv_time <
+			session->current_time) {
 			g_snprintf(buf, sizeof(buf), _("Deleting message %d"),
-				   state->cur_msg);
+				   session->cur_msg);
 			progress_dialog_set_label(dialog, buf);
 		}
 		break;
-	case POP3_LOGOUT_SEND:
+	case POP3_LOGOUT:
 		progress_dialog_set_label(dialog, _("Quitting"));
 		break;
 	default:
@@ -808,16 +761,97 @@ void inc_progress_update(Pop3State *state, Pop3Phase phase)
 	}
 }
 
-gint inc_drop_message(const gchar *file, Pop3State *state)
+static gint inc_recv_data_progressive(Session *session, guint cur_len,
+				      guint total_len, gpointer data)
+{
+	gchar buf[MSGBUFSIZE];
+	IncSession *inc_session = (IncSession *)data;
+	Pop3Session *pop3_session = POP3_SESSION(session);
+	IncProgressDialog *inc_dialog;
+	ProgressDialog *dialog;
+	gint cur_total;
+	gchar *total_size;
+
+	g_return_val_if_fail(inc_session != NULL, -1);
+
+	if (pop3_session->state != POP3_RETR &&
+	    pop3_session->state != POP3_RETR_RECV &&
+	    pop3_session->state != POP3_DELETE &&
+	    pop3_session->state != POP3_LOGOUT) return 0;
+
+	if (!pop3_session->new_msg_exist) return 0;
+
+	inc_dialog = (IncProgressDialog *)inc_session->data;
+	dialog = inc_dialog->dialog;
+
+	cur_total = pop3_session->cur_total_bytes + cur_len;
+	if (cur_total > pop3_session->total_bytes)
+		cur_total = pop3_session->total_bytes;
+
+	Xstrdup_a(total_size, to_human_readable(pop3_session->total_bytes),
+		  return FALSE);
+	g_snprintf(buf, sizeof(buf),
+		   _("Retrieving message (%d / %d) (%s / %s)"),
+		   pop3_session->cur_msg, pop3_session->count,
+		   to_human_readable(cur_total), total_size);
+	progress_dialog_set_label(dialog, buf);
+	progress_dialog_set_percentage
+		(dialog, (gfloat)cur_total / (gfloat)pop3_session->total_bytes);
+	gtk_progress_bar_update
+		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
+		 (gfloat)cur_total / (gfloat)pop3_session->total_bytes);
+
+	return 0;
+}
+
+static gint inc_recv_data_finished(Session *session, guint len, gpointer data)
+{
+	IncSession *inc_session = (IncSession *)data;
+	Pop3Session *pop3_session = POP3_SESSION(session);
+	IncProgressDialog *inc_dialog;
+	gchar *msg;
+
+	g_return_val_if_fail(inc_session != NULL, -1);
+
+	inc_dialog = (IncProgressDialog *)inc_session->data;
+	inc_recv_data_progressive(session, 0, len, inc_session);
+	inc_progress_dialog_set_label(inc_dialog, inc_session);
+
+	if (pop3_session->cur_total_num == 0) return 0;
+
+	msg = g_strdup_printf(_("Retrieving (%d message(s) (%s) received)"),
+			      pop3_session->cur_total_num,
+			      to_human_readable
+				(pop3_session->cur_total_recv_bytes));
+	gtk_clist_set_text(GTK_CLIST(inc_dialog->dialog->clist),
+			   inc_dialog->cur_row, 2, msg);
+
+	return 0;
+}
+
+static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
+{
+	IncSession *inc_session = (IncSession *)data;
+	IncProgressDialog *inc_dialog;
+
+	g_return_val_if_fail(inc_session != NULL, -1);
+
+	inc_dialog = (IncProgressDialog *)inc_session->data;
+	inc_progress_dialog_set_label(inc_dialog, inc_session);
+
+	return 0;
+}
+
+gint inc_drop_message(const gchar *file, Pop3Session *session)
 {
 	FolderItem *inbox;
 	FolderItem *dropfolder;
-	IncSession *session = (IncSession *)state->data;
+	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
 
-	if (state->ac_prefs->inbox) {
+	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
-			(state->ac_prefs->inbox);
+			(session->ac_prefs->inbox);
 		if (!inbox)
 			inbox = folder_get_default_inbox();
 	} else
@@ -827,7 +861,7 @@ gint inc_drop_message(const gchar *file, Pop3State *state)
 		return -1;
 	}
 
-	if (state->ac_prefs->filter_on_recv) {
+	if (session->ac_prefs->filter_on_recv) {
 		dropfolder =
 			filter_get_dest_folder(prefs_common.fltlist, file);
 		if (!dropfolder) dropfolder = inbox;
@@ -840,17 +874,17 @@ gint inc_drop_message(const gchar *file, Pop3State *state)
 		dropfolder = inbox;
 
 	val = GPOINTER_TO_INT(g_hash_table_lookup
-			      (session->folder_table, dropfolder));
+			      (inc_session->folder_table, dropfolder));
 	if (val == 0) {
 		folder_item_scan(dropfolder);
 		/* force updating */
 		if (FOLDER_IS_LOCAL(dropfolder->folder))
 			dropfolder->mtime = 0;
-		g_hash_table_insert(session->folder_table, dropfolder,
+		g_hash_table_insert(inc_session->folder_table, dropfolder,
 				    GINT_TO_POINTER(1));
 	}
 
-	if (folder_item_add_msg(dropfolder, file, TRUE) < 0) {
+	if (folder_item_add_msg(dropfolder, file, NULL, TRUE) < 0) {
 		unlink(file);
 		return -1;
 	}
@@ -858,30 +892,91 @@ gint inc_drop_message(const gchar *file, Pop3State *state)
 	return 0;
 }
 
-static void inc_put_error(IncState istate)
+static void inc_put_error(IncState istate, const gchar *msg)
 {
+	gchar *log_msg = NULL;
+	gchar *err_msg = NULL;
+	gboolean fatal_error = FALSE;
+
 	switch (istate) {
-	case INC_ERROR:
-		if (!prefs_common.no_recv_err_panel)
-			alertpanel_error
-				(_("Error occurred while processing mail."));
+	case INC_CONNECT_ERROR:
+		log_msg = _("Connection failed.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		err_msg = g_strdup(log_msg);
 		break;
-	case INC_NOSPACE:
-		alertpanel_error(_("No disk space left."));
+	case INC_ERROR:
+		log_msg = _("Error occurred while processing mail.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		if (msg)
+			err_msg = g_strdup_printf
+				(_("Error occurred while processing mail:\n%s"),
+				 msg);
+		else
+			err_msg = g_strdup(log_msg);
+		break;
+	case INC_NO_SPACE:
+		log_msg = _("No disk space left.");
+		err_msg = g_strdup(log_msg);
+		fatal_error = TRUE;
+		break;
+	case INC_IO_ERROR:
+		log_msg = _("Can't write file.");
+		err_msg = g_strdup(log_msg);
+		fatal_error = TRUE;
+		break;
+	case INC_SOCKET_ERROR:
+		log_msg = _("Socket error.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		err_msg = g_strdup(log_msg);
+		break;
+	case INC_EOF:
+		log_msg = _("Connection closed by the remote host.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		err_msg = g_strdup(log_msg);
 		break;
 	case INC_LOCKED:
-		if (!prefs_common.no_recv_err_panel)
-			alertpanel_error(_("Mailbox is locked."));
+		log_msg = _("Mailbox is locked.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		if (msg)
+			err_msg = g_strdup_printf(_("Mailbox is locked:\n%s"),
+						  msg);
+		else
+			err_msg = g_strdup(log_msg);
+		break;
+	case INC_AUTH_FAILED:
+		log_msg = _("Authentication failed.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		if (msg)
+			err_msg = g_strdup_printf
+				(_("Authentication failed:\n%s"), msg);
+		else
+			err_msg = g_strdup(log_msg);
 		break;
 	default:
 		break;
 	}
+
+	if (log_msg) {
+		if (fatal_error)
+			log_error("%s\n", log_msg);
+		else
+			log_warning("%s\n", log_msg);
+	}
+	if (err_msg) {
+		alertpanel_error(err_msg);
+		g_free(err_msg);
+	}
 }
 
 static void inc_cancel(IncProgressDialog *dialog)
 {
 	IncSession *session;
-	SockInfo *sockinfo;
 
 	g_return_if_fail(dialog != NULL);
 
@@ -891,13 +986,8 @@ static void inc_cancel(IncProgressDialog *dialog)
 	}
 
 	session = dialog->queue_list->data;
-	sockinfo = session->pop3_state->sockinfo;
 
-	if (!sockinfo || session->atm->terminated == TRUE) return;
-
-	session->pop3_state->cancelled = TRUE;
 	session->inc_state = INC_CANCEL;
-	session->atm->cancelled = TRUE;
 
 	log_message(_("Incorporation cancelled\n"));
 }
@@ -1054,7 +1144,7 @@ static gint inc_autocheck_func(gpointer data)
 		return FALSE;
 	}
 
-	inc_all_account_mail(mainwin);
+	inc_all_account_mail(mainwin, TRUE);
 
 	return FALSE;
 }
