@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,6 +64,7 @@
 #include "prefs_account.h"
 #include "prefs_summary_column.h"
 #include "prefs_template.h"
+#include "action.h"
 #include "account.h"
 #include "addressbook.h"
 #include "logwindow.h"
@@ -149,22 +150,26 @@ static void ac_label_button_pressed		(GtkWidget	*widget,
 static void ac_menu_popup_closed		(GtkMenuShell	*menu_shell,
 						 gpointer	 data);
 
-static gint main_window_close_cb	(GtkWidget	*widget,
-					 GdkEventAny	*event,
-					 gpointer	 data);
-static gint folder_window_close_cb	(GtkWidget	*widget,
-					 GdkEventAny	*event,
-					 gpointer	 data);
-static gint message_window_close_cb	(GtkWidget	*widget,
-					 GdkEventAny	*event,
-					 gpointer	 data);
+static gint main_window_close_cb		(GtkWidget	*widget,
+						 GdkEventAny	*event,
+						 gpointer	 data);
+static gint folder_window_close_cb		(GtkWidget	*widget,
+						 GdkEventAny	*event,
+						 gpointer	 data);
+static gint message_window_close_cb		(GtkWidget	*widget,
+						 GdkEventAny	*event,
+						 gpointer	 data);
+
+static void main_window_size_allocate_cb	(GtkWidget	*widget,
+						 GtkAllocation	*allocation,
+						 gpointer	 data);
+static void folder_window_size_allocate_cb	(GtkWidget	*widget,
+						 GtkAllocation	*allocation,
+						 gpointer	 data);
+static void message_window_size_allocate_cb	(GtkWidget	*widget,
+						 GtkAllocation	*allocation,
+						 gpointer	 data);
 
-static void add_mailbox_cb	 (MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
-static void update_folderview_cb (MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
 static void new_folder_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -174,6 +179,12 @@ static void rename_folder_cb	 (MainWindow	*mainwin,
 static void delete_folder_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void update_folderview_cb (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void add_mailbox_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
 static void import_mbox_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -415,14 +426,15 @@ static void scan_tree_func	 (Folder	*folder,
 static GtkItemFactoryEntry mainwin_entries[] =
 {
 	{N_("/_File"),				NULL, NULL, 0, "<Branch>"},
-	{N_("/_File/_Add mailbox..."),		NULL, add_mailbox_cb, 0, NULL},
-	{N_("/_File/_Check for new messages in all folders"),
-						NULL, update_folderview_cb, 0, NULL},
 	{N_("/_File/_Folder"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_File/_Folder/Create _new folder..."),
 						NULL, new_folder_cb, 0, NULL},
 	{N_("/_File/_Folder/_Rename folder..."),NULL, rename_folder_cb, 0, NULL},
 	{N_("/_File/_Folder/_Delete folder"),	NULL, delete_folder_cb, 0, NULL},
+	{N_("/_File/_Folder/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_File/_Folder/_Check for new messages in all folders"),
+						NULL, update_folderview_cb, 0, NULL},
+	{N_("/_File/_Add mailbox..."),		NULL, add_mailbox_cb, 0, NULL},
 	{N_("/_File/_Import mbox file..."),	NULL, import_mbox_cb, 0, NULL},
 	{N_("/_File/_Export to mbox file..."),	NULL, export_mbox_cb, 0, NULL},
 	{N_("/_File/Empty _trash"),		NULL, empty_trash_cb, 0, NULL},
@@ -469,6 +481,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Sort/by s_ize"),		NULL, sort_summary_cb, SORT_BY_SIZE, "/View/Sort/by number"},
 	{N_("/_View/_Sort/by _date"),		NULL, sort_summary_cb, SORT_BY_DATE, "/View/Sort/by number"},
 	{N_("/_View/_Sort/by _from"),		NULL, sort_summary_cb, SORT_BY_FROM, "/View/Sort/by number"},
+	{N_("/_View/_Sort/by _recipient"),	NULL, sort_summary_cb, SORT_BY_TO, "/View/Sort/by number"},
 	{N_("/_View/_Sort/by _subject"),	NULL, sort_summary_cb, SORT_BY_SUBJECT, "/View/Sort/by number"},
 	{N_("/_View/_Sort/by _color label"),
 						NULL, sort_summary_cb, SORT_BY_LABEL, "/View/Sort/by number"},
@@ -526,7 +539,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Code set/7bit ascii (US-ASC_II)"),
 	 CODESET_ACTION(C_US_ASCII)},
 
-#if HAVE_LIBJCONV
+#if HAVE_ICONV
 	{N_("/_View/_Code set/Unicode (_UTF-8)"),
 	 CODESET_ACTION(C_UTF_8)},
 	CODESET_SEPARATOR,
@@ -536,7 +549,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Code set/Western European (ISO-8859-15)"),
 	 CODESET_ACTION(C_ISO_8859_15)},
 	CODESET_SEPARATOR,
-#if HAVE_LIBJCONV
+#if HAVE_ICONV
 	{N_("/_View/_Code set/Central European (ISO-8859-_2)"),
 	 CODESET_ACTION(C_ISO_8859_2)},
 	CODESET_SEPARATOR,
@@ -561,7 +574,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 #endif
 	{N_("/_View/_Code set/Japanese (ISO-2022-_JP)"),
 	 CODESET_ACTION(C_ISO_2022_JP)},
-#if HAVE_LIBJCONV
+#if HAVE_ICONV
 	{N_("/_View/_Code set/Japanese (ISO-2022-JP-2)"),
 	 CODESET_ACTION(C_ISO_2022_JP_2)},
 #endif
@@ -569,7 +582,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	 CODESET_ACTION(C_EUC_JP)},
 	{N_("/_View/_Code set/Japanese (_Shift__JIS)"),
 	 CODESET_ACTION(C_SHIFT_JIS)},
-#if HAVE_LIBJCONV
+#if HAVE_ICONV
 	CODESET_SEPARATOR,
 	{N_("/_View/_Code set/Simplified Chinese (_GB2312)"),
 	 CODESET_ACTION(C_GB2312)},
@@ -852,10 +865,14 @@ MainWindow *main_window_create(SeparateType type)
 			g_warning(_("MainWindow: color allocation %d failed\n"), i);
 	}
 
-	messageview->visible = TRUE;
+	messageview->visible = prefs_common.msgview_visible;
 
 	main_window_set_widgets(mainwin, type);
 
+	gtk_signal_connect(GTK_OBJECT(window), "size_allocate",
+			   GTK_SIGNAL_FUNC(main_window_size_allocate_cb),
+			   mainwin);
+
 	/* set menu items */
 	menuitem = gtk_item_factory_get_item
 		(ifactory, "/View/Code set/Auto detect");
@@ -896,14 +913,9 @@ MainWindow *main_window_create(SeparateType type)
 	main_window_set_toolbar_sensitive(mainwin);
 
 	/* create actions menu */
-	update_mainwin_actions_menu(ifactory, mainwin);
+	action_update_mainwin_menu(ifactory, mainwin);
 
 	/* show main window */
-	gtk_widget_set_uposition(mainwin->window,
-				 prefs_common.mainwin_x,
-				 prefs_common.mainwin_y);
-	gtk_widget_set_usize(window, prefs_common.mainwin_width,
-			     prefs_common.mainwin_height);
 	gtk_widget_show(mainwin->window);
 
 	/* initialize views */
@@ -1074,6 +1086,30 @@ static void main_window_show_cur_account(MainWindow *mainwin)
 	g_free(ac_name);
 }
 
+GtkWidget *main_window_get_folder_window(MainWindow *mainwin)
+{
+	switch (mainwin->type) {
+	case SEPARATE_FOLDER:
+		return mainwin->win.sep_folder.folderwin;
+	case SEPARATE_BOTH:
+		return mainwin->win.sep_both.folderwin;
+	default:
+		return NULL;
+	}
+}
+
+GtkWidget *main_window_get_message_window(MainWindow *mainwin)
+{
+	switch (mainwin->type) {
+	case SEPARATE_MESSAGE:
+		return mainwin->win.sep_message.messagewin;
+	case SEPARATE_BOTH:
+		return mainwin->win.sep_both.messagewin;
+	default:
+		return NULL;
+	}
+}
+
 void main_window_separation_change(MainWindow *mainwin, SeparateType type)
 {
 	GtkWidget *folder_wid  = GTK_WIDGET_PTR(mainwin->folderview);
@@ -1177,6 +1213,8 @@ void main_window_toggle_message_view(MainWindow *mainwin)
 
 	main_window_set_menu_sensitive(mainwin);
 
+	prefs_common.msgview_visible = mainwin->messageview->visible;
+
 	gtk_widget_grab_focus(summaryview->ctree);
 }
 
@@ -1186,30 +1224,51 @@ void main_window_get_size(MainWindow *mainwin)
 
 	allocation = &(GTK_WIDGET_PTR(mainwin->summaryview)->allocation);
 
-	prefs_common.summaryview_width  = allocation->width;
+	if (allocation->width > 1 && allocation->height > 1) {
+		prefs_common.summaryview_width = allocation->width;
 
-	if ((mainwin->type == SEPARATE_NONE ||
-	     mainwin->type == SEPARATE_FOLDER) &&
-	    messageview_is_visible(mainwin->messageview))
-		prefs_common.summaryview_height = allocation->height;
+		if ((mainwin->type == SEPARATE_NONE ||
+		     mainwin->type == SEPARATE_FOLDER) &&
+		    messageview_is_visible(mainwin->messageview))
+			prefs_common.summaryview_height = allocation->height;
 
-	prefs_common.mainview_width     = allocation->width;
+		prefs_common.mainview_width = allocation->width;
+	}
 
 	allocation = &mainwin->window->allocation;
-
-	prefs_common.mainview_height = allocation->height;
-	prefs_common.mainwin_width   = allocation->width;
-	prefs_common.mainwin_height  = allocation->height;
+	if (allocation->width > 1 && allocation->height > 1) {
+		prefs_common.mainview_height = allocation->height;
+		prefs_common.mainwin_width   = allocation->width;
+		prefs_common.mainwin_height  = allocation->height;
+	}
 
 	allocation = &(GTK_WIDGET_PTR(mainwin->folderview)->allocation);
+	if (allocation->width > 1 && allocation->height > 1) {
+		prefs_common.folderview_width  = allocation->width;
+		prefs_common.folderview_height = allocation->height;
+	}
 
-	prefs_common.folderview_width  = allocation->width;
-	prefs_common.folderview_height = allocation->height;
+	allocation = &(GTK_WIDGET_PTR(mainwin->messageview)->allocation);
+	if (allocation->width > 1 && allocation->height > 1) {
+		prefs_common.msgview_width = allocation->width;
+		prefs_common.msgview_height = allocation->height;
+	}
+
+	debug_print("summaryview size: %d x %d\n",
+		    prefs_common.summaryview_width,
+		    prefs_common.summaryview_height);
+	debug_print("folderview size: %d x %d\n",
+		    prefs_common.folderview_width,
+		    prefs_common.folderview_height);
+	debug_print("messageview size: %d x %d\n",
+		    prefs_common.msgview_width,
+		    prefs_common.msgview_height);
 }
 
 void main_window_get_position(MainWindow *mainwin)
 {
 	gint x, y;
+	GtkWidget *window;
 
 	gtkut_widget_get_uposition(mainwin->window, &x, &y);
 
@@ -1218,7 +1277,22 @@ void main_window_get_position(MainWindow *mainwin)
 	prefs_common.mainwin_x = x;
 	prefs_common.mainwin_y = y;
 
-	debug_print(_("window position: x = %d, y = %d\n"), x, y);
+	debug_print("main window position: %d, %d\n", x, y);
+
+	window = main_window_get_folder_window(mainwin);
+	if (window) {
+		gtkut_widget_get_uposition(window, &x, &y);
+		prefs_common.folderwin_x = x;
+		prefs_common.folderwin_y = y;
+		debug_print("folder window position: %d, %d\n", x, y);
+	}
+	window = main_window_get_message_window(mainwin);
+	if (window) {
+		gtkut_widget_get_uposition(window, &x, &y);
+		prefs_common.main_msgwin_x = x;
+		prefs_common.main_msgwin_y = y;
+		debug_print("message window position: %d, %d\n", x, y);
+	}
 }
 
 void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
@@ -1239,10 +1313,8 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 		Folder *folder;
 
 		folder = list->data;
-		if (folder->trash && folder->trash->total != 0) {
-			folder_item_scan(folder->trash);
+		if (folder->trash)
 			folderview_update_item(folder->trash, TRUE);
-		}
 	}
 
 	if (mainwin->summaryview->folder_item &&
@@ -1272,7 +1344,7 @@ void main_window_add_mailbox(MainWindow *mainwin)
 		folder = folder_new(F_MH, g_basename(path), path);
 	g_free(path);
 
-	if (folder->create_tree(folder) < 0) {
+	if (folder->klass->create_tree(folder) < 0) {
 		alertpanel_error(_("Creation of the mailbox failed.\n"
 				   "Maybe some files already exist, or you don't have the permission to write there."));
 		folder_destroy(folder);
@@ -1281,7 +1353,7 @@ void main_window_add_mailbox(MainWindow *mainwin)
 
 	folder_add(folder);
 	folder_set_ui_func(folder, scan_tree_func, mainwin);
-	folder->scan_tree(folder);
+	folder->klass->scan_tree(folder);
 	folder_set_ui_func(folder, NULL, NULL);
 
 	folderview_set(mainwin->folderview);
@@ -1320,7 +1392,7 @@ static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
 			state |= M_THREADED;
 		else
 			state |= M_UNTHREADED;	
-		if (item->folder->type != F_NEWS)
+		if (FOLDER_TYPE(item->folder) != F_NEWS)
 			state |= M_ALLOW_DELETE;
 	}
 	if (selection == SUMMARY_SELECTED_SINGLE ||
@@ -1374,8 +1446,8 @@ void main_window_set_toolbar_sensitive(MainWindow *mainwin)
 #endif
 	SET_WIDGET_COND(mainwin->next_btn, M_MSG_EXIST);
 	SET_WIDGET_COND(mainwin->delete_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE|M_UNLOCKED);
-	SET_WIDGET_COND(mainwin->exec_btn, M_MSG_EXIST|M_EXEC|M_UNLOCKED);
+			M_TARGET_EXIST|M_ALLOW_DELETE);
+	SET_WIDGET_COND(mainwin->exec_btn, M_MSG_EXIST|M_EXEC);
 	SET_WIDGET_COND(NULL, 0);
 
 #undef SET_WIDGET_COND
@@ -1402,15 +1474,14 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		gchar *const entry;
 		SensitiveCond cond;
 	} entry[] = {
-		{"/File/Add mailbox..."                       , M_UNLOCKED},
-		{"/File/Check for new messages in all folders", M_UNLOCKED},
 		{"/File/Folder"                               , M_UNLOCKED},
+		{"/File/Add mailbox..."                       , M_UNLOCKED},
 		{"/File/Import mbox file..."                  , M_UNLOCKED},
 		{"/File/Export to mbox file..."               , M_UNLOCKED},
 		{"/File/Empty trash"                          , M_UNLOCKED},
 
-		{"/File/Save as...", M_SINGLE_TARGET_EXIST|M_UNLOCKED},
-		{"/File/Print..."  , M_TARGET_EXIST|M_UNLOCKED},
+		{"/File/Save as...", M_SINGLE_TARGET_EXIST},
+		{"/File/Print..."  , M_TARGET_EXIST},
 		/* {"/File/Close"  , M_UNLOCKED}, */
 		{"/File/Exit"      , M_UNLOCKED},
 
@@ -1444,17 +1515,17 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/Message/Forward as attachment", M_HAVE_ACCOUNT|M_TARGET_EXIST},
 		{"/Message/Redirect"             , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
 		{"/Message/Re-edit"              , M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
-		{"/Message/Move..."              , M_TARGET_EXIST|M_ALLOW_DELETE|M_UNLOCKED},
-		{"/Message/Copy..."              , M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
-		{"/Message/Delete"               , M_TARGET_EXIST|M_ALLOW_DELETE|M_UNLOCKED},
+		{"/Message/Move..."              , M_TARGET_EXIST|M_ALLOW_DELETE},
+		{"/Message/Copy..."              , M_TARGET_EXIST|M_EXEC},
+		{"/Message/Delete"               , M_TARGET_EXIST|M_ALLOW_DELETE},
 		{"/Message/Mark"                 , M_TARGET_EXIST},
 
 		{"/Tools/Add sender to address book", M_SINGLE_TARGET_EXIST},
-		{"/Tools/Filter messages"           , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Tools/Filter messages"           , M_MSG_EXIST|M_EXEC},
 		{"/Tools/Create filter rule"        , M_SINGLE_TARGET_EXIST|M_UNLOCKED},
 		{"/Tools/Actions"                   , M_TARGET_EXIST|M_UNLOCKED},
-		{"/Tools/Execute"                   , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
-		{"/Tools/Delete duplicated messages", M_MSG_EXIST|M_ALLOW_DELETE|M_UNLOCKED},
+		{"/Tools/Execute"                   , M_MSG_EXIST|M_EXEC},
+		{"/Tools/Delete duplicated messages", M_MSG_EXIST|M_ALLOW_DELETE},
 
 		{"/Configuration", M_UNLOCKED},
 
@@ -1491,6 +1562,8 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 			menu_path = "/View/Sort/by date"; break;
 		case SORT_BY_FROM:
 			menu_path = "/View/Sort/by from"; break;
+		case SORT_BY_TO:
+			menu_path = "/View/Sort/by recipient"; break;
 		case SORT_BY_SUBJECT:
 			menu_path = "/View/Sort/by subject"; break;
 		case SORT_BY_LABEL:
@@ -1533,51 +1606,18 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 
 void main_window_popup(MainWindow *mainwin)
 {
-	gint x, y;
-	gint sx, sy;
-	GtkWidget *widget;
-
-	gdk_window_get_origin(mainwin->window->window, &x, &y);
-	sx = gdk_screen_width();
-	sy = gdk_screen_height();
-	x %= sx; if (x < 0) x = 0;
-	y %= sy; if (y < 0) y = 0;
-	gdk_window_move(mainwin->window->window, x, y);
-	gdk_window_raise(mainwin->window->window);
-	gdk_window_show(mainwin->window->window);
-
-	debug_print("window position: x = %d, y = %d\n", x, y);
+	gtkut_window_popup(mainwin->window);
 
 	switch (mainwin->type) {
 	case SEPARATE_FOLDER:
-		widget = mainwin->win.sep_folder.folderwin;
-		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x = 0;
-		y %= sy; if (y < 0) y = 0;
-		gdk_window_move(widget->window, x, y);
-		gdk_window_raise(widget->window);
+		gtkut_window_popup(mainwin->win.sep_folder.folderwin);
 		break;
 	case SEPARATE_MESSAGE:
-		widget = mainwin->win.sep_message.messagewin;
-		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x = 0;
-		y %= sy; if (y < 0) y = 0;
-		gdk_window_move(widget->window, x, y);
-		gdk_window_raise(widget->window);
+		gtkut_window_popup(mainwin->win.sep_message.messagewin);
 		break;
 	case SEPARATE_BOTH:
-		widget = mainwin->win.sep_both.folderwin;
-		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x = 0;
-		y %= sy; if (y < 0) y = 0;
-		gdk_window_move(widget->window, x, y);
-		gdk_window_raise(widget->window);
-		widget = mainwin->win.sep_both.messagewin;
-		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x = 0;
-		y %= sy; if (y < 0) y = 0;
-		gdk_window_move(widget->window, x, y);
-		gdk_window_raise(widget->window);
+		gtkut_window_popup(mainwin->win.sep_both.folderwin);
+		gtkut_window_popup(mainwin->win.sep_both.messagewin);
 		break;
 	default:
 		break;
@@ -1594,7 +1634,7 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	GtkItemFactory *ifactory = mainwin->menu_factory;
 	GtkWidget *menuitem;
 
-	debug_print(_("Setting widgets..."));
+	debug_print("Setting widgets... ");
 
 	/* create separated window(s) if needed */
 	if (type & SEPARATE_FOLDER) {
@@ -1605,13 +1645,18 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 				       "folder_view", "Sylpheed");
 		gtk_window_set_policy(GTK_WINDOW(folderwin),
 				      TRUE, TRUE, FALSE);
-		gtk_widget_set_usize(folderwin, -1,
-				     prefs_common.mainview_height);
+		gtk_widget_set_uposition(folderwin, prefs_common.folderwin_x,
+					 prefs_common.folderwin_y);
 		gtk_container_set_border_width(GTK_CONTAINER(folderwin),
 					       BORDER_WIDTH);
 		gtk_signal_connect(GTK_OBJECT(folderwin), "delete_event",
 				   GTK_SIGNAL_FUNC(folder_window_close_cb),
 				   mainwin);
+		gtk_container_add(GTK_CONTAINER(folderwin),
+				  GTK_WIDGET_PTR(mainwin->folderview));
+		gtk_widget_realize(folderwin);
+		if (prefs_common.folderview_visible)
+			gtk_widget_show(folderwin);
 	}
 	if (type & SEPARATE_MESSAGE) {
 		messagewin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -1621,25 +1666,38 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 				       "message_view", "Sylpheed");
 		gtk_window_set_policy(GTK_WINDOW(messagewin),
 				      TRUE, TRUE, FALSE);
-		gtk_widget_set_usize
-			(messagewin, prefs_common.mainview_width,
-			 prefs_common.mainview_height
-			 - prefs_common.summaryview_height
-			 + DEFAULT_HEADERVIEW_HEIGHT);
+		gtk_widget_set_uposition(messagewin, prefs_common.main_msgwin_x,
+					 prefs_common.main_msgwin_y);
 		gtk_container_set_border_width(GTK_CONTAINER(messagewin),
 					       BORDER_WIDTH);
 		gtk_signal_connect(GTK_OBJECT(messagewin), "delete_event",
 				   GTK_SIGNAL_FUNC(message_window_close_cb),
 				   mainwin);
+		gtk_container_add(GTK_CONTAINER(messagewin),
+				  GTK_WIDGET_PTR(mainwin->messageview));
+		gtk_widget_realize(messagewin);
+		if (messageview_is_visible(mainwin->messageview))
+			gtk_widget_show(messagewin);
 	}
 
+	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->folderview),
+			     prefs_common.folderview_width,
+			     prefs_common.folderview_height);
+	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
+			     prefs_common.summaryview_width,
+			     prefs_common.summaryview_height);
+	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->messageview),
+			     prefs_common.msgview_width,
+			     prefs_common.msgview_height);
+
 	switch (type) {
 	case SEPARATE_NONE:
 		hpaned = gtk_hpaned_new();
-		gtk_widget_show(hpaned);
 		gtk_box_pack_start(GTK_BOX(vbox_body), hpaned, TRUE, TRUE, 0);
 		gtk_paned_add1(GTK_PANED(hpaned),
 			       GTK_WIDGET_PTR(mainwin->folderview));
+		gtk_widget_show(hpaned);
+		gtk_widget_queue_resize(hpaned);
 
 		vpaned = gtk_vpaned_new();
 		if (messageview_is_visible(mainwin->messageview)) {
@@ -1651,18 +1709,10 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 				       GTK_WIDGET_PTR(mainwin->summaryview));
 			gtk_widget_ref(vpaned);
 		}
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
-				     prefs_common.summaryview_width,
-				     prefs_common.summaryview_height);
 		gtk_paned_add2(GTK_PANED(vpaned),
 			       GTK_WIDGET_PTR(mainwin->messageview));
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->messageview),
-				     prefs_common.mainview_width, -1);
-		gtk_widget_set_usize(mainwin->window,
-				     prefs_common.folderview_width +
-				     prefs_common.mainview_width,
-				     prefs_common.mainwin_height);
-		gtk_widget_show_all(vpaned);
+		gtk_widget_show(vpaned);
+		gtk_widget_queue_resize(vpaned);
 
 		mainwin->win.sep_none.hpaned = hpaned;
 		mainwin->win.sep_none.vpaned = vpaned;
@@ -1682,82 +1732,60 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		}
 		gtk_paned_add2(GTK_PANED(vpaned),
 			       GTK_WIDGET_PTR(mainwin->messageview));
-		gtk_widget_show_all(vpaned);
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
-				     prefs_common.summaryview_width,
-				     prefs_common.summaryview_height);
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->messageview),
-				     prefs_common.mainview_width, -1);
-		gtk_widget_set_usize(mainwin->window,
-				     prefs_common.mainview_width,
-				     prefs_common.mainview_height);
-
-		gtk_container_add(GTK_CONTAINER(folderwin),
-				  GTK_WIDGET_PTR(mainwin->folderview));
+		gtk_widget_show(vpaned);
+		gtk_widget_queue_resize(vpaned);
 
 		mainwin->win.sep_folder.folderwin = folderwin;
 		mainwin->win.sep_folder.vpaned    = vpaned;
 
-		gtk_widget_show_all(folderwin);
 		break;
 	case SEPARATE_MESSAGE:
 		hpaned = gtk_hpaned_new();
 		gtk_box_pack_start(GTK_BOX(vbox_body), hpaned, TRUE, TRUE, 0);
-
 		gtk_paned_add1(GTK_PANED(hpaned),
 			       GTK_WIDGET_PTR(mainwin->folderview));
 		gtk_paned_add2(GTK_PANED(hpaned),
 			       GTK_WIDGET_PTR(mainwin->summaryview));
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
-				     prefs_common.summaryview_width,
-				     prefs_common.summaryview_height);
-		gtk_widget_set_usize(mainwin->window,
-				     prefs_common.folderview_width +
-				     prefs_common.mainview_width,
-				     prefs_common.mainwin_height);
-		gtk_widget_show_all(hpaned);
-		gtk_container_add(GTK_CONTAINER(messagewin),
-				  GTK_WIDGET_PTR(mainwin->messageview));
+		gtk_widget_show(hpaned);
+		gtk_widget_queue_resize(hpaned);
 
 		mainwin->win.sep_message.messagewin = messagewin;
 		mainwin->win.sep_message.hpaned     = hpaned;
 
-		gtk_widget_show_all(messagewin);
 		break;
 	case SEPARATE_BOTH:
 		gtk_box_pack_start(GTK_BOX(vbox_body),
 				   GTK_WIDGET_PTR(mainwin->summaryview),
 				   TRUE, TRUE, 0);
-		gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
-				     prefs_common.summaryview_width,
-				     prefs_common.summaryview_height);
-		gtk_widget_set_usize(mainwin->window,
-				     prefs_common.mainview_width,
-				     prefs_common.mainwin_height);
-		gtk_container_add(GTK_CONTAINER(folderwin),
-				  GTK_WIDGET_PTR(mainwin->folderview));
-		gtk_container_add(GTK_CONTAINER(messagewin),
-				  GTK_WIDGET_PTR(mainwin->messageview));
 
 		mainwin->win.sep_both.folderwin = folderwin;
 		mainwin->win.sep_both.messagewin = messagewin;
 
-		gtk_widget_show_all(folderwin);
-		gtk_widget_show_all(messagewin);
 		break;
 	}
 
-	mainwin->type = type;
+	gtk_widget_set_uposition(mainwin->window,
+				 prefs_common.mainwin_x,
+				 prefs_common.mainwin_y);
 
-	mainwin->messageview->visible = TRUE;
+	gtk_widget_queue_resize(vbox_body);
+	gtk_widget_queue_resize(mainwin->vbox);
+	gtk_widget_queue_resize(mainwin->window);
+
+	mainwin->type = type;
 
 	/* toggle menu state */
 	menuitem = gtk_item_factory_get_item
 		(ifactory, "/View/Show or hide/Folder tree");
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), TRUE);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+				       (type & SEPARATE_FOLDER) == 0 ? TRUE :
+				       prefs_common.folderview_visible);
+	gtk_widget_set_sensitive(menuitem, ((type & SEPARATE_FOLDER) != 0));
 	menuitem = gtk_item_factory_get_item
 		(ifactory, "/View/Show or hide/Message view");
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), TRUE);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+				       (type & SEPARATE_MESSAGE) == 0 ? TRUE :
+				       prefs_common.msgview_visible);
 
 	menuitem = gtk_item_factory_get_item
 		(ifactory, "/View/Separate folder tree");
@@ -1768,7 +1796,20 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
 				       ((type & SEPARATE_MESSAGE) != 0));
 
-	debug_print(_("done.\n"));
+	if (folderwin) {
+		gtk_signal_connect
+			(GTK_OBJECT(folderwin), "size_allocate",
+			 GTK_SIGNAL_FUNC(folder_window_size_allocate_cb),
+			 mainwin);
+	}
+	if (messagewin) {
+		gtk_signal_connect
+			(GTK_OBJECT(messagewin), "size_allocate",
+			 GTK_SIGNAL_FUNC(message_window_size_allocate_cb),
+			 mainwin);
+	}
+
+	debug_print("done.\n");
 }
 
 static void main_window_toolbar_create(MainWindow *mainwin,
@@ -2117,6 +2158,33 @@ static gint message_window_close_cb(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
+static void main_window_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation,
+					 gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+
+	main_window_get_size(mainwin);
+}
+
+static void folder_window_size_allocate_cb(GtkWidget *widget,
+					   GtkAllocation *allocation,
+					   gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+
+	main_window_get_size(mainwin);
+}
+
+static void message_window_size_allocate_cb(GtkWidget *widget,
+					    GtkAllocation *allocation,
+					    gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+
+	main_window_get_size(mainwin);
+}
+
 static void add_mailbox_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -2226,6 +2294,8 @@ static void toggle_folder_cb(MainWindow *mainwin, guint action,
 			gtk_widget_hide(mainwin->win.sep_both.folderwin);
 		break;
 	}
+
+	prefs_common.folderview_visible = active;
 }
 
 static void toggle_message_cb(MainWindow *mainwin, guint action,
@@ -2316,7 +2386,7 @@ static void inc_mail_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 static void inc_all_account_mail_cb(MainWindow *mainwin, guint action,
 				    GtkWidget *widget)
 {
-	inc_all_account_mail(mainwin);
+	inc_all_account_mail(mainwin, FALSE);
 }
 
 static void inc_cancel_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -2332,12 +2402,18 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 		Folder *folder = list->data;
 
 		if (folder->queue) {
-			if (procmsg_send_queue
-				(folder->queue, prefs_common.savemsg) < 0)
-				alertpanel_error(_("Some errors occurred while sending queued messages."));
+			gint ret;
+
+			ret = procmsg_send_queue(folder->queue,
+						 prefs_common.savemsg);
 			statusbar_pop_all();
-			folder_item_scan(folder->queue);
-			folderview_update_item(folder->queue, TRUE);
+			if (ret >= 0) {
+				folder_item_scan(folder->queue);
+				folderview_update_item(folder->queue, TRUE);
+				if (prefs_common.savemsg && folder->outbox)
+					folderview_update_item(folder->outbox,
+							       TRUE);
+			}
 		}
 	}
 }
@@ -2345,18 +2421,18 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 static void compose_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	PrefsAccount *ac = NULL;
+	FolderItem *item = mainwin->summaryview->folder_item;
 
-	if (mainwin->summaryview->folder_item) {
-		ac = mainwin->summaryview->folder_item->folder->account;
-		if (ac && ac->protocol == A_NNTP) {
-			compose_new(ac,
-				    mainwin->summaryview->folder_item->path,
-				    NULL);
+	if (item) {
+		ac = account_find_from_item(item);
+		if (ac && ac->protocol == A_NNTP &&
+		    FOLDER_TYPE(item->folder) == F_NEWS) {
+			compose_new(ac, item, item->path, NULL);
 			return;
 		}
 	}
 
-	compose_new(ac, NULL, NULL);
+	compose_new(ac, item, NULL, NULL);
 }
 
 static void reply_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -2442,13 +2518,16 @@ static void set_charset_cb(MainWindow *mainwin, guint action,
 {
 	const gchar *str;
 
-	str = conv_get_charset_str((CharSet)action);
-	g_free(prefs_common.force_charset);
-	prefs_common.force_charset = str ? g_strdup(str) : NULL;
+	if (GTK_CHECK_MENU_ITEM(widget)->active) {
+		str = conv_get_charset_str((CharSet)action);
+		g_free(prefs_common.force_charset);
+		prefs_common.force_charset = str ? g_strdup(str) : NULL;
 
-	summary_redisplay_msg(mainwin->summaryview);
+		summary_redisplay_msg(mainwin->summaryview);
 
-	debug_print(_("forced charset: %s\n"), str ? str : "Auto-Detect");
+		debug_print("forced charset: %s\n",
+			    str ? str : "Auto-Detect");
+	}
 }
 
 static void thread_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -2490,7 +2569,8 @@ static void sort_summary_cb(MainWindow *mainwin, guint action,
 	GtkWidget *menuitem;
 
 	if (mainwin->menu_lock_count) return;
-	if (item) {
+
+	if (GTK_CHECK_MENU_ITEM(widget)->active && item) {
 		menuitem = gtk_item_factory_get_item
 			(mainwin->menu_factory, "/View/Sort/Ascending");
 		summary_sort(mainwin->summaryview, (FolderSortKey)action,
@@ -2505,7 +2585,8 @@ static void sort_summary_type_cb(MainWindow *mainwin, guint action,
 	FolderItem *item = mainwin->summaryview->folder_item;
 
 	if (mainwin->menu_lock_count) return;
-	if (item)
+
+	if (GTK_CHECK_MENU_ITEM(widget)->active && item)
 		summary_sort(mainwin->summaryview,
 			     item->sort_key, (FolderSortType)action);
 }
@@ -2654,7 +2735,7 @@ static void prefs_common_open_cb(MainWindow *mainwin, guint action,
 static void prefs_filter_open_cb(MainWindow *mainwin, guint action,
 				 GtkWidget *widget)
 {
-	prefs_filter_open(NULL, NULL);
+	prefs_filter_open(NULL, NULL, NULL);
 }
 
 static void prefs_template_open_cb(MainWindow *mainwin, guint action,
@@ -2675,14 +2756,7 @@ static void prefs_account_open_cb(MainWindow *mainwin, guint action,
 	if (!cur_account) {
 		new_account_cb(mainwin, 0, widget);
 	} else {
-		gboolean prev_default = cur_account->is_default;
-
-		prefs_account_open(cur_account);
-		if (!prev_default && cur_account->is_default)
-			account_set_as_default(cur_account);
-		account_save_config_all();
-		account_set_menu();
-		main_window_reflect_prefs_all();
+		account_open(cur_account);
 	}
 }
 
