@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenushell.h>
+#include <gtk/gtkoptionmenu.h>
 
 #define MENUITEM_ADD(menu, menuitem, label, data) \
 { \
@@ -49,6 +50,12 @@ GtkWidget *menu_create_items	(GtkItemFactoryEntry	*entries,
 				 const gchar		*path,
 				 GtkItemFactory	       **factory,
 				 gpointer		 data);
+
+GString *menu_factory_get_rc	(const gchar		*path);
+void menu_factory_clear_rc	(const gchar		*rc_str);
+void menu_factory_copy_rc	(const gchar		*src_path,
+				 const gchar		*dest_path);
+
 void menu_set_sensitive		(GtkItemFactory		*ifactory,
 				 const gchar		*path,
 				 gboolean		 sensitive);
@@ -60,4 +67,8 @@ void menu_button_position	(GtkMenu		*menu,
 				 gint			*y,
 				 gpointer		 user_data);
 
+gint menu_find_option_menu_index(GtkOptionMenu		*optmenu,
+				 gpointer		 data,
+				 GCompareFunc		 func);
+
 #endif /* __MENU_H__ */
