@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,23 +43,124 @@
 #include "procheader.h"
 #include "utils.h"
 
-static void	mh_folder_init			(Folder		*folder,
-						 const gchar	*name,
-						 const gchar	*path);
+static void	mh_folder_init		(Folder		*folder,
+					 const gchar	*name,
+					 const gchar	*path);
+
+static Folder	*mh_folder_new		(const gchar	*name,
+					 const gchar	*path);
+static void     mh_folder_destroy	(Folder		*folder);
+
+static GSList  *mh_get_msg_list		(Folder		*folder,
+					 FolderItem	*item,
+					 gboolean	 use_cache);
+static gchar   *mh_fetch_msg		(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+static MsgInfo *mh_get_msginfo		(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+static gint     mh_add_msg		(Folder		*folder,
+					 FolderItem	*dest,
+					 const gchar	*file,
+					 MsgFlags	*flags,
+					 gboolean	 remove_source);
+static gint     mh_add_msgs		(Folder		*folder,
+					 FolderItem	*dest,
+					 GSList		*file_list,
+					 gboolean	 remove_source,
+					 gint		*first);
+static gint     mh_move_msg		(Folder		*folder,
+					 FolderItem	*dest,
+					 MsgInfo	*msginfo);
+static gint     mh_move_msgs		(Folder		*folder,
+					 FolderItem	*dest,
+					 GSList		*msglist);
+static gint     mh_copy_msg		(Folder		*folder,
+					 FolderItem	*dest,
+					 MsgInfo	*msginfo);
+static gint     mh_copy_msgs		(Folder		*folder,
+					 FolderItem	*dest,
+					 GSList		*msglist);
+static gint     mh_remove_msg		(Folder		*folder,
+					 FolderItem	*item,
+					 MsgInfo	*msginfo);
+static gint     mh_remove_all_msg	(Folder		*folder,
+					 FolderItem	*item);
+static gboolean mh_is_msg_changed	(Folder		*folder,
+					 FolderItem	*item,
+					 MsgInfo	*msginfo);
+static gint    mh_close			(Folder		*folder,
+					 FolderItem	*item);
+
+static gint    mh_scan_folder		(Folder		*folder,
+					 FolderItem	*item);
+static gint    mh_scan_tree		(Folder		*folder);
+
+static gint    mh_create_tree		(Folder		*folder);
+static FolderItem *mh_create_folder	(Folder		*folder,
+					 FolderItem	*parent,
+					 const gchar	*name);
+static gint    mh_rename_folder		(Folder		*folder,
+					 FolderItem	*item,
+					 const gchar	*name);
+static gint    mh_remove_folder		(Folder		*folder,
+					 FolderItem	*item);
 
 static gchar   *mh_get_new_msg_filename		(FolderItem	*dest);
 
+static gint	mh_do_move_msgs			(Folder		*folder,
+						 FolderItem	*dest,
+						 GSList		*msglist);
+
 static GSList  *mh_get_uncached_msgs		(GHashTable	*msg_table,
 						 FolderItem	*item);
 static MsgInfo *mh_parse_msg			(const gchar	*file,
 						 FolderItem	*item);
+static void	mh_remove_missing_folder_items	(Folder		*folder);
 static void	mh_scan_tree_recursive		(FolderItem	*item);
 
 static gboolean mh_rename_folder_func		(GNode		*node,
 						 gpointer	 data);
 
+static FolderClass mh_class =
+{
+	F_MH,
 
-Folder *mh_folder_new(const gchar *name, const gchar *path)
+	mh_folder_new,
+	mh_folder_destroy,
+
+	mh_scan_tree,
+	mh_create_tree,
+
+	mh_get_msg_list,
+	mh_fetch_msg,
+	mh_get_msginfo,
+	mh_add_msg,
+	mh_add_msgs,
+	mh_move_msg,
+	mh_move_msgs,
+	mh_copy_msg,
+	mh_copy_msgs,
+	mh_remove_msg,
+	NULL,
+	mh_remove_all_msg,
+	mh_is_msg_changed,
+	mh_close,
+	mh_scan_folder,
+
+	mh_create_folder,
+	mh_rename_folder,
+	mh_remove_folder,
+};
+
+
+FolderClass *mh_get_class(void)
+{
+	return &mh_class;
+}
+
+static Folder *mh_folder_new(const gchar *name, const gchar *path)
 {
 	Folder *folder;
 
@@ -69,37 +170,19 @@ Folder *mh_folder_new(const gchar *name, const gchar *path)
 	return folder;
 }
 
-void mh_folder_destroy(Folder *folder)
+static void mh_folder_destroy(Folder *folder)
 {
 	folder_local_folder_destroy(LOCAL_FOLDER(folder));
 }
 
 static void mh_folder_init(Folder *folder, const gchar *name, const gchar *path)
 {
+	folder->klass = mh_get_class();
 	folder_local_folder_init(folder, name, path);
-
-	folder->type = F_MH;
-
-	folder->get_msg_list        = mh_get_msg_list;
-	folder->fetch_msg           = mh_fetch_msg;
-	folder->add_msg             = mh_add_msg;
-	folder->move_msg            = mh_move_msg;
-	folder->move_msgs_with_dest = mh_move_msgs_with_dest;
-	folder->copy_msg            = mh_copy_msg;
-	folder->copy_msgs_with_dest = mh_copy_msgs_with_dest;
-	folder->remove_msg          = mh_remove_msg;
-	folder->remove_all_msg      = mh_remove_all_msg;
-	folder->is_msg_changed      = mh_is_msg_changed;
-	folder->scan                = mh_scan_folder;
-	folder->scan_tree           = mh_scan_tree;
-	folder->create_tree         = mh_create_tree;
-	folder->create_folder       = mh_create_folder;
-	folder->rename_folder       = mh_rename_folder;
-	folder->remove_folder       = mh_remove_folder;
-	folder->destroy             = mh_folder_destroy;
 }
 
-GSList *mh_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
+static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
+			       gboolean use_cache)
 {
 	GSList *mlist;
 	GHashTable *msg_table;
@@ -149,6 +232,8 @@ GSList *mh_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 
 	procmsg_set_flags(mlist, item);
 
+	mlist = procmsg_sort_msg_list(mlist, item->sort_key, item->sort_type);
+
 #ifdef MEASURE_TIME
 	gettimeofday(&tv_after, NULL);
 
@@ -160,13 +245,20 @@ GSList *mh_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 	return mlist;
 }
 
-gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
+static gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gchar *path;
 	gchar *file;
 
 	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(num > 0 && num <= item->last_num, NULL);
+	g_return_val_if_fail(num > 0, NULL);
+
+	if (item->last_num < 0 || num > item->last_num) {
+		mh_scan_folder(folder, item);
+		if (item->last_num < 0) return NULL;
+	}
+
+	g_return_val_if_fail(num <= item->last_num, NULL);
 
 	path = folder_item_get_path(item);
 	file = g_strconcat(path, G_DIR_SEPARATOR_S, itos(num), NULL);
@@ -179,6 +271,26 @@ gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	return file;
 }
 
+static MsgInfo *mh_get_msginfo(Folder *folder, FolderItem *item, gint num)
+{
+	MsgInfo *msginfo;
+	gchar *file;
+
+	g_return_val_if_fail(item != NULL, NULL);
+	g_return_val_if_fail(num > 0, NULL);
+
+	file = mh_fetch_msg(folder, item, num);
+	if (!file) return NULL;
+
+	msginfo = mh_parse_msg(file, item);
+	if (msginfo)
+		msginfo->msgnum = num;
+
+	g_free(file);
+
+	return msginfo;
+}
+
 static gchar *mh_get_new_msg_filename(FolderItem *dest)
 {
 	gchar *destfile;
@@ -205,151 +317,117 @@ static gchar *mh_get_new_msg_filename(FolderItem *dest)
 	return destfile;
 }
 
-#define SET_DEST_MSG_FLAGS(fp, dest, msginfo) \
-{ \
-	MsgInfo newmsginfo; \
- \
-	newmsginfo.msgnum = dest->last_num; \
-	newmsginfo.flags = msginfo->flags; \
-	if (dest->stype == F_OUTBOX || \
-	    dest->stype == F_QUEUE  || \
-	    dest->stype == F_DRAFT  || \
-	    dest->stype == F_TRASH) \
-		MSG_UNSET_PERM_FLAGS(newmsginfo.flags, \
-				     MSG_NEW|MSG_UNREAD|MSG_DELETED); \
- \
-	procmsg_write_flags(&newmsginfo, fp); \
+#define SET_DEST_MSG_FLAGS(fp, dest, n, fl)				\
+{									\
+	MsgInfo newmsginfo;						\
+									\
+	newmsginfo.msgnum = n;						\
+	newmsginfo.flags = fl;						\
+	if (dest->stype == F_OUTBOX ||					\
+	    dest->stype == F_QUEUE  ||					\
+	    dest->stype == F_DRAFT  ||					\
+	    dest->stype == F_TRASH)					\
+		MSG_UNSET_PERM_FLAGS(newmsginfo.flags,			\
+				     MSG_NEW|MSG_UNREAD|MSG_DELETED);	\
+									\
+	if (fp)								\
+		procmsg_write_flags(&newmsginfo, fp);			\
+	else if (dest->opened)						\
+		procmsg_add_flags(dest, n, newmsginfo.flags);		\
 }
 
-gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file,
-		gboolean remove_source)
+static gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file,
+		       MsgFlags *flags, gboolean remove_source)
 {
-	gchar *destfile;
+	GSList file_list;
+	MsgFileInfo fileinfo;
 
-	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(file != NULL, -1);
 
-	if (dest->last_num < 0) {
-		mh_scan_folder(folder, dest);
-		if (dest->last_num < 0) return -1;
-	}
+	fileinfo.file = (gchar *)file;
+	fileinfo.flags = flags;
+	file_list.data = &fileinfo;
+	file_list.next = NULL;
 
-	destfile = mh_get_new_msg_filename(dest);
-	g_return_val_if_fail(destfile != NULL, -1);
-
-	if (link(file, destfile) < 0) {
-		if (copy_file(file, destfile, TRUE) < 0) {
-			g_warning(_("can't copy message %s to %s\n"),
-				  file, destfile);
-			g_free(destfile);
-			return -1;
-		}
-	}
-
-	if (remove_source) {
-		if (unlink(file) < 0)
-			FILE_OP_ERROR(file, "unlink");
-	}
-
-	g_free(destfile);
-	dest->last_num++;
-	return dest->last_num;
+	return mh_add_msgs(folder, dest, &file_list, remove_source, NULL);
 }
 
-static gint mh_do_move(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
+			gboolean remove_source, gint *first)
 {
-	gchar *destdir;
-	gchar *srcfile;
 	gchar *destfile;
+	GSList *cur;
+	MsgFileInfo *fileinfo;
+	gint first_ = 0;
 	FILE *fp;
 
 	g_return_val_if_fail(dest != NULL, -1);
-	g_return_val_if_fail(msginfo != NULL, -1);
-
-	if (msginfo->folder == dest) {
-		g_warning(_("the src folder is identical to the dest.\n"));
-		return -1;
-	}
+	g_return_val_if_fail(file_list != NULL, -1);
 
 	if (dest->last_num < 0) {
 		mh_scan_folder(folder, dest);
 		if (dest->last_num < 0) return -1;
 	}
 
-	destfile = mh_get_new_msg_filename(dest);
-	g_return_val_if_fail(destfile != NULL, -1);
+	if ((((MsgFileInfo *)file_list->data)->flags == NULL &&
+	    file_list->next == NULL) || dest->opened)
+		fp = NULL;
+	else if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
+		g_warning("Can't open mark file.\n");
 
-	debug_print(_("Moving message %s%c%d to %s ...\n"),
-		    msginfo->folder->path, G_DIR_SEPARATOR,
-		    msginfo->msgnum, dest->path);
+	for (cur = file_list; cur != NULL; cur = cur->next) {
+		fileinfo = (MsgFileInfo *)cur->data;
 
-	srcfile = procmsg_get_message_file(msginfo);
+		destfile = mh_get_new_msg_filename(dest);
+		if (destfile == NULL) return -1;
+		if (first_ == 0 || first_ > dest->last_num + 1)
+			first_ = dest->last_num + 1;
+
+		if (link(fileinfo->file, destfile) < 0) {
+			if (copy_file(fileinfo->file, destfile, TRUE) < 0) {
+				g_warning(_("can't copy message %s to %s\n"),
+					  fileinfo->file, destfile);
+				g_free(destfile);
+				return -1;
+			}
+		}
 
-	if (move_file(srcfile, destfile, FALSE) < 0) {
-		g_free(srcfile);
 		g_free(destfile);
-		return -1;
+		dest->last_num++;
+		dest->total++;
+
+		if (fileinfo->flags) {
+			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
+					   *fileinfo->flags);
+			if (MSG_IS_NEW(*fileinfo->flags))
+				dest->new++;
+			if (MSG_IS_UNREAD(*fileinfo->flags))
+				dest->unread++;
+		} else {
+			dest->new++;
+			dest->unread++;
+		}
 	}
 
-	g_free(srcfile);
-	g_free(destfile);
-	dest->last_num++;
+	if (fp) fclose(fp);
 
-	destdir = folder_item_get_path(dest);
+	if (first)
+		*first = first_;
 
-	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
-		g_warning(_("Can't open mark file.\n"));
-	else {
-		SET_DEST_MSG_FLAGS(fp, dest, msginfo);
-		fclose(fp);
+	if (remove_source) {
+		for (cur = file_list; cur != NULL; cur = cur->next) {
+			fileinfo = (MsgFileInfo *)cur->data;
+			if (unlink(fileinfo->file) < 0)
+				FILE_OP_ERROR(fileinfo->file, "unlink");
+		}
 	}
-	g_free(destdir);
 
 	return dest->last_num;
 }
 
-gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 {
-	gchar *srcfile;
-	gint ret = 0;
-
-	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(dest != NULL, -1);
-	g_return_val_if_fail(msginfo != NULL, -1);
-	g_return_val_if_fail(msginfo->folder != NULL, -1);
-
-	if (folder == msginfo->folder->folder)
-		return mh_do_move(folder, dest, msginfo);
-
-	srcfile = procmsg_get_message_file(msginfo);
-	if (!srcfile) return -1;
-
-	ret = mh_add_msg(folder, dest, srcfile, FALSE);
-	g_free(srcfile);
-
-	if (ret != -1) {
-		gchar *destdir;
-		FILE *fp;
-
-		destdir = folder_item_get_path(dest);
-		if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
-			g_warning(_("Can't open mark file.\n"));
-		else {
-			SET_DEST_MSG_FLAGS(fp, dest, msginfo);
-			fclose(fp);
-		}
-		g_free(destdir);
-
-		ret = folder_item_remove_msg(msginfo->folder, msginfo->msgnum);
-	}
-
-	return ret;
-}
-
-static gint mh_do_move_msgs_with_dest(Folder *folder, FolderItem *dest,
-				      GSList *msglist)
-{
-	gchar *destdir;
+	FolderItem *src;
 	gchar *srcfile;
 	gchar *destfile;
 	FILE *fp;
@@ -364,20 +442,22 @@ static gint mh_do_move_msgs_with_dest(Folder *folder, FolderItem *dest,
 		if (dest->last_num < 0) return -1;
 	}
 
-	destdir = folder_item_get_path(dest);
-	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
+	if (dest->opened)
+		fp = NULL;
+	else if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
 		g_warning(_("Can't open mark file.\n"));
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
+		src = msginfo->folder;
 
-		if (msginfo->folder == dest) {
+		if (src == dest) {
 			g_warning(_("the src folder is identical to the dest.\n"));
 			continue;
 		}
-		debug_print(_("Moving message %s%c%d to %s ...\n"),
-			    msginfo->folder->path, G_DIR_SEPARATOR,
-			    msginfo->msgnum, dest->path);
+		debug_print("Moving message %s%c%d to %s ...\n",
+			    src->path, G_DIR_SEPARATOR, msginfo->msgnum,
+			    dest->path);
 
 		destfile = mh_get_new_msg_filename(dest);
 		if (!destfile) break;
@@ -391,94 +471,80 @@ static gint mh_do_move_msgs_with_dest(Folder *folder, FolderItem *dest,
 
 		g_free(srcfile);
 		g_free(destfile);
+		src->total--;
 		dest->last_num++;
+		dest->total++;
 
 		if (fp) {
-			SET_DEST_MSG_FLAGS(fp, dest, msginfo);
+			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
+					   msginfo->flags);
+		}
+
+		if (MSG_IS_NEW(msginfo->flags)) {
+			src->new--;
+			dest->new++;
+		}
+		if (MSG_IS_UNREAD(msginfo->flags)) {
+			src->unread--;
+			dest->unread++;
 		}
 	}
 
-	g_free(destdir);
 	if (fp) fclose(fp);
 
 	return dest->last_num;
 }
 
-gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
+static gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+{
+	GSList msglist;
+
+	g_return_val_if_fail(msginfo != NULL, -1);
+
+	msglist.data = msginfo;
+	msglist.next = NULL;
+
+	return mh_move_msgs(folder, dest, &msglist);
+}
+
+static gint mh_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 {
 	MsgInfo *msginfo;
-	GSList *cur;
+	GSList *file_list;
 	gint ret = 0;
+	gint first;
 
 	msginfo = (MsgInfo *)msglist->data;
 	if (folder == msginfo->folder->folder)
-		return mh_do_move_msgs_with_dest(folder, dest, msglist);
+		return mh_do_move_msgs(folder, dest, msglist);
 
-	for (cur = msglist; cur != NULL; cur = cur->next) {
-		msginfo = (MsgInfo *)cur->data;
-		ret = mh_move_msg(folder, dest, msginfo);
-		if (ret == -1) break;
-	}
+	file_list = procmsg_get_message_file_list(msglist);
+	g_return_val_if_fail(file_list != NULL, -1);
+
+	ret = mh_add_msgs(folder, dest, file_list, FALSE, &first);
+
+	procmsg_message_file_list_free(file_list);
+
+	if (ret != -1)
+		ret = folder_item_remove_msgs(msginfo->folder, msglist);
 
 	return ret;
 }
 
-gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+static gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 {
-	gchar *destdir;
-	gchar *srcfile;
-	gchar *destfile;
-	FILE *fp;
+	GSList msglist;
 
-	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
 
-	if (msginfo->folder == dest) {
-		g_warning(_("the src folder is identical to the dest.\n"));
-		return -1;
-	}
+	msglist.data = msginfo;
+	msglist.next = NULL;
 
-	if (dest->last_num < 0) {
-		mh_scan_folder(folder, dest);
-		if (dest->last_num < 0) return -1;
-	}
-
-	destfile = mh_get_new_msg_filename(dest);
-	g_return_val_if_fail(destfile != NULL, -1);
-
-	debug_print(_("Copying message %s%c%d to %s ...\n"),
-		    msginfo->folder->path, G_DIR_SEPARATOR,
-		    msginfo->msgnum, dest->path);
-
-	srcfile = procmsg_get_message_file(msginfo);
-
-	if (copy_file(srcfile, destfile, TRUE) < 0) {
-		FILE_OP_ERROR(srcfile, "copy");
-		g_free(srcfile);
-		g_free(destfile);
-		return -1;
-	}
-
-	g_free(srcfile);
-	g_free(destfile);
-	dest->last_num++;
-
-	destdir = folder_item_get_path(dest);
-
-	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
-		g_warning(_("Can't open mark file.\n"));
-	else {
-		SET_DEST_MSG_FLAGS(fp, dest, msginfo);
-		fclose(fp);
-	}
-	g_free(destdir);
-
-	return dest->last_num;
+	return mh_copy_msgs(folder, dest, &msglist);
 }
 
-gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
+static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 {
-	gchar *destdir;
 	gchar *srcfile;
 	gchar *destfile;
 	FILE *fp;
@@ -493,8 +559,9 @@ gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 		if (dest->last_num < 0) return -1;
 	}
 
-	destdir = folder_item_get_path(dest);
-	if ((fp = procmsg_open_mark_file(destdir, TRUE)) == NULL)
+	if (dest->opened)
+		fp = NULL;
+	if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
 		g_warning(_("Can't open mark file.\n"));
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
@@ -522,25 +589,31 @@ gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 		g_free(srcfile);
 		g_free(destfile);
 		dest->last_num++;
+		dest->total++;
 
 		if (fp) {
-			SET_DEST_MSG_FLAGS(fp, dest, msginfo);
+			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
+					   msginfo->flags);
 		}
+
+		if (MSG_IS_NEW(msginfo->flags))
+			dest->new++;
+		if (MSG_IS_UNREAD(msginfo->flags))
+			dest->unread++;
 	}
 
-	g_free(destdir);
 	if (fp) fclose(fp);
 
 	return dest->last_num;
 }
 
-gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
+static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 {
 	gchar *file;
 
 	g_return_val_if_fail(item != NULL, -1);
 
-	file = mh_fetch_msg(folder, item, num);
+	file = mh_fetch_msg(folder, item, msginfo->msgnum);
 	g_return_val_if_fail(file != NULL, -1);
 
 	if (unlink(file) < 0) {
@@ -549,11 +622,17 @@ gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 		return -1;
 	}
 
+	item->total--;
+	if (MSG_IS_NEW(msginfo->flags))
+		item->new--;
+	if (MSG_IS_UNREAD(msginfo->flags))
+		item->unread--;
+
 	g_free(file);
 	return 0;
 }
 
-gint mh_remove_all_msg(Folder *folder, FolderItem *item)
+static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 {
 	gchar *path;
 	gint val;
@@ -564,11 +643,14 @@ gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 	g_return_val_if_fail(path != NULL, -1);
 	val = remove_all_numbered_files(path);
 	g_free(path);
+	if (val == 0)
+		item->new = item->unread = item->total = 0;
 
 	return val;
 }
 
-gboolean mh_is_msg_changed(Folder *folder, FolderItem *item, MsgInfo *msginfo)
+static gboolean mh_is_msg_changed(Folder *folder, FolderItem *item,
+				  MsgInfo *msginfo)
 {
 	struct stat s;
 
@@ -580,7 +662,12 @@ gboolean mh_is_msg_changed(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 	return FALSE;
 }
 
-gint mh_scan_folder(Folder *folder, FolderItem *item)
+static gint mh_close(Folder *folder, FolderItem *item)
+{
+	return 0;
+}
+
+static gint mh_scan_folder(Folder *folder, FolderItem *item)
 {
 	gchar *path;
 	DIR *dp;
@@ -627,7 +714,8 @@ gint mh_scan_folder(Folder *folder, FolderItem *item)
 	else {
 		gint new, unread, total, min, max;
 
-		procmsg_get_mark_sum(".", &new, &unread, &total, &min, &max, 0);
+		procmsg_get_mark_sum
+			(item, &new, &unread, &total, &min, &max, 0);
 		if (n_msg > total) {
 			new += n_msg - total;
 			unread += n_msg - total;
@@ -643,27 +731,32 @@ gint mh_scan_folder(Folder *folder, FolderItem *item)
 	return 0;
 }
 
-void mh_scan_tree(Folder *folder)
+static gint mh_scan_tree(Folder *folder)
 {
 	FolderItem *item;
 	gchar *rootpath;
 
-	g_return_if_fail(folder != NULL);
+	g_return_val_if_fail(folder != NULL, -1);
 
-	folder_tree_destroy(folder);
-	item = folder_item_new(folder->name, NULL);
-	item->folder = folder;
-	folder->node = g_node_new(item);
+	if (!folder->node) {
+		item = folder_item_new(folder->name, NULL);
+		item->folder = folder;
+		folder->node = item->node = g_node_new(item);
+	} else
+		item = FOLDER_ITEM(folder->node->data);
 
 	rootpath = folder_item_get_path(item);
 	if (change_dir(rootpath) < 0) {
 		g_free(rootpath);
-		return;
+		return -1;
 	}
 	g_free(rootpath);
 
 	mh_create_tree(folder);
+	mh_remove_missing_folder_items(folder);
 	mh_scan_tree_recursive(item);
+
+	return 0;
 }
 
 #define MAKE_DIR_IF_NOT_EXIST(dir) \
@@ -679,7 +772,7 @@ void mh_scan_tree(Folder *folder)
 	} \
 }
 
-gint mh_create_tree(Folder *folder)
+static gint mh_create_tree(Folder *folder)
 {
 	gchar *rootpath;
 
@@ -700,8 +793,8 @@ gint mh_create_tree(Folder *folder)
 
 #undef MAKE_DIR_IF_NOT_EXIST
 
-FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
-			     const gchar *name)
+static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
+				    const gchar *name)
 {
 	gchar *path;
 	gchar *fullpath;
@@ -734,12 +827,12 @@ FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 	return new_item;
 }
 
-gint mh_rename_folder(Folder *folder, FolderItem *item, const gchar *name)
+static gint mh_rename_folder(Folder *folder, FolderItem *item,
+			     const gchar *name)
 {
 	gchar *oldpath;
 	gchar *dirname;
 	gchar *newpath;
-	GNode *node;
 	gchar *paths[2];
 
 	g_return_val_if_fail(folder != NULL, -1);
@@ -772,11 +865,9 @@ gint mh_rename_folder(Folder *folder, FolderItem *item, const gchar *name)
 	g_free(item->name);
 	item->name = g_strdup(name);
 
-	node = g_node_find(item->folder->node, G_PRE_ORDER, G_TRAVERSE_ALL,
-			   item);
 	paths[0] = g_strdup(item->path);
 	paths[1] = newpath;
-	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+	g_node_traverse(item->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			mh_rename_folder_func, paths);
 
 	g_free(paths[0]);
@@ -784,7 +875,7 @@ gint mh_rename_folder(Folder *folder, FolderItem *item, const gchar *name)
 	return 0;
 }
 
-gint mh_remove_folder(Folder *folder, FolderItem *item)
+static gint mh_remove_folder(Folder *folder, FolderItem *item)
 {
 	gchar *path;
 
@@ -832,7 +923,7 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 		return NULL;
 	}
 
-	debug_print(_("\tSearching uncached messages... "));
+	debug_print("Searching uncached messages...\n");
 
 	if (msg_table) {
 		while ((d = readdir(dp)) != NULL) {
@@ -887,16 +978,16 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 	closedir(dp);
 
 	if (n_newmsg)
-		debug_print(_("%d uncached message(s) found.\n"), n_newmsg);
+		debug_print("%d uncached message(s) found.\n", n_newmsg);
 	else
-		debug_print(_("done.\n"));
+		debug_print("done.\n");
 
 	/* sort new messages in numerical order */
-	if (newlist) {
-		debug_print(_("\tSorting uncached messages in numerical order... "));
+	if (newlist && item->sort_key == SORT_BY_NONE) {
+		debug_print("Sorting uncached messages in numerical order...\n");
 		newlist = g_slist_sort
 			(newlist, (GCompareFunc)procmsg_cmp_msgnum_for_sort);
-		debug_print(_("done.\n"));
+		debug_print("done.\n");
 	}
 
 	return newlist;
@@ -964,8 +1055,41 @@ static gboolean mh_is_maildir(const gchar *path)
 }
 #endif
 
+static gboolean mh_remove_missing_folder_items_func(GNode *node, gpointer data)
+{
+	FolderItem *item;
+	gchar *path;
+
+	g_return_val_if_fail(node->data != NULL, FALSE);
+
+	if (G_NODE_IS_ROOT(node))
+		return FALSE;
+
+	item = FOLDER_ITEM(node->data);
+
+	path = folder_item_get_path(item);
+	if (!is_dir_exist(path)) {
+		debug_print("folder '%s' not found. removing...\n", path);
+		folder_item_remove(item);
+	}
+	g_free(path);
+
+	return FALSE;
+}
+
+static void mh_remove_missing_folder_items(Folder *folder)
+{
+	g_return_if_fail(folder != NULL);
+
+	debug_print("searching missing folders...\n");
+
+	g_node_traverse(folder->node, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+			mh_remove_missing_folder_items_func, folder);
+}
+
 static void mh_scan_tree_recursive(FolderItem *item)
 {
+	Folder *folder;
 	DIR *dp;
 	struct dirent *d;
 	struct stat s;
@@ -975,6 +1099,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	folder = item->folder;
+
 	dp = opendir(item->path ? item->path : ".");
 	if (!dp) {
 		FILE_OP_ERROR(item->path ? item->path : ".", "opendir");
@@ -984,9 +1110,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	debug_print("scanning %s ...\n",
 		    item->path ? item->path
 		    : LOCAL_FOLDER(item->folder)->rootpath);
-	if (item->folder->ui_func)
-		item->folder->ui_func(item->folder, item,
-				      item->folder->ui_func_data);
+	if (folder->ui_func)
+		folder->ui_func(folder, item, folder->ui_func_data);
 
 	while ((d = readdir(dp)) != NULL) {
 		if (d->d_name[0] == '.') continue;
@@ -1004,7 +1129,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		}
 
 		if (S_ISDIR(s.st_mode)) {
-			FolderItem *new_item;
+			FolderItem *new_item = NULL;
+			GNode *node;
 
 #if 0
 			if (mh_is_maildir(entry)) {
@@ -1013,26 +1139,44 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			}
 #endif
 
-			new_item = folder_item_new(d->d_name, entry);
-			folder_item_append(item, new_item);
-			if (!item->path) {
-				if (!strcmp(d->d_name, INBOX_DIR)) {
-					new_item->stype = F_INBOX;
-					item->folder->inbox = new_item;
-				} else if (!strcmp(d->d_name, OUTBOX_DIR)) {
-					new_item->stype = F_OUTBOX;
-					item->folder->outbox = new_item;
-				} else if (!strcmp(d->d_name, DRAFT_DIR)) {
-					new_item->stype = F_DRAFT;
-					item->folder->draft = new_item;
-				} else if (!strcmp(d->d_name, QUEUE_DIR)) {
-					new_item->stype = F_QUEUE;
-					item->folder->queue = new_item;
-				} else if (!strcmp(d->d_name, TRASH_DIR)) {
-					new_item->stype = F_TRASH;
-					item->folder->trash = new_item;
+			node = item->node;
+			for (node = node->children; node != NULL; node = node->next) {
+				FolderItem *cur_item = FOLDER_ITEM(node->data);
+				if (!strcmp2(cur_item->path, entry)) {
+					new_item = cur_item;
+					break;
 				}
 			}
+			if (!new_item) {
+				debug_print("new folder '%s' found.\n", entry);
+				new_item = folder_item_new(d->d_name, entry);
+				folder_item_append(item, new_item);
+			}
+
+			if (!item->path) {
+				if (!folder->inbox &&
+				    !strcmp(d->d_name, INBOX_DIR)) {
+					new_item->stype = F_INBOX;
+					folder->inbox = new_item;
+				} else if (!folder->outbox &&
+					   !strcmp(d->d_name, OUTBOX_DIR)) {
+					new_item->stype = F_OUTBOX;
+					folder->outbox = new_item;
+				} else if (!folder->draft &&
+					   !strcmp(d->d_name, DRAFT_DIR)) {
+					new_item->stype = F_DRAFT;
+					folder->draft = new_item;
+				} else if (!folder->queue &&
+					   !strcmp(d->d_name, QUEUE_DIR)) {
+					new_item->stype = F_QUEUE;
+					folder->queue = new_item;
+				} else if (!folder->trash &&
+					   !strcmp(d->d_name, TRASH_DIR)) {
+					new_item->stype = F_TRASH;
+					folder->trash = new_item;
+				}
+			}
+
 			mh_scan_tree_recursive(new_item);
 		} else if (to_number(d->d_name) != -1) n_msg++;
 
@@ -1044,8 +1188,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	if (item->path) {
 		gint new, unread, total, min, max;
 
-		procmsg_get_mark_sum(item->path, &new, &unread, &total,
-				     &min, &max, 0);
+		procmsg_get_mark_sum
+			(item, &new, &unread, &total, &min, &max, 0);
 		if (n_msg > total) {
 			new += n_msg - total;
 			unread += n_msg - total;
