@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -219,6 +219,7 @@ MimeView *mimeview_create(void)
 
 void mimeview_init(MimeView *mimeview)
 {
+	textview_init(mimeview->textview);
 }
 
 /* 
@@ -240,8 +241,7 @@ static gboolean mimeview_is_signed(MimeView *mimeview)
 
         debug_print("mimeview_is_signed: file\n" );
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	g_return_val_if_fail(partinfo != NULL, FALSE);
 
 	/* walk the tree and see whether there is a signature somewhere */
@@ -276,7 +276,6 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 {
 	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
 	GtkCTreeNode *node;
-	gchar buf[BUFFSIZE];
 	FILE *fp;
 
 	mimeview_clear(mimeview);
@@ -287,25 +286,17 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 
 	mimeview->mimeinfo = mimeinfo;
 
-	if ((fp = fopen(file, "rb")) == NULL) {
-		FILE_OP_ERROR(file, "fopen");
-		return;
-	}
 	mimeview->file = g_strdup(file);
 
-	/* skip MIME part headers */
-	if (mimeinfo->mime_type == MIME_MULTIPART) {
-		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
-			perror("fseek");
-		while (fgets(buf, sizeof(buf), fp) != NULL)
-			if (buf[0] == '\r' || buf[0] == '\n') break;
-	}
-
-	procmime_scan_multipart_message(mimeinfo, fp);
 #if USE_GPGME
-	if (prefs_common.auto_check_signatures)
+	if (prefs_common.auto_check_signatures) {
+		if ((fp = fopen(file, "rb")) == NULL) {
+			FILE_OP_ERROR(file, "fopen");
+			return;
+		}
 		rfc2015_check_signature(mimeinfo, fp);
-	else
+		fclose(fp);
+	} else
 		set_unchecked_signature(mimeinfo);
 #endif
 
@@ -328,7 +319,6 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 		     partinfo->mime_type == MIME_TEXT_HTML))
 			break;
 	}
-	fclose(fp);
 	textview_show_message(mimeview->messageview->textview, mimeinfo, file);
 
 	if (!node)
@@ -348,33 +338,38 @@ void mimeview_destroy(MimeView *mimeview)
 	g_free(mimeview);
 }
 
+MimeInfo *mimeview_get_selected_part(MimeView *mimeview)
+{
+	if (gtk_notebook_get_current_page
+		(GTK_NOTEBOOK(mimeview->notebook)) == 0)
+		return NULL;
+
+	return gtk_ctree_node_get_row_data
+		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+}
+
 static void mimeview_set_multipart_tree(MimeView *mimeview,
 					MimeInfo *mimeinfo,
 					GtkCTreeNode *parent)
 {
+	GtkCTreeNode *node;
+
 	g_return_if_fail(mimeinfo != NULL);
 
-	if (!mimeinfo->main && mimeinfo->parent)
-		parent = mimeview_append_part(mimeview, mimeinfo, parent);
+	if (mimeinfo->children)
+		mimeinfo = mimeinfo->children;
 
-	if (mimeinfo->sub && !mimeinfo->sub->children &&
-	    mimeinfo->sub->mime_type != MIME_TEXT &&
-	    mimeinfo->sub->mime_type != MIME_TEXT_HTML) {
-		mimeview_append_part(mimeview, mimeinfo->sub, parent);
-		return;
-	}
+	while (mimeinfo != NULL) {
+		node = mimeview_append_part(mimeview, mimeinfo, parent);
 
-	if (mimeinfo->sub && mimeinfo->sub->children)
-		mimeview_set_multipart_tree(mimeview, mimeinfo->sub, parent);
-
-	if (mimeinfo->children) {
-		MimeInfo *child;
-
-		child = mimeinfo->children;
-		while (child) {
-			mimeview_set_multipart_tree(mimeview, child, parent);
-			child = child->next;
-		}
+		if (mimeinfo->children)
+			mimeview_set_multipart_tree(mimeview, mimeinfo, node);
+		else if (mimeinfo->sub &&
+			 mimeinfo->sub->mime_type != MIME_TEXT &&
+			 mimeinfo->sub->mime_type != MIME_TEXT_HTML)
+			mimeview_set_multipart_tree(mimeview, mimeinfo->sub,
+						    node);
+		mimeinfo = mimeinfo->next;
 	}
 }
 
@@ -463,7 +458,8 @@ static void mimeview_show_image_part(MimeView *mimeview, MimeInfo *partinfo)
 			(_("Can't get the part of multipart message."));
 	else {
 		mimeview_change_view_type(mimeview, MIMEVIEW_IMAGE);
-		imageview_show_image(mimeview->imageview, partinfo, filename);
+		imageview_show_image(mimeview->imageview, partinfo, filename,
+				     prefs_common.resize_image);
 		unlink(filename);
 	}
 
@@ -571,11 +567,10 @@ static void mimeview_start_drag(GtkWidget *widget, gint button,
 	GtkTargetList *list;
 	GdkDragContext *context;
 	MimeInfo *partinfo;
-	GtkCTree *ctree;
 
 	g_return_if_fail(mimeview != NULL);
-	ctree = GTK_CTREE(mimeview->ctree);
-	partinfo = gtk_ctree_node_get_row_data(ctree, mimeview->opened);
+
+	partinfo = mimeview_get_selected_part(mimeview);
 	if (partinfo->filename == NULL && partinfo->name == NULL) return;
 
 	list = gtk_target_list_new(mimeview_mime_types, 1);
@@ -607,8 +602,7 @@ static gint mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 		/* call external program for image, audio or html */
 		mimeview_launch(mimeview);
 	} else if (event->button == 3) {
-		partinfo = gtk_ctree_node_get_row_data
-			(GTK_CTREE(mimeview->ctree), mimeview->opened);
+		partinfo = mimeview_get_selected_part(mimeview);
 		if (partinfo && (partinfo->mime_type == MIME_TEXT ||
 				 partinfo->mime_type == MIME_TEXT_HTML ||
 				 partinfo->mime_type == MIME_MESSAGE_RFC822 ||
@@ -740,8 +734,7 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) return;
 	if (!partinfo->filename && !partinfo->name) return;
 
@@ -770,8 +763,8 @@ static void mimeview_display_as_text(MimeView *mimeview)
 
 	if (!mimeview->opened) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
+	g_return_if_fail(partinfo != NULL);
 	mimeview_show_message_part(mimeview, partinfo);
 }
 
@@ -784,8 +777,7 @@ static void mimeview_save_as(MimeView *mimeview)
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	g_return_if_fail(partinfo != NULL);
 
 	if (partinfo->filename)
@@ -819,8 +811,7 @@ static void mimeview_launch(MimeView *mimeview)
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	g_return_if_fail(partinfo != NULL);
 
 	filename = procmime_get_tmp_file_name(partinfo);
@@ -843,8 +834,7 @@ static void mimeview_open_with(MimeView *mimeview)
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	g_return_if_fail(partinfo != NULL);
 
 	filename = procmime_get_tmp_file_name(partinfo);
@@ -883,8 +873,7 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 {
 	static gchar *default_image_cmdline = "display '%s'";
 	static gchar *default_audio_cmdline = "play '%s'";
-	static gchar *default_html_cmdline =
-		"netscape -remote 'openURL(%s,raise)'";
+	static gchar *default_html_cmdline = DEFAULT_BROWSER_CMD;
 	static gchar *mime_cmdline = "metamail -d -b -x -c %s '%s'";
 	gchar buf[1024];
 	gchar m_buf[1024];
@@ -956,8 +945,7 @@ static void mimeview_update_signature_info(MimeView *mimeview)
 	if (!mimeview) return;
 	if (!mimeview->opened) return;
 
-	partinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) return;
 
 	if (g_strcasecmp(partinfo->content_type,
@@ -974,8 +962,7 @@ static void mimeview_check_signature(MimeView *mimeview)
 
 	g_return_if_fail (mimeview_is_signed(mimeview));
 
-	mimeinfo = gtk_ctree_node_get_row_data
-		(GTK_CTREE(mimeview->ctree), mimeview->opened);
+	mimeinfo = mimeview_get_selected_part(mimeview);
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(mimeview->file != NULL);
 
