@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,6 +60,26 @@ static void news_folder_init		 (Folder	*folder,
 					  const gchar	*name,
 					  const gchar	*path);
 
+static Folder	*news_folder_new	(const gchar	*name,
+					 const gchar	*folder);
+static void	 news_folder_destroy	(Folder		*folder);
+
+static GSList *news_get_article_list	(Folder		*folder,
+					 FolderItem	*item,
+					 gboolean	 use_cache);
+static gchar *news_fetch_msg		(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+static MsgInfo *news_get_msginfo	(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+
+static gint news_close			(Folder		*folder,
+					 FolderItem	*item);
+
+static gint news_scan_group		(Folder		*folder,
+					 FolderItem	*item);
+
 #if USE_SSL
 static Session *news_session_new	 (const gchar	*server,
 					  gushort	 port,
@@ -80,9 +100,11 @@ static gint news_get_article_cmd	 (NNTPSession	*session,
 static gint news_get_article		 (NNTPSession	*session,
 					  gint		 num,
 					  gchar		*filename);
+#if 0
 static gint news_get_header		 (NNTPSession	*session,
 					  gint		 num,
 					  gchar		*filename);
+#endif
 
 static gint news_select_group		 (NNTPSession	*session,
 					  const gchar	*group,
@@ -104,8 +126,44 @@ static void news_delete_all_articles	 (FolderItem	*item);
 static void news_delete_expired_caches	 (GSList	*alist,
 					  FolderItem	*item);
 
+static FolderClass news_class =
+{
+	F_NEWS,
 
-Folder *news_folder_new(const gchar *name, const gchar *path)
+	news_folder_new,
+	news_folder_destroy,
+
+	NULL,
+	NULL,
+
+	news_get_article_list,
+	news_fetch_msg,
+	news_get_msginfo,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	news_close,
+	news_scan_group,
+
+	NULL,
+	NULL,
+	NULL
+};
+
+
+FolderClass *news_get_class(void)
+{
+	return &news_class;
+}
+
+static Folder *news_folder_new(const gchar *name, const gchar *path)
 {
 	Folder *folder;
 
@@ -115,7 +173,7 @@ Folder *news_folder_new(const gchar *name, const gchar *path)
 	return folder;
 }
 
-void news_folder_destroy(Folder *folder)
+static void news_folder_destroy(Folder *folder)
 {
 	gchar *dir;
 
@@ -130,14 +188,8 @@ void news_folder_destroy(Folder *folder)
 static void news_folder_init(Folder *folder, const gchar *name,
 			     const gchar *path)
 {
+	folder->klass = news_get_class();
 	folder_remote_folder_init(folder, name, path);
-
-	folder->type = F_NEWS;
-
-	folder->get_msg_list = news_get_article_list;
-	folder->fetch_msg    = news_fetch_msg;
-	folder->scan         = news_scan_group;
-	folder->destroy      = news_folder_destroy;
 }
 
 #if USE_SSL
@@ -150,53 +202,19 @@ static Session *news_session_new(const gchar *server, gushort port,
 #endif
 {
 	gchar buf[NNTPBUFSIZE];
-	NNTPSession *session;
-	NNTPSockInfo *nntp_sock;
+	Session *session;
 
 	g_return_val_if_fail(server != NULL, NULL);
 
 	log_message(_("creating NNTP connection to %s:%d ...\n"), server, port);
 
 #if USE_SSL
-	if (userid && passwd)
-		nntp_sock = nntp_open_auth(server, port, buf, userid, passwd,
-					   ssl_type);
-	else
-		nntp_sock = nntp_open(server, port, buf, ssl_type);
+	session = nntp_session_new(server, port, buf, userid, passwd, ssl_type);
 #else
-	if (userid && passwd)
-		nntp_sock = nntp_open_auth(server, port, buf, userid, passwd);
-	else
-		nntp_sock = nntp_open(server, port, buf);
+	session = nntp_session_new(server, port, buf, userid, passwd);
 #endif
 
-	if (nntp_sock == NULL)
-		return NULL;
-
-	session = g_new(NNTPSession, 1);
-	SESSION(session)->type             = SESSION_NEWS;
-	SESSION(session)->server           = g_strdup(server);
-	session->nntp_sock                 = nntp_sock;
-	SESSION(session)->sock             = nntp_sock->sock;
-	SESSION(session)->connected        = TRUE;
-	SESSION(session)->phase            = SESSION_READY;
-	SESSION(session)->last_access_time = time(NULL);
-	SESSION(session)->data             = NULL;
-
-	SESSION(session)->destroy          = news_session_destroy;
-
-	session->group = NULL;
-
-	return SESSION(session);
-}
-
-void news_session_destroy(Session *session)
-{
-	nntp_close(NNTP_SESSION(session)->nntp_sock);
-	NNTP_SESSION(session)->nntp_sock = NULL;
-	session->sock = NULL;
-
-	g_free(NNTP_SESSION(session)->group);
+	return session;
 }
 
 static Session *news_session_new_for_folder(Folder *folder)
@@ -235,12 +253,12 @@ static Session *news_session_new_for_folder(Folder *folder)
 	return session;
 }
 
-NNTPSession *news_session_get(Folder *folder)
+static NNTPSession *news_session_get(Folder *folder)
 {
 	RemoteFolder *rfolder = REMOTE_FOLDER(folder);
 
 	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
 	if (!rfolder->session) {
@@ -255,7 +273,7 @@ NNTPSession *news_session_get(Folder *folder)
 		return NNTP_SESSION(rfolder->session);
 	}
 
-	if (nntp_mode(NNTP_SESSION(rfolder->session)->nntp_sock, FALSE)
+	if (nntp_mode(NNTP_SESSION(rfolder->session), FALSE)
 	    != NN_SUCCESS) {
 		log_warning(_("NNTP connection to %s:%d has been"
 			      " disconnected. Reconnecting...\n"),
@@ -272,15 +290,15 @@ NNTPSession *news_session_get(Folder *folder)
 	return NNTP_SESSION(rfolder->session);
 }
 
-GSList *news_get_article_list(Folder *folder, FolderItem *item,
-			      gboolean use_cache)
+static GSList *news_get_article_list(Folder *folder, FolderItem *item,
+				     gboolean use_cache)
 {
 	GSList *alist;
 	NNTPSession *session;
 
 	g_return_val_if_fail(folder != NULL, NULL);
 	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, NULL);
 
 	session = news_session_get(folder);
 
@@ -320,12 +338,14 @@ GSList *news_get_article_list(Folder *folder, FolderItem *item,
 
 	procmsg_set_flags(alist, item);
 
+	alist = procmsg_sort_msg_list(alist, item->sort_key, item->sort_type);
+
 	statusbar_pop_all();
 
 	return alist;
 }
 
-gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
+static gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gchar *path, *filename;
 	NNTPSession *session;
@@ -372,13 +392,36 @@ gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	return filename;
 }
 
-gint news_scan_group(Folder *folder, FolderItem *item)
+static MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
+{
+	MsgInfo *msginfo;
+	MsgFlags flags = {0, 0};
+	gchar *file;
+
+	g_return_val_if_fail(folder != NULL, NULL);
+	g_return_val_if_fail(item != NULL, NULL);
+
+	file = news_fetch_msg(folder, item, num);
+	if (!file) return NULL;
+
+	msginfo = procheader_parse_file(file, flags, FALSE);
+
+	g_free(file);
+
+	return msginfo;
+}
+
+static gint news_close(Folder *folder, FolderItem *item)
+{
+	return 0;
+}
+
+static gint news_scan_group(Folder *folder, FolderItem *item)
 {
 	NNTPSession *session;
 	gint num = 0, first = 0, last = 0;
 	gint new = 0, unread = 0, total = 0;
 	gint min = 0, max = 0;
-	gchar *path;
 	gint ok;
 
 	g_return_val_if_fail(folder != NULL, -1);
@@ -398,12 +441,7 @@ gint news_scan_group(Folder *folder, FolderItem *item)
 		return 0;
 	}
 
-	path = folder_item_get_path(item);
-	if (path && is_dir_exist(path)) {
-		procmsg_get_mark_sum(path, &new, &unread, &total, &min, &max,
-				     first);
-	}
-	g_free(path);
+	procmsg_get_mark_sum(item, &new, &unread, &total, &min, &max, first);
 
 	if (max < first || last < min)
 		new = unread = total = num;
@@ -465,7 +503,7 @@ GSList *news_get_group_list(Folder *folder)
 	gchar buf[NNTPBUFSIZE];
 
 	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, NULL);
 
 	path = folder_item_get_path(FOLDER_ITEM(folder->node->data));
 	if (!is_dir_exist(path))
@@ -482,7 +520,7 @@ GSList *news_get_group_list(Folder *folder)
 			return NULL;
 		}
 
-		if (nntp_list(session->nntp_sock) != NN_SUCCESS) {
+		if (nntp_list(session) != NN_SUCCESS) {
 			g_free(filename);
 			statusbar_pop_all();
 			return NULL;
@@ -556,7 +594,7 @@ void news_remove_group_list_cache(Folder *folder)
 	gchar *path, *filename;
 
 	g_return_if_fail(folder != NULL);
-	g_return_if_fail(folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(folder) == F_NEWS);
 
 	path = folder_item_get_path(FOLDER_ITEM(folder->node->data));
 	filename = g_strconcat(path, G_DIR_SEPARATOR_S, NEWSGROUP_LIST, NULL);
@@ -575,7 +613,7 @@ gint news_post(Folder *folder, const gchar *file)
 	gint ok;
 
 	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(folder->type == F_NEWS, -1);
+	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, -1);
 	g_return_val_if_fail(file != NULL, -1);
 
 	if ((fp = fopen(file, "rb")) == NULL) {
@@ -598,13 +636,13 @@ gint news_post_stream(Folder *folder, FILE *fp)
 	gint ok;
 
 	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(folder->type == F_NEWS, -1);
+	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, -1);
 	g_return_val_if_fail(fp != NULL, -1);
 
 	session = news_session_get(folder);
 	if (!session) return -1;
 
-	ok = nntp_post(session->nntp_sock, fp);
+	ok = nntp_post(session, fp);
 	if (ok != NN_SUCCESS) {
 		log_warning(_("can't post article.\n"));
 		return -1;
@@ -618,14 +656,14 @@ static gint news_get_article_cmd(NNTPSession *session, const gchar *cmd,
 {
 	gchar *msgid;
 
-	if (nntp_get_article(session->nntp_sock, cmd, num, &msgid)
+	if (nntp_get_article(session, cmd, num, &msgid)
 	    != NN_SUCCESS)
 		return -1;
 
 	debug_print("Message-Id = %s, num = %d\n", msgid, num);
 	g_free(msgid);
 
-	if (recv_write_to_file(session->nntp_sock->sock, filename) < 0) {
+	if (recv_write_to_file(SESSION(session)->sock, filename) < 0) {
 		log_warning(_("can't retrieve article %d\n"), num);
 		return -1;
 	}
@@ -638,10 +676,12 @@ static gint news_get_article(NNTPSession *session, gint num, gchar *filename)
 	return news_get_article_cmd(session, "ARTICLE", num, filename);
 }
 
+#if 0
 static gint news_get_header(NNTPSession *session, gint num, gchar *filename)
 {
 	return news_get_article_cmd(session, "HEAD", num, filename);
 }
+#endif
 
 /**
  * news_select_group:
@@ -673,7 +713,7 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	g_free(session->group);
 	session->group = NULL;
 
-	ok = nntp_group(session->nntp_sock, group, num, first, last);
+	ok = nntp_group(session, group, num, first, last);
 	if (ok == NN_SUCCESS)
 		session->group = g_strdup(group);
 
@@ -697,7 +737,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 	g_return_val_if_fail(session != NULL, NULL);
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(item->folder != NULL, NULL);
-	g_return_val_if_fail(item->folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_TYPE(item->folder) == F_NEWS, NULL);
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
 	if (ok != NN_SUCCESS) {
@@ -732,7 +772,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 
 	log_message(_("getting xover %d - %d in %s...\n"),
 		    begin, end, item->path);
-	if (nntp_xover(session->nntp_sock, begin, end) != NN_SUCCESS) {
+	if (nntp_xover(session, begin, end) != NN_SUCCESS) {
 		log_warning(_("can't get xover\n"));
 		return NULL;
 	}
@@ -764,7 +804,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		}
 	}
 
-	if (nntp_xhdr(session->nntp_sock, "to", begin, end) != NN_SUCCESS) {
+	if (nntp_xhdr(session, "to", begin, end) != NN_SUCCESS) {
 		log_warning(_("can't get xhdr\n"));
 		return newlist;
 	}
@@ -789,7 +829,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		llast = llast->next;
 	}
 
-	if (nntp_xhdr(session->nntp_sock, "cc", begin, end) != NN_SUCCESS) {
+	if (nntp_xhdr(session, "cc", begin, end) != NN_SUCCESS) {
 		log_warning(_("can't get xhdr\n"));
 		return newlist;
 	}
@@ -917,7 +957,7 @@ static GSList *news_delete_old_articles(GSList *alist, FolderItem *item,
 
 	g_return_val_if_fail(item != NULL, alist);
 	g_return_val_if_fail(item->folder != NULL, alist);
-	g_return_val_if_fail(item->folder->type == F_NEWS, alist);
+	g_return_val_if_fail(FOLDER_TYPE(item->folder) == F_NEWS, alist);
 
 	if (first < 2) return alist;
 
@@ -948,7 +988,7 @@ static void news_delete_all_articles(FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 
 	debug_print("Deleting all cached articles...\n");
 
@@ -963,7 +1003,7 @@ static void news_delete_expired_caches(GSList *alist, FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 
 	debug_print("Deleting expired cached articles...\n");
 
