@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,8 @@
 #include "gtkutils.h"
 #include "utils.h"
 #include "alertpanel.h"
+#include "smtp.h"
+#include "imap.h"
 
 static gboolean cancelled;
 
@@ -89,6 +91,9 @@ static struct Receive {
 	GtkWidget *inbox_entry;
 	GtkWidget *inbox_btn;
 
+	GtkWidget *imap_frame;
+	GtkWidget *imap_auth_type_optmenu;
+
 	GtkWidget *recvatgetall_chkbtn;
 } receive;
 
@@ -102,10 +107,11 @@ static struct Send {
 	GtkWidget *smtp_auth_type_optmenu;
 	GtkWidget *smtp_uid_entry;
 	GtkWidget *smtp_pass_entry;
-	GtkWidget *pop_bfr_smtp_chkbtn;
+	/* GtkWidget *pop_bfr_smtp_chkbtn; */
 } p_send;
 
 static struct Compose {
+	GtkWidget *sigfile_radiobtn;
 	GtkWidget *sigpath_entry;
 
 	GtkWidget *autocc_chkbtn;
@@ -149,6 +155,8 @@ static struct SSLPrefs {
 	GtkWidget *smtp_nossl_radiobtn;
 	GtkWidget *smtp_ssltunnel_radiobtn;
 	GtkWidget *smtp_starttls_radiobtn;
+
+	GtkWidget *use_nonblocking_ssl_chkbtn;
 } ssl;
 #endif /* USE_SSL */
 
@@ -182,14 +190,15 @@ static void prefs_account_protocol_set_data_from_optmenu(PrefParam *pparam);
 static void prefs_account_protocol_set_optmenu		(PrefParam *pparam);
 static void prefs_account_protocol_activated		(GtkMenuItem *menuitem);
 
+static void prefs_account_imap_auth_type_set_data_from_optmenu
+							(PrefParam *pparam);
+static void prefs_account_imap_auth_type_set_optmenu	(PrefParam *pparam);
 static void prefs_account_smtp_auth_type_set_data_from_optmenu
 							(PrefParam *pparam);
 static void prefs_account_smtp_auth_type_set_optmenu	(PrefParam *pparam);
 
-#if USE_GPGME || USE_SSL
 static void prefs_account_enum_set_data_from_radiobtn	(PrefParam *pparam);
 static void prefs_account_enum_set_radiobtn		(PrefParam *pparam);
-#endif /* USE_GPGME || USE_SSL */
 
 #if USE_GPGME
 static void prefs_account_ascii_armored_warning		(GtkWidget *widget);
@@ -264,6 +273,11 @@ static PrefParam param[] = {
 	 &receive.filter_on_recv_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
+	{"imap_auth_method", "0", &tmp_ac_prefs.imap_auth_type, P_ENUM,
+	 &receive.imap_auth_type_optmenu,
+	 prefs_account_imap_auth_type_set_data_from_optmenu,
+	 prefs_account_imap_auth_type_set_optmenu},
+
 	{"receive_at_get_all", "TRUE", &tmp_ac_prefs.recv_at_getall, P_BOOL,
 	 &receive.recvatgetall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -296,10 +310,17 @@ static PrefParam param[] = {
 	 &p_send.smtp_pass_entry, prefs_set_data_from_entry, prefs_set_entry},
 
 	{"pop_before_smtp", "FALSE", &tmp_ac_prefs.pop_before_smtp, P_BOOL,
+	 NULL, NULL, NULL},
+#if 0
 	 &p_send.pop_bfr_smtp_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+#endif
 
 	/* Compose */
+	{"signature_type", "0", &tmp_ac_prefs.sig_type, P_ENUM,
+	 &compose.sigfile_radiobtn,
+	 prefs_account_enum_set_data_from_radiobtn,
+	 prefs_account_enum_set_radiobtn},
 	{"signature_path", "~/"DEFAULT_SIGNATURE, &tmp_ac_prefs.sig_path, P_STRING,
 	 &compose.sigpath_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
@@ -369,6 +390,10 @@ static PrefParam param[] = {
 	 &ssl.smtp_nossl_radiobtn,
 	 prefs_account_enum_set_data_from_radiobtn,
 	 prefs_account_enum_set_radiobtn},
+
+	{"use_nonblocking_ssl", "1", &tmp_ac_prefs.use_nonblocking_ssl, P_BOOL,
+	 &ssl.use_nonblocking_ssl_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 #endif /* USE_SSL */
 
 	/* Advanced */
@@ -500,17 +525,6 @@ void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 	prefs_custom_header_read_config(ac_prefs);
 }
 
-void prefs_account_save_config(PrefsAccount *ac_prefs)
-{
-	gchar *buf;
-
-	g_return_if_fail(ac_prefs != NULL);
-
-	tmp_ac_prefs = *ac_prefs;
-	buf = g_strdup_printf("Account: %d", ac_prefs->account_id);
-	prefs_save_config(param, buf, ACCOUNT_RC);
-}
-
 void prefs_account_save_config_all(GList *account_list)
 {
 	GList *cur;
@@ -518,7 +532,7 @@ void prefs_account_save_config_all(GList *account_list)
 	PrefFile *pfile;
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACCOUNT_RC, NULL);
-	if ((pfile = prefs_write_open(rcpath)) == NULL) {
+	if ((pfile = prefs_file_open(rcpath)) == NULL) {
 		g_free(rcpath);
 		return;
 	}
@@ -528,21 +542,21 @@ void prefs_account_save_config_all(GList *account_list)
 		tmp_ac_prefs = *(PrefsAccount *)cur->data;
 		if (fprintf(pfile->fp, "[Account: %d]\n",
 			    tmp_ac_prefs.account_id) <= 0 ||
-		    prefs_write_param(param, pfile->fp) < 0) {
+		    prefs_file_write_param(pfile, param) < 0) {
 			g_warning(_("failed to write configuration to file\n"));
-			prefs_write_close_revert(pfile);
+			prefs_file_close_revert(pfile);
 			return;
 		}
 		if (cur->next) {
 			if (fputc('\n', pfile->fp) == EOF) {
 				FILE_OP_ERROR(rcpath, "fputc");
-				prefs_write_close_revert(pfile);
+				prefs_file_close_revert(pfile);
 				return;
 			}
 		}
 	}
 
-	if (prefs_write_close(pfile) < 0)
+	if (prefs_file_close(pfile) < 0)
 		g_warning(_("failed to write configuration to file\n"));
 }
 
@@ -947,6 +961,10 @@ static void prefs_account_receive_create(void)
 	GtkWidget *inbox_label;
 	GtkWidget *inbox_entry;
 	GtkWidget *inbox_btn;
+	GtkWidget *imap_frame;
+	GtkWidget *optmenu;
+	GtkWidget *optmenu_menu;
+	GtkWidget *menuitem;
 	GtkWidget *recvatgetall_chkbtn;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1019,7 +1037,7 @@ static void prefs_account_receive_create(void)
 	gtk_widget_set_usize (size_limit_entry, 64, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), size_limit_entry, FALSE, FALSE, 0);
 
-	label = gtk_label_new ("KB");
+	label = gtk_label_new (_("KB"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
 
@@ -1062,6 +1080,33 @@ static void prefs_account_receive_create(void)
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
+	PACK_FRAME (vbox1, imap_frame, _("IMAP4"));
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (imap_frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("Authentication method"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+
+	optmenu = gtk_option_menu_new ();
+	gtk_widget_show (optmenu);
+	gtk_box_pack_start (GTK_BOX (hbox1), optmenu, FALSE, FALSE, 0);
+
+	optmenu_menu = gtk_menu_new ();
+
+	MENUITEM_ADD (optmenu_menu, menuitem, _("Automatic"), 0);
+	MENUITEM_ADD (optmenu_menu, menuitem, "LOGIN", IMAP_AUTH_LOGIN);
+	MENUITEM_ADD (optmenu_menu, menuitem, "CRAM-MD5", IMAP_AUTH_CRAM_MD5);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
+
 	PACK_CHECK_BUTTON
 		(vbox1, recvatgetall_chkbtn,
 		 _("`Get all' checks for new messages on this account"));
@@ -1077,7 +1122,10 @@ static void prefs_account_receive_create(void)
 	receive.inbox_entry           = inbox_entry;
 	receive.inbox_btn             = inbox_btn;
 
-	receive.recvatgetall_chkbtn   = recvatgetall_chkbtn;
+	receive.imap_frame             = imap_frame;
+	receive.imap_auth_type_optmenu = optmenu;
+
+	receive.recvatgetall_chkbtn = recvatgetall_chkbtn;
 }
 
 static void prefs_account_send_create(void)
@@ -1101,7 +1149,7 @@ static void prefs_account_send_create(void)
 	GtkWidget *smtp_uid_entry;
 	GtkWidget *smtp_pass_entry;
 	GtkWidget *vbox_spc;
-	GtkWidget *pop_bfr_smtp_chkbtn;
+	/* GtkWidget *pop_bfr_smtp_chkbtn; */
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -1226,9 +1274,11 @@ static void prefs_account_send_create(void)
 
 	SET_TOGGLE_SENSITIVITY (smtp_auth_chkbtn, vbox4);
 
+#if 0
 	PACK_CHECK_BUTTON (vbox3, pop_bfr_smtp_chkbtn,
 		_("Authenticate with POP3 before sending"));
 	gtk_widget_set_sensitive(pop_bfr_smtp_chkbtn, FALSE);
+#endif
 
 	p_send.date_chkbtn      = date_chkbtn;
 	p_send.msgid_chkbtn     = msgid_chkbtn;
@@ -1238,14 +1288,16 @@ static void prefs_account_send_create(void)
 	p_send.smtp_auth_type_optmenu = optmenu;
 	p_send.smtp_uid_entry         = smtp_uid_entry;
 	p_send.smtp_pass_entry        = smtp_pass_entry;
-	p_send.pop_bfr_smtp_chkbtn    = pop_bfr_smtp_chkbtn;
+	/* p_send.pop_bfr_smtp_chkbtn    = pop_bfr_smtp_chkbtn; */
 }
 
 static void prefs_account_compose_create(void)
 {
 	GtkWidget *vbox1;
-	GtkWidget *hbox;
-	GtkWidget *label;
+	GtkWidget *sig_vbox;
+	GtkWidget *sig_hbox;
+	GtkWidget *sigfile_radiobtn;
+	GtkWidget *sigcmd_radiobtn;
 	GtkWidget *sigpath_entry;
 	GtkWidget *frame;
 	GtkWidget *table;
@@ -1261,17 +1313,35 @@ static void prefs_account_compose_create(void)
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
-	hbox = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, 0);
+	PACK_FRAME (vbox1, frame, _("Signature"));
 
-	label = gtk_label_new (_("Signature file"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	sig_vbox = gtk_vbox_new (FALSE, VSPACING_NARROW_2);
+	gtk_widget_show (sig_vbox);
+	gtk_container_add (GTK_CONTAINER (frame), sig_vbox);
+	gtk_container_set_border_width (GTK_CONTAINER (sig_vbox), 8);
+
+	sig_hbox = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (sig_hbox);
+	gtk_box_pack_start (GTK_BOX (sig_vbox), sig_hbox, FALSE, FALSE, 0);
+
+	sigfile_radiobtn = gtk_radio_button_new_with_label (NULL, _("File"));
+	gtk_widget_show (sigfile_radiobtn);
+	gtk_box_pack_start (GTK_BOX (sig_hbox), sigfile_radiobtn,
+			    FALSE, FALSE, 0);
+	gtk_object_set_user_data (GTK_OBJECT (sigfile_radiobtn),
+				  GINT_TO_POINTER (SIG_FILE));
+
+	sigcmd_radiobtn = gtk_radio_button_new_with_label_from_widget
+		(GTK_RADIO_BUTTON(sigfile_radiobtn), _("Command output"));
+	gtk_widget_show (sigcmd_radiobtn);
+	gtk_box_pack_start (GTK_BOX (sig_hbox), sigcmd_radiobtn,
+			    FALSE, FALSE, 0);
+	gtk_object_set_user_data (GTK_OBJECT (sigcmd_radiobtn),
+				  GINT_TO_POINTER (SIG_COMMAND));
 
 	sigpath_entry = gtk_entry_new ();
 	gtk_widget_show (sigpath_entry);
-	gtk_box_pack_start (GTK_BOX (hbox), sigpath_entry, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (sig_vbox), sigpath_entry, TRUE, TRUE, 0);
 
 	PACK_FRAME (vbox1, frame, _("Automatically set the following addresses"));
 
@@ -1322,7 +1392,8 @@ static void prefs_account_compose_create(void)
 	SET_TOGGLE_SENSITIVITY (autoreplyto_chkbtn, autoreplyto_entry);
 
 
-	compose.sigpath_entry = sigpath_entry;
+	compose.sigfile_radiobtn = sigfile_radiobtn;
+	compose.sigpath_entry    = sigpath_entry;
 
 	compose.autocc_chkbtn      = autocc_chkbtn;
 	compose.autocc_entry       = autocc_entry;
@@ -1487,6 +1558,12 @@ static void prefs_account_ssl_create(void)
 	GtkWidget *smtp_ssltunnel_radiobtn;
 	GtkWidget *smtp_starttls_radiobtn;
 
+	GtkWidget *vbox6;
+	GtkWidget *use_nonblocking_ssl_chkbtn;
+	GtkWidget *hbox;
+	GtkWidget *hbox_spc;
+	GtkWidget *label;
+
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
@@ -1560,6 +1637,27 @@ static void prefs_account_ssl_create(void)
 			     _("Use STARTTLS command to start SSL session"),
 			     SSL_STARTTLS);
 
+	vbox6 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox6);
+	gtk_box_pack_start (GTK_BOX (vbox1), vbox6, FALSE, FALSE, 0);
+
+	PACK_CHECK_BUTTON(vbox6, use_nonblocking_ssl_chkbtn,
+			  _("Use non-blocking SSL"));
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (vbox6), hbox, FALSE, FALSE, 0);
+
+	hbox_spc = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox_spc);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
+	gtk_widget_set_usize (hbox_spc, 16, -1);
+
+	label = gtk_label_new
+		(_("(Turn this off if you have problems in SSL connection)"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
 	ssl.pop_frame               = pop_frame;
 	ssl.pop_nossl_radiobtn      = pop_nossl_radiobtn;
 	ssl.pop_ssltunnel_radiobtn  = pop_ssltunnel_radiobtn;
@@ -1578,6 +1676,8 @@ static void prefs_account_ssl_create(void)
 	ssl.smtp_nossl_radiobtn     = smtp_nossl_radiobtn;
 	ssl.smtp_ssltunnel_radiobtn = smtp_ssltunnel_radiobtn;
 	ssl.smtp_starttls_radiobtn  = smtp_starttls_radiobtn;
+
+	ssl.use_nonblocking_ssl_chkbtn = use_nonblocking_ssl_chkbtn;
 }
 
 #undef CREATE_RADIO_BUTTONS
@@ -1863,7 +1963,6 @@ static void prefs_account_edit_custom_header(void)
 	prefs_custom_header_open(&tmp_ac_prefs);
 }
 
-#if USE_GPGME || USE_SSL
 static void prefs_account_enum_set_data_from_radiobtn(PrefParam *pparam)
 {
 	GtkRadioButton *radiobtn;
@@ -1902,7 +2001,6 @@ static void prefs_account_enum_set_radiobtn(PrefParam *pparam)
 	}
 }
 
-#endif /* USE_GPGME || USE_SSL */
 
 #if USE_GPGME
 static void prefs_account_ascii_armored_warning(GtkWidget *widget)
@@ -1972,6 +2070,41 @@ static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
 	gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));
 }
 
+static void prefs_account_imap_auth_type_set_data_from_optmenu(PrefParam *pparam)
+{
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
+		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+}
+
+static void prefs_account_imap_auth_type_set_optmenu(PrefParam *pparam)
+{
+	IMAPAuthType type = *((IMAPAuthType *)pparam->data);
+	GtkOptionMenu *optmenu = GTK_OPTION_MENU(*pparam->widget);
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
+	switch (type) {
+	case IMAP_AUTH_LOGIN:
+		gtk_option_menu_set_history(optmenu, 1);
+		break;
+	case IMAP_AUTH_CRAM_MD5:
+		gtk_option_menu_set_history(optmenu, 2);
+		break;
+	case 0:
+	default:
+		gtk_option_menu_set_history(optmenu, 0);
+	}
+
+	menu = gtk_option_menu_get_menu(optmenu);
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));
+}
+
 static void prefs_account_smtp_auth_type_set_data_from_optmenu(PrefParam *pparam)
 {
 	GtkWidget *menu;
@@ -2033,7 +2166,8 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_label, active);
 		gtk_widget_set_sensitive(basic.uid_entry,  active);
 		gtk_widget_set_sensitive(basic.pass_entry, active);
-		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_hide(receive.pop3_frame);
+		gtk_widget_hide(receive.imap_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2067,7 +2201,8 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_label, FALSE);
 		gtk_widget_set_sensitive(basic.uid_entry,  FALSE);
 		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
-		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_hide(receive.pop3_frame);
+		gtk_widget_hide(receive.imap_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, FALSE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2101,7 +2236,8 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_label, TRUE);
 		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
-		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_hide(receive.pop3_frame);
+		gtk_widget_show(receive.imap_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2136,7 +2272,8 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_label, TRUE);
 		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
-		gtk_widget_set_sensitive(receive.pop3_frame, TRUE);
+		gtk_widget_show(receive.pop3_frame);
+		gtk_widget_hide(receive.imap_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
