@@ -407,7 +407,7 @@ void prefs_display_header_write_config(void)
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			     DISPLAY_HEADER_RC, NULL);
 
-	if ((pfile = prefs_write_open(rcpath)) == NULL) {
+	if ((pfile = prefs_file_open(rcpath)) == NULL) {
 		g_warning(_("failed to write configuration to file\n"));
 		g_free(rcpath);
 		return;
@@ -422,7 +422,7 @@ void prefs_display_header_write_config(void)
 		if (fputs(dpstr, pfile->fp) == EOF ||
 		    fputc('\n', pfile->fp) == EOF) {
 			FILE_OP_ERROR(rcpath, "fputs || fputc");
-			prefs_write_close_revert(pfile);
+			prefs_file_close_revert(pfile);
 			g_free(rcpath);
 			g_free(dpstr);
 			return;
@@ -432,7 +432,7 @@ void prefs_display_header_write_config(void)
 
 	g_free(rcpath);
 
-	if (prefs_write_close(pfile) < 0) {
+	if (prefs_file_close(pfile) < 0) {
 		g_warning(_("failed to write configuration to file\n"));
 		return;
 	}
