@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include "foldersel.h"
 #include "manage_window.h"
 #include "inc.h"
+#include "procheader.h"
 #include "filter.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -72,6 +73,9 @@ static struct Filter {
 	GtkWidget *notrecv_radiobtn;
 
 	GtkWidget *cond_clist;
+
+	GSList *hdr_list;
+	GHashTable *hdr_table;
 } filter;
 
 /*
@@ -82,20 +86,26 @@ static struct Filter {
  */
 
 /* widget creating functions */
-static void prefs_filter_create		(void);
+static void prefs_filter_create			(void);
 
-static void prefs_filter_set_dialog	(const gchar	*header,
-					 const gchar	*key);
-static void prefs_filter_set_list	(void);
-static gint prefs_filter_clist_set_row	(gint	 row);
+static void prefs_filter_set_dialog		(void);
+static void prefs_filter_set_keyword_entry	(MsgInfo	*msginfo,
+						 const gchar	*header_name,
+						 const gchar	*key);
+static void prefs_filter_set_list		(void);
+static gint prefs_filter_clist_set_row		(gint	 row);
+static Filter *prefs_filter_dialog_to_filter	(void);
 
 /* callback functions */
+static void prefs_filter_combo_select_cb(GtkWidget	*widget);
 static void prefs_filter_select_dest_cb	(void);
 static void prefs_filter_register_cb	(void);
 static void prefs_filter_substitute_cb	(void);
 static void prefs_filter_delete_cb	(void);
+static void prefs_filter_top		(void);
 static void prefs_filter_up		(void);
 static void prefs_filter_down		(void);
+static void prefs_filter_bottom		(void);
 static void prefs_filter_select		(GtkCList	*clist,
 					 gint		 row,
 					 gint		 column,
@@ -116,7 +126,7 @@ static void prefs_filter_key_pressed	(GtkWidget	*widget,
 					 gpointer	 data);
 static void prefs_filter_close		(void);
 
-void prefs_filter_open(const gchar *header, const gchar *key)
+void prefs_filter_open(MsgInfo *msginfo, const gchar *header, const gchar *key)
 {
 	inc_lock();
 
@@ -127,7 +137,8 @@ void prefs_filter_open(const gchar *header, const gchar *key)
 	manage_window_set_transient(GTK_WINDOW(filter.window));
 	gtk_widget_grab_focus(filter.close_btn);
 
-	prefs_filter_set_dialog(header, key);
+	prefs_filter_set_dialog();
+	prefs_filter_set_keyword_entry(msginfo, header, key);
 
 	gtk_widget_show(filter.window);
 }
@@ -150,7 +161,6 @@ static void prefs_filter_create(void)
 	GtkWidget *key_label;
 	GtkWidget *key_entry1;
 	GtkWidget *key_entry2;
-	GtkWidget *pred_label;
 	GtkWidget *pred_combo1;
 	GtkWidget *pred_entry1;
 	GtkWidget *pred_combo2;
@@ -178,12 +188,15 @@ static void prefs_filter_create(void)
 	GtkWidget *cond_clist;
 
 	GtkWidget *btn_vbox;
+	GtkWidget *spc_vbox;
+	GtkWidget *top_btn;
 	GtkWidget *up_btn;
 	GtkWidget *down_btn;
+	GtkWidget *bottom_btn;
 
 	gchar *title[1];
 
-	debug_print(_("Creating filter setting window...\n"));
+	debug_print("Creating filter setting window...\n");
 
 	window = gtk_window_new (GTK_WINDOW_DIALOG);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
@@ -248,23 +261,23 @@ static void prefs_filter_create(void)
 	gtk_widget_show (hdr_combo1);
 	gtk_table_attach (GTK_TABLE (table1), hdr_combo1, 1, 2, 1, 2,
 			  0, 0, 0, 0);
-	gtk_widget_set_usize (hdr_combo1, 96, -1);
-	gtkut_combo_set_items (GTK_COMBO (hdr_combo1),
-			       "Subject", "From", "To", "Cc", "Reply-To",
-			       "Sender", "List-Id",
-			       "X-ML-Name", "X-List", "X-Sequence", "X-Mailer",
-			       NULL);
+	gtk_widget_set_usize (hdr_combo1, 128, -1);
+	gtk_signal_connect_object_after
+		(GTK_OBJECT (GTK_COMBO (hdr_combo1)->list),
+		 "select_child",
+		 GTK_SIGNAL_FUNC (prefs_filter_combo_select_cb),
+		 GTK_OBJECT (hdr_combo1));
 
 	hdr_combo2 = gtk_combo_new ();
 	gtk_widget_show (hdr_combo2);
 	gtk_table_attach (GTK_TABLE (table1), hdr_combo2, 1, 2, 2, 3,
 			  0, 0, 0, 0);
-	gtk_widget_set_usize (hdr_combo2, 96, -1);
-	gtkut_combo_set_items (GTK_COMBO (hdr_combo2), _("(none)"),
-			       "Subject", "From", "To", "Cc", "Reply-To",
-			       "Sender", "List-Id",
-			       "X-ML-Name", "X-List", "X-Sequence", "X-Mailer",
-			       NULL);
+	gtk_widget_set_usize (hdr_combo2, 128, -1);
+	gtk_signal_connect_object_after
+		(GTK_OBJECT (GTK_COMBO (hdr_combo2)->list),
+		 "select_child",
+		 GTK_SIGNAL_FUNC (prefs_filter_combo_select_cb),
+		 GTK_OBJECT(hdr_combo2));
 
 	key_label = gtk_label_new (_("Keyword"));
 	gtk_widget_show (key_label);
@@ -275,24 +288,18 @@ static void prefs_filter_create(void)
 
 	key_entry1 = gtk_entry_new ();
 	gtk_widget_show (key_entry1);
-	gtk_widget_set_usize (key_entry1, 128, -1);
+	gtk_widget_set_usize (key_entry1, 192, -1);
 	gtk_table_attach (GTK_TABLE (table1), key_entry1, 2, 3, 1, 2,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  0, 0, 0);
 
 	key_entry2 = gtk_entry_new ();
 	gtk_widget_show (key_entry2);
-	gtk_widget_set_usize (key_entry2, 128, -1);
+	gtk_widget_set_usize (key_entry2, 192, -1);
 	gtk_table_attach (GTK_TABLE (table1), key_entry2, 2, 3, 2, 3,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  0, 0, 0);
 
-	pred_label = gtk_label_new (_("Predicate"));
-	gtk_widget_show (pred_label);
-	gtk_table_attach (GTK_TABLE (table1), pred_label, 3, 4, 0, 1,
-			  GTK_FILL, 0, 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (pred_label), 0, 0.5);
-
 	pred_combo1 = gtk_combo_new ();
 	gtk_widget_show (pred_combo1);
 	gtk_table_attach (GTK_TABLE (table1), pred_combo1, 3, 4, 1, 2,
@@ -367,7 +374,7 @@ static void prefs_filter_create(void)
 		 GTK_SIGNAL_FUNC (prefs_filter_notrecv_radio_button_toggled),
 		 NULL);
 
-	/* register / substitute / delete */
+	/* add / replace / delete */
 
 	reg_hbox = gtk_hbox_new (FALSE, 4);
 	gtk_widget_show (reg_hbox);
@@ -382,13 +389,13 @@ static void prefs_filter_create(void)
 	gtk_widget_show (btn_hbox);
 	gtk_box_pack_start (GTK_BOX (reg_hbox), btn_hbox, FALSE, FALSE, 0);
 
-	reg_btn = gtk_button_new_with_label (_("Register"));
+	reg_btn = gtk_button_new_with_label (_("Add"));
 	gtk_widget_show (reg_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), reg_btn, FALSE, TRUE, 0);
 	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
 			    GTK_SIGNAL_FUNC (prefs_filter_register_cb), NULL);
 
-	subst_btn = gtk_button_new_with_label (_(" Substitute "));
+	subst_btn = gtk_button_new_with_label (_(" Replace "));
 	gtk_widget_show (subst_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), subst_btn, FALSE, TRUE, 0);
 	gtk_signal_connect (GTK_OBJECT (subst_btn), "clicked",
@@ -433,6 +440,14 @@ static void prefs_filter_create(void)
 	gtk_widget_show (btn_vbox);
 	gtk_box_pack_start (GTK_BOX (cond_hbox), btn_vbox, FALSE, FALSE, 0);
 
+	top_btn = gtk_button_new_with_label (_("Top"));
+	gtk_widget_show (top_btn);
+	gtk_box_pack_start (GTK_BOX (btn_vbox), top_btn, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (top_btn), "clicked",
+			    GTK_SIGNAL_FUNC (prefs_filter_top), NULL);
+
+	PACK_VSPACER (btn_vbox, spc_vbox, VSPACING_NARROW_2);
+
 	up_btn = gtk_button_new_with_label (_("Up"));
 	gtk_widget_show (up_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
@@ -445,6 +460,14 @@ static void prefs_filter_create(void)
 	gtk_signal_connect (GTK_OBJECT (down_btn), "clicked",
 			    GTK_SIGNAL_FUNC (prefs_filter_down), NULL);
 
+	PACK_VSPACER (btn_vbox, spc_vbox, VSPACING_NARROW_2);
+
+	bottom_btn = gtk_button_new_with_label (_("Bottom"));
+	gtk_widget_show (bottom_btn);
+	gtk_box_pack_start (GTK_BOX (btn_vbox), bottom_btn, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (bottom_btn), "clicked",
+			    GTK_SIGNAL_FUNC (prefs_filter_bottom), NULL);
+
 	gtk_widget_show_all(window);
 
 	filter.window    = window;
@@ -469,7 +492,10 @@ static void prefs_filter_create(void)
 	filter.notrecv_radiobtn	= notrecv_radiobtn;
 	filter.regex_chkbtn     = regex_chkbtn;
 
-	filter.cond_clist   = cond_clist;
+	filter.cond_clist = cond_clist;
+
+	filter.hdr_list  = NULL;
+	filter.hdr_table = NULL;
 }
 
 void prefs_filter_read_config(void)
@@ -479,10 +505,10 @@ void prefs_filter_read_config(void)
 	gchar buf[PREFSBUFSIZE];
 	Filter *flt;
 
-	debug_print(_("Reading filter configuration...\n"));
+	debug_print("Reading filter configuration...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_RC, NULL);
-	if ((fp = fopen(rcpath, "r")) == NULL) {
+	if ((fp = fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		return;
@@ -515,10 +541,10 @@ void prefs_filter_write_config(void)
 	PrefFile *pfile;
 	GSList *cur;
 
-	debug_print(_("Writing filter configuration...\n"));
+	debug_print("Writing filter configuration...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_RC, NULL);
-	if ((pfile = prefs_write_open(rcpath)) == NULL) {
+	if ((pfile = prefs_file_open(rcpath)) == NULL) {
 		g_warning(_("failed to write configuration to file\n"));
 		g_free(rcpath);
 		return;
@@ -532,7 +558,7 @@ void prefs_filter_write_config(void)
 		if (fputs(fstr, pfile->fp) == EOF ||
 		    fputc('\n', pfile->fp) == EOF) {
 			FILE_OP_ERROR(rcpath, "fputs || fputc");
-			prefs_write_close_revert(pfile);
+			prefs_file_close_revert(pfile);
 			g_free(rcpath);
 			g_free(fstr);
 			return;
@@ -542,7 +568,7 @@ void prefs_filter_write_config(void)
 
 	g_free(rcpath);
 
-	if (prefs_write_close(pfile) < 0) {
+	if (prefs_file_close(pfile) < 0) {
 		g_warning(_("failed to write configuration to file\n"));
 		return;
 	}
@@ -600,7 +626,7 @@ void prefs_filter_delete_path(const gchar *path)
 	prefs_filter_write_config();
 }
 
-static void prefs_filter_set_dialog(const gchar *header, const gchar *key)
+static void prefs_filter_set_dialog(void)
 {
 	GtkCList *clist = GTK_CLIST(filter.cond_clist);
 	GSList *cur;
@@ -626,9 +652,62 @@ static void prefs_filter_set_dialog(const gchar *header, const gchar *key)
 	}
 
 	gtk_clist_thaw(clist);
+}
 
-	if (header && key) {
-		gtk_entry_set_text(GTK_ENTRY(filter.hdr_entry1), header);
+static void prefs_filter_set_keyword_entry(MsgInfo *msginfo,
+					   const gchar *header_name,
+					   const gchar *key)
+{
+	if (msginfo) {
+		GList *str_list = NULL;
+		GSList *cur;
+		gchar *file;
+		Header *header;
+
+		file = procmsg_get_message_file(msginfo);
+		if (file) {
+			filter.hdr_list =
+				procheader_get_header_list_from_file(file);
+			filter.hdr_table = g_hash_table_new(str_case_hash,
+							    str_case_equal);
+			g_free(file);
+		}
+
+		for (cur = filter.hdr_list; cur != NULL; cur = cur->next) {
+			header = (Header *)cur->data;
+			if (g_hash_table_lookup(filter.hdr_table, header->name)
+			    == NULL &&
+			    g_strcasecmp(header->name, "Received") != 0 &&
+			    g_strcasecmp(header->name, "Mime-Version") != 0 &&
+			    g_strcasecmp(header->name, "X-UIDL") != 0) {
+				g_hash_table_insert(filter.hdr_table,
+						    header->name,
+						    header->body);
+				str_list = g_list_append(str_list,
+							 header->name);
+			}
+		}
+		gtk_combo_set_popdown_strings(GTK_COMBO(filter.hdr_combo1),
+					      str_list);
+		str_list = g_list_prepend(str_list, _("(none)"));
+		gtk_combo_set_popdown_strings(GTK_COMBO(filter.hdr_combo2),
+					      str_list);
+		g_list_free(str_list);
+	} else {
+		gtkut_combo_set_items(GTK_COMBO(filter.hdr_combo1),
+				      "Subject", "From", "To", "Cc", "Reply-To",
+				      "Sender", "List-Id",
+				      "X-ML-Name", "X-List", "X-Sequence",
+				      "X-Mailer", NULL);
+		gtkut_combo_set_items(GTK_COMBO(filter.hdr_combo2), _("(none)"),
+				      "Subject", "From", "To", "Cc", "Reply-To",
+				      "Sender", "List-Id",
+				      "X-ML-Name", "X-List", "X-Sequence",
+				      "X-Mailer", NULL);
+	}
+
+	if (header_name && key) {
+		gtk_entry_set_text(GTK_ENTRY(filter.hdr_entry1), header_name);
 		gtk_entry_set_text(GTK_ENTRY(filter.key_entry1), key);
 	}
 }
@@ -675,6 +754,46 @@ static gint prefs_filter_clist_set_row(gint row)
 		return -1;
 	}
 
+	flt = prefs_filter_dialog_to_filter();
+
+	cond_str[0] = filter_get_str(flt);
+	subst_char(cond_str[0], '\t', ':');
+
+	if (row < 0)
+		row = gtk_clist_append(clist, cond_str);
+	else {
+		Filter *tmpflt;
+
+		gtk_clist_set_text(clist, row, 0, cond_str[0]);
+		tmpflt = gtk_clist_get_row_data(clist, row);
+		if (tmpflt)
+			filter_free(tmpflt);
+	}
+
+	gtk_clist_set_row_data(clist, row, flt);
+
+	g_free(cond_str[0]);
+
+	prefs_filter_set_list();
+
+	return row;
+}
+
+static Filter *prefs_filter_dialog_to_filter(void)
+{
+	Filter *flt;
+	gchar *entry_text;
+
+	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry)) {
+		GET_ENTRY(filter.dest_entry);
+		if (entry_text[0] == '\0')
+			return NULL;
+	}
+
+	GET_ENTRY(filter.hdr_entry1);
+	if (entry_text[0] == '\0')
+		return NULL;
+
 	flt = g_new0(Filter, 1);
 
 	flt->name1 = g_strdup(entry_text);
@@ -710,43 +829,50 @@ static gint prefs_filter_clist_set_row(gint row)
 		flt->cond = FLT_OR;
 
 	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry)) {
-		entry_text = gtk_entry_get_text(GTK_ENTRY(filter.dest_entry));
+		GET_ENTRY(filter.dest_entry);
 		flt->dest = g_strdup(entry_text);
 		flt->action = FLT_MOVE;
 	} else
 		flt->action = FLT_NOTRECV;
 
-	cond_str[0] = filter_get_str(flt);
-	subst_char(cond_str[0], '\t', ':');
+	return flt;
+}
 
-	if (row < 0)
-		row = gtk_clist_append(clist, cond_str);
-	else {
-		Filter *tmpflt;
+static void prefs_filter_combo_select_cb(GtkWidget *widget)
+{
+	gchar *header_name;
+	gchar *header_body;
 
-		gtk_clist_set_text(clist, row, 0, cond_str[0]);
-		tmpflt = gtk_clist_get_row_data(clist, row);
-		if (tmpflt)
-			filter_free(tmpflt);
+	if (!filter.hdr_table) return;
+
+	if (widget == filter.hdr_combo1) {
+		header_name = gtk_entry_get_text(GTK_ENTRY(filter.hdr_entry1));
+		header_body = g_hash_table_lookup(filter.hdr_table,
+						  header_name);
+		gtk_entry_set_text(GTK_ENTRY(filter.key_entry1),
+				   header_body ? header_body : "");
+	} else if (widget == filter.hdr_combo2) {
+		header_name = gtk_entry_get_text(GTK_ENTRY(filter.hdr_entry2));
+		header_body = g_hash_table_lookup(filter.hdr_table,
+						  header_name);
+		gtk_entry_set_text(GTK_ENTRY(filter.key_entry2),
+				   header_body ? header_body : "");
 	}
-
-	gtk_clist_set_row_data(clist, row, flt);
-
-	g_free(cond_str[0]);
-
-	prefs_filter_set_list();
-
-	return row;
 }
 
 static void prefs_filter_select_dest_cb(void)
 {
 	FolderItem *dest;
+	gchar *id;
 
 	dest = foldersel_folder_sel(NULL, FOLDER_SEL_COPY, NULL);
-	if (!dest) return;
+	if (!dest || !dest->path) return;
 
-	gtk_entry_set_text(GTK_ENTRY(filter.dest_entry), dest->path);
+	id = folder_item_get_identifier(dest);
+	if (id) {
+		gtk_entry_set_text(GTK_ENTRY(filter.dest_entry), id);
+		g_free(id);
+	}
 }
 
 static void prefs_filter_register_cb(void)
@@ -797,6 +923,20 @@ static void prefs_filter_delete_cb(void)
 	filter_free(flt);
 	gtk_clist_remove(clist, row);
 	prefs_common.fltlist = g_slist_remove(prefs_common.fltlist, flt);
+	if (!clist->selection)
+		gtk_clist_select_row(clist, row - 1, -1);
+}
+
+static void prefs_filter_top(void)
+{
+	GtkCList *clist = GTK_CLIST(filter.cond_clist);
+	gint row;
+
+	if (!clist->selection) return;
+
+	row = GPOINTER_TO_INT(clist->selection->data);
+	if (row > 1)
+		gtk_clist_row_move(clist, row, 1);
 }
 
 static void prefs_filter_up(void)
@@ -823,6 +963,18 @@ static void prefs_filter_down(void)
 		gtk_clist_row_move(clist, row, row + 1);
 }
 
+static void prefs_filter_bottom(void)
+{
+	GtkCList *clist = GTK_CLIST(filter.cond_clist);
+	gint row;
+
+	if (!clist->selection) return;
+
+	row = GPOINTER_TO_INT(clist->selection->data);
+	if (row > 0 && row < clist->rows - 1)
+		gtk_clist_row_move(clist, row, clist->rows - 1);
+}
+
 #define ENTRY_SET_TEXT(entry, str) \
 	gtk_entry_set_text(GTK_ENTRY(entry), str ? str : "")
 
@@ -875,10 +1027,8 @@ static void prefs_filter_row_move(GtkCList *clist, gint source_row,
 				  gint dest_row)
 {
 	prefs_filter_set_list();
-	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL) {
-		gtk_clist_moveto(clist, dest_row, -1,
-				 source_row < dest_row ? 1.0 : 0.0, 0.0);
-	}
+	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL)
+		gtk_clist_moveto(clist, dest_row, -1, 0.5, 0.0);
 }
 
 static void prefs_filter_dest_radio_button_toggled(void)
@@ -919,6 +1069,39 @@ static void prefs_filter_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 static void prefs_filter_close(void)
 {
+	Filter *flt = NULL, *edit_flt = NULL;
+	gint row;
+	AlertValue aval;
+
+	if (GTK_CLIST(filter.cond_clist)->selection) {
+		row = GPOINTER_TO_INT
+			(GTK_CLIST(filter.cond_clist)->selection->data);
+		flt = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist), row);
+		edit_flt = prefs_filter_dialog_to_filter();
+	}
+
+	if ((!flt && edit_flt) ||
+	    (flt && edit_flt && filter_is_equal(flt, edit_flt) == FALSE)) {
+		aval = alertpanel(_("Entry not saved"),
+				  _("The entry was not saved. Close anyway?"),
+				  _("Yes"), _("No"), NULL);
+		if (G_ALERTDEFAULT != aval) {
+			filter_free(edit_flt);
+			return;
+		}
+	}
+
+	filter_free(edit_flt);
+
+	if (filter.hdr_table) {
+		g_hash_table_destroy(filter.hdr_table);
+		filter.hdr_table = NULL;
+	}
+	if (filter.hdr_list) {
+		procheader_header_list_destroy(filter.hdr_list);
+		filter.hdr_list = NULL;
+	}
+
 	prefs_filter_write_config();
 	gtk_widget_hide(filter.window);
 	inc_unlock();
