@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -606,6 +606,11 @@ static gint procheader_scan_date_string(const gchar *str,
 			day, month, year, hh, mm, ss, zone);
 	if (result == 7) return 0;
 
+	*zone = '\0';
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d",
+			weekday, day, month, year, hh, mm, ss);
+	if (result == 7) return 0;
+
 	*ss = 0;
 	result = sscanf(str, "%10s %d %9s %d %2d:%2d %5s",
 			weekday, day, month, year, hh, mm, zone);
@@ -627,10 +632,11 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	gint year;
 	gint hh, mm, ss;
 	gchar zone[6];
-	GDateMonth dmonth;
+	GDateMonth dmonth = G_DATE_BAD_MONTH;
 	struct tm t;
 	gchar *p;
 	time_t timer;
+	time_t tz_offset;
 
 	if (procheader_scan_date_string(src, weekday, &day, month, &year,
 					&hh, &mm, &ss, zone) < 0) {
@@ -641,20 +647,22 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	}
 
 	/* Y2K compliant :) */
-	if (year < 100) {
-		if (year < 70)
+	if (year < 1000) {
+		if (year < 50)
 			year += 2000;
 		else
 			year += 1900;
 	}
 
 	month[3] = '\0';
-	if ((p = strstr(monthstr, month)) != NULL)
-		dmonth = (gint)(p - monthstr) / 3 + 1;
-	else {
-		g_warning("Invalid month: %s\n", month);
-		dmonth = G_DATE_BAD_MONTH;
+	for (p = monthstr; *p != '\0'; p += 3) {
+		if (!strncasecmp(p, month, 3)) {
+			dmonth = (gint)(p - monthstr) / 3 + 1;
+			break;
+		}
 	}
+	if (*p == '\0')
+		g_warning("Invalid month: %s\n", month);
 
 	t.tm_sec = ss;
 	t.tm_min = mm;
@@ -667,7 +675,9 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	t.tm_isdst = -1;
 
 	timer = mktime(&t);
-	timer += tzoffset_sec(&timer) - remote_tzoffset_sec(zone);
+	tz_offset = remote_tzoffset_sec(zone);
+	if (tz_offset != -1)
+		timer += tzoffset_sec(&timer) - tz_offset;
 
 	if (dest)
 		procheader_date_get_localtime(dest, len, timer);
