@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "procmime.h"
 #include "procheader.h"
 #include "base64.h"
+#include "quoted-printable.h"
 #include "uuencode.h"
 #include "unmime.h"
 #include "html.h"
@@ -59,28 +60,6 @@ MimeInfo *procmime_mimeinfo_new(void)
 	return mimeinfo;
 }
 
-void procmime_mimeinfo_free(MimeInfo *mimeinfo)
-{
-	if (!mimeinfo) return;
-
-	g_free(mimeinfo->encoding);
-	g_free(mimeinfo->content_type);
-	g_free(mimeinfo->charset);
-	g_free(mimeinfo->name);
-	g_free(mimeinfo->boundary);
-	g_free(mimeinfo->content_disposition);
-	g_free(mimeinfo->filename);
-#if USE_GPGME
-	g_free(mimeinfo->plaintextfile);
-	g_free(mimeinfo->sigstatus);
-	g_free(mimeinfo->sigstatus_full);
-#endif
-
-	procmime_mimeinfo_free(mimeinfo->sub);
-
-	g_free(mimeinfo);
-}
-
 void procmime_mimeinfo_free_all(MimeInfo *mimeinfo)
 {
 	while (mimeinfo != NULL) {
@@ -187,6 +166,22 @@ MimeInfo *procmime_mimeinfo_next(MimeInfo *mimeinfo)
 	return NULL;
 }
 
+#if 0
+void procmime_dump_mimeinfo(MimeInfo *mimeinfo)
+{
+	gint i;
+
+	g_print("\n");
+
+	for (; mimeinfo != NULL; mimeinfo = procmime_mimeinfo_next(mimeinfo)) {
+		for (i = 0; i < mimeinfo->level; i++)
+			g_print("  ");
+		g_print("%s%s\n", mimeinfo->main ? "sub: " : "",
+			mimeinfo->content_type);
+	}
+}
+#endif
+
 MimeInfo *procmime_scan_message(MsgInfo *msginfo)
 {
 	FILE *fp;
@@ -194,22 +189,21 @@ MimeInfo *procmime_scan_message(MsgInfo *msginfo)
 
 	g_return_val_if_fail(msginfo != NULL, NULL);
 
+#if USE_GPGME
+	if ((fp = procmsg_open_message_decrypted(msginfo, &mimeinfo)) == NULL)
+		return NULL;
+#else
 	if ((fp = procmsg_open_message(msginfo)) == NULL) return NULL;
 	mimeinfo = procmime_scan_mime_header(fp);
+#endif
 
 	if (mimeinfo) {
-		if (mimeinfo->mime_type != MIME_MULTIPART) {
-			if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
-				perror("fseek");
-		}
-		if (mimeinfo->mime_type != MIME_TEXT)
+		mimeinfo->size = get_left_file_size(fp);
+		if (mimeinfo->mime_type == MIME_MULTIPART ||
+		    mimeinfo->mime_type == MIME_MESSAGE_RFC822)
 			procmime_scan_multipart_message(mimeinfo, fp);
 	}
 
-#if USE_GPGME
-        if (prefs_common.auto_check_signatures)
-		rfc2015_check_signature(mimeinfo, fp);
-#endif
 	fclose(fp);
 
 	return mimeinfo;
@@ -222,10 +216,10 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 	gint boundary_len = 0;
 	gchar buf[BUFFSIZE];
 	glong fpos, prev_fpos;
-	gint npart;
 
 	g_return_if_fail(mimeinfo != NULL);
-	g_return_if_fail(mimeinfo->mime_type != MIME_TEXT);
+	g_return_if_fail(mimeinfo->mime_type == MIME_MULTIPART ||
+			 mimeinfo->mime_type == MIME_MESSAGE_RFC822);
 
 	if (mimeinfo->mime_type == MIME_MULTIPART) {
 		g_return_if_fail(mimeinfo->boundary != NULL);
@@ -242,6 +236,9 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 		while ((p = fgets(buf, sizeof(buf), fp)) != NULL)
 			if (IS_BOUNDARY(buf, boundary, boundary_len)) break;
 		if (!p) return;
+	} else if (mimeinfo->parent && mimeinfo->parent->boundary) {
+		boundary = mimeinfo->parent->boundary;
+		boundary_len = strlen(boundary);
 	}
 
 	if ((fpos = ftell(fp)) < 0) {
@@ -249,40 +246,37 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 		return;
 	}
 
-	for (npart = 0;; npart++) {
+	for (;;) {
 		MimeInfo *partinfo;
 		gboolean eom = FALSE;
+		gint len;
 
 		prev_fpos = fpos;
 		debug_print("prev_fpos: %ld\n", fpos);
 
-		partinfo = procmime_scan_mime_header(fp);
-		if (!partinfo) break;
-		procmime_mimeinfo_insert(mimeinfo, partinfo);
-
-		if (partinfo->mime_type == MIME_MULTIPART) {
-			if (partinfo->level < 8)
-				procmime_scan_multipart_message(partinfo, fp);
-		} else if (partinfo->mime_type == MIME_MESSAGE_RFC822) {
+		if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 			MimeInfo *sub;
 
-			partinfo->sub = sub = procmime_scan_mime_header(fp);
+			mimeinfo->sub = sub = procmime_scan_mime_header(fp);
 			if (!sub) break;
 
-			sub->level = partinfo->level + 1;
-			sub->parent = partinfo->parent;
-			sub->main = partinfo;
+			sub->level = mimeinfo->level + 1;
+			sub->parent = mimeinfo->parent;
+			sub->main = mimeinfo;
 
-			if (sub->level < 8) {
-				if (sub->mime_type == MIME_MULTIPART) {
-					procmime_scan_multipart_message
-						(sub, fp);
-				} else if (sub->mime_type == MIME_MESSAGE_RFC822) {
-					fseek(fp, sub->fpos, SEEK_SET);
-					procmime_scan_multipart_message
-						(sub, fp);
-				}
-			}
+			partinfo = sub;
+		} else {
+			partinfo = procmime_scan_mime_header(fp);
+			if (!partinfo) break;
+			procmime_mimeinfo_insert(mimeinfo, partinfo);
+			debug_print("content-type: %s\n",
+				    partinfo->content_type);
+		}
+
+		if (partinfo->mime_type == MIME_MULTIPART ||
+		    partinfo->mime_type == MIME_MESSAGE_RFC822) {
+			if (partinfo->level < 8)
+				procmime_scan_multipart_message(partinfo, fp);
 		}
 
 		/* look for next boundary */
@@ -300,18 +294,27 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 			buf[0] = '\0';
 			eom = TRUE;
 		}
+		debug_print("boundary: %s\n", buf);
+
 		fpos = ftell(fp);
 		debug_print("fpos: %ld\n", fpos);
 
-		partinfo->size = fpos - prev_fpos - strlen(buf);
+		len = strlen(buf);
+		partinfo->size = fpos - prev_fpos - len;
 		debug_print("partinfo->size: %d\n", partinfo->size);
 		if (partinfo->sub && !partinfo->sub->sub &&
 		    !partinfo->sub->children) {
-			partinfo->sub->size = fpos - partinfo->sub->fpos - strlen(buf);
+			partinfo->sub->size =
+				fpos - partinfo->sub->fpos - strlen(buf);
 			debug_print("partinfo->sub->size: %d\n",
 				    partinfo->sub->size);
 		}
-		debug_print("boundary: %s\n", buf);
+
+		if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
+			if (len > 0 && fseek(fp, fpos - len, SEEK_SET) < 0)
+				perror("fseek");
+			break;
+		}
 
 		if (eom) break;
 	}
@@ -561,7 +564,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		       (!boundary ||
 			!IS_BOUNDARY(buf, boundary, boundary_len))) {
 			gint len;
-			len = unmime_quoted_printable_line(buf);
+			len = qp_decode_line(buf);
 			fwrite(buf, len, 1, outfp);
 		}
 	} else if (mimeinfo->encoding_type == ENC_BASE64) {
@@ -1031,10 +1034,14 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 	else if (!strncasecmp(charset, "ISO-2022-", 9) ||
 		 !strcasecmp(charset, "US-ASCII"))
 		return ENC_7BIT;
+	else if (!strcasecmp(charset, "ISO-8859-5") ||
+		 !strncasecmp(charset, "KOI8-", 5) ||
+		 !strcasecmp(charset, "Windows-1251"))
+		return ENC_8BIT;
+	else if (!strncasecmp(charset, "ISO-8859-", 9))
+		return ENC_QUOTED_PRINTABLE;
 	else
 		return ENC_8BIT;
-		/* return ENC_BASE64; */
-		/* return ENC_QUOTED_PRINTABLE; */
 }
 
 EncodingType procmime_get_encoding_for_file(const gchar *file)
