@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,36 +22,48 @@
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 #include "intl.h"
 #include "main.h"
 #include "utils.h"
 #include "procmsg.h"
 #include "procheader.h"
-#include "send.h"
+#include "send_message.h"
 #include "procmime.h"
 #include "statusbar.h"
+#include "prefs_filter.h"
+#include "filter.h"
 #include "folder.h"
 #if USE_GPGME
 #  include "rfc2015.h"
 #endif
 
-typedef struct _FlagInfo	FlagInfo;
-
-struct _FlagInfo
-{
-	guint    msgnum;
-	MsgFlags flags;
-};
-
 static void mark_sum_func			(gpointer	 key,
 						 gpointer	 value,
 						 gpointer	 data);
 
-static GHashTable *procmsg_read_mark_file	(const gchar	*folder);
-static gint procmsg_cmp_msgnum			(gconstpointer	 a,
+static GHashTable *procmsg_read_mark_file	(FolderItem	*item);
+
+static gint procmsg_cmp_by_mark			(gconstpointer	 a,
 						 gconstpointer	 b);
-static gint procmsg_cmp_flag_msgnum		(gconstpointer	 a,
+static gint procmsg_cmp_by_unread		(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_mime			(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_label		(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_number		(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_size			(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_date			(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_from			(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_to			(gconstpointer	 a,
+						 gconstpointer	 b);
+static gint procmsg_cmp_by_subject		(gconstpointer	 a,
 						 gconstpointer	 b);
 
 
@@ -175,7 +187,7 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(item->folder != NULL, NULL);
-	type = item->folder->type;
+	type = FOLDER_TYPE(item->folder);
 
 	default_flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	default_flags.tmp_flags = MSG_CACHED;
@@ -271,9 +283,9 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 void procmsg_set_flags(GSList *mlist, FolderItem *item)
 {
 	GSList *cur, *tmp;
-	gint newmsg = 0;
+	gint new = 0, unread = 0, total = 0;
 	gint lastnum = 0;
-	gchar *markdir;
+	gint unflagged = 0;
 	MsgInfo *msginfo;
 	GHashTable *mark_table;
 	MsgFlags *flags;
@@ -282,13 +294,13 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	debug_print(_("\tMarking the messages..."));
+	debug_print("Marking the messages...\n");
 
-	markdir = folder_item_get_path(item);
-	mark_table = procmsg_read_mark_file(markdir);
-	g_free(markdir);
-
-	if (!mark_table) return;
+	mark_table = procmsg_read_mark_file(item);
+	if (!mark_table) {
+		item->new = item->unread = item->total = g_slist_length(mlist);
+		return;
+	}
 
 	for (cur = mlist; cur != NULL; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
@@ -302,33 +314,87 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 		if (flags != NULL) {
 			/* add the permanent flags only */
 			msginfo->flags.perm_flags = flags->perm_flags;
-			if (item->folder->type == F_IMAP) {
+			if (MSG_IS_NEW(*flags))
+				++new;
+			if (MSG_IS_UNREAD(*flags))
+				++unread;
+			if (FOLDER_TYPE(item->folder) == F_IMAP) {
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_IMAP);
-			} else if (item->folder->type == F_NEWS) {
+			} else if (FOLDER_TYPE(item->folder) == F_NEWS) {
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_NEWS);
 			}
 		} else {
 			/* not found (new message) */
-			if (newmsg == 0) {
-				for (tmp = mlist; tmp != cur; tmp = tmp->next)
-					MSG_UNSET_PERM_FLAGS
-						(((MsgInfo *)tmp->data)->flags,
-						 MSG_NEW);
+			if (unflagged == 0) {
+				for (tmp = mlist; tmp != cur; tmp = tmp->next) {
+					msginfo = (MsgInfo *)tmp->data;
+					if (MSG_IS_NEW(msginfo->flags)) {
+						MSG_UNSET_PERM_FLAGS
+							(msginfo->flags,
+							 MSG_NEW);
+						--new;
+					}
+				}
 			}
-			newmsg++;
+			++unflagged;
+			++new;
+			++unread;
 		}
+
+		++total;
 	}
 
+	item->new = new;
+	item->unread = unread;
+	item->total = total;
 	item->last_num = lastnum;
 
-	debug_print(_("done.\n"));
-	if (newmsg)
-		debug_print(_("\t%d new message(s)\n"), newmsg);
+	debug_print("new: %d unread: %d unflagged: %d total: %d\n",
+		    new, unread, unflagged, total);
 
 	hash_free_value_mem(mark_table);
 	g_hash_table_destroy(mark_table);
 }
 
+static FolderSortType cmp_func_sort_type;
+
+GSList *procmsg_sort_msg_list(GSList *mlist, FolderSortKey sort_key,
+			      FolderSortType sort_type)
+{
+	GCompareFunc cmp_func;
+
+	switch (sort_key) {
+	case SORT_BY_MARK:
+		cmp_func = procmsg_cmp_by_mark; break;
+	case SORT_BY_UNREAD:
+		cmp_func = procmsg_cmp_by_unread; break;
+	case SORT_BY_MIME:
+		cmp_func = procmsg_cmp_by_mime; break;
+	case SORT_BY_LABEL:
+		cmp_func = procmsg_cmp_by_label; break;
+	case SORT_BY_NUMBER:
+		cmp_func = procmsg_cmp_by_number; break;
+	case SORT_BY_SIZE:
+		cmp_func = procmsg_cmp_by_size; break;
+	case SORT_BY_DATE:
+		cmp_func = procmsg_cmp_by_date; break;
+	case SORT_BY_FROM:
+		cmp_func = procmsg_cmp_by_from; break;
+	case SORT_BY_SUBJECT:
+		cmp_func = procmsg_cmp_by_subject; break;
+	case SORT_BY_TO:
+		cmp_func = procmsg_cmp_by_to; break;
+	default:
+		return mlist;
+	}
+
+	cmp_func_sort_type = sort_type;
+
+	mlist = g_slist_sort(mlist, cmp_func);
+
+	return mlist;
+}
+
 gint procmsg_get_last_num_in_msg_list(GSList *mlist)
 {
 	GSList *cur;
@@ -403,7 +469,6 @@ void procmsg_flush_mark_queue(FolderItem *item, FILE *fp)
 void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
 {
 	FILE *fp;
-	gchar *path;
 	MsgInfo msginfo;
 
 	g_return_if_fail(item != NULL);
@@ -419,15 +484,10 @@ void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
 		return;
 	}
 
-	path = folder_item_get_path(item);
-	g_return_if_fail(path != NULL);
-
-	if ((fp = procmsg_open_mark_file(path, TRUE)) == NULL) {
+	if ((fp = procmsg_open_mark_file(item, TRUE)) == NULL) {
 		g_warning(_("can't open mark file\n"));
-		g_free(path);
 		return;
 	}
-	g_free(path);
 
 	msginfo.msgnum = num;
 	msginfo.flags = flags;
@@ -462,7 +522,7 @@ static void mark_sum_func(gpointer key, gpointer value, gpointer data)
 	g_free(flags);
 }
 
-void procmsg_get_mark_sum(const gchar *folder,
+void procmsg_get_mark_sum(FolderItem *item,
 			  gint *new, gint *unread, gint *total,
 			  gint *min, gint *max,
 			  gint first)
@@ -478,7 +538,7 @@ void procmsg_get_mark_sum(const gchar *folder,
 	marksum.max    = max;
 	marksum.first  = first;
 
-	mark_table = procmsg_read_mark_file(folder);
+	mark_table = procmsg_read_mark_file(item);
 
 	if (mark_table) {
 		g_hash_table_foreach(mark_table, mark_sum_func, &marksum);
@@ -486,7 +546,7 @@ void procmsg_get_mark_sum(const gchar *folder,
 	}
 }
 
-static GHashTable *procmsg_read_mark_file(const gchar *folder)
+static GHashTable *procmsg_read_mark_file(FolderItem *item)
 {
 	FILE *fp;
 	GHashTable *mark_table = NULL;
@@ -494,7 +554,7 @@ static GHashTable *procmsg_read_mark_file(const gchar *folder)
 	MsgFlags *flags;
 	MsgPermFlags perm_flags;
 
-	if ((fp = procmsg_open_mark_file(folder, FALSE)) == NULL)
+	if ((fp = procmsg_open_mark_file(item, FALSE)) == NULL)
 		return NULL;
 
 	mark_table = g_hash_table_new(NULL, g_direct_equal);
@@ -516,13 +576,13 @@ static GHashTable *procmsg_read_mark_file(const gchar *folder)
 	return mark_table;
 }
 
-FILE *procmsg_open_mark_file(const gchar *folder, gboolean append)
+FILE *procmsg_open_mark_file(FolderItem *item, gboolean append)
 {
 	gchar *markfile;
 	FILE *fp;
 	gint ver;
 
-	markfile = g_strconcat(folder, G_DIR_SEPARATOR_S, MARK_FILE, NULL);
+	markfile = folder_item_get_mark_file(item);
 
 	if ((fp = fopen(markfile, "rb")) == NULL)
 		debug_print(_("Mark file not found.\n"));
@@ -593,7 +653,10 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 		msginfo = (MsgInfo *)node->data;
 		if (msginfo->inreplyto) {
 			parent = g_hash_table_lookup(table, msginfo->inreplyto);
-			if (parent && parent != node) {
+			/* node should not be the parent, and node should not
+			   be an ancestor of parent (circular reference) */
+			if (parent && parent != node &&
+			    !g_node_is_ancestor(node, parent)) {
 				g_node_unlink(node);
 				g_node_insert_before
 					(parent, parent->children, node);
@@ -628,7 +691,7 @@ void procmsg_move_messages(GSList *mlist)
 		} else if (dest == msginfo->to_folder) {
 			movelist = g_slist_append(movelist, msginfo);
 		} else {
-			folder_item_move_msgs_with_dest(dest, movelist);
+			folder_item_move_msgs(dest, movelist);
 			g_slist_free(movelist);
 			movelist = NULL;
 			dest = msginfo->to_folder;
@@ -637,7 +700,7 @@ void procmsg_move_messages(GSList *mlist)
 	}
 
 	if (movelist) {
-		folder_item_move_msgs_with_dest(dest, movelist);
+		folder_item_move_msgs(dest, movelist);
 		g_slist_free(movelist);
 	}
 }
@@ -663,7 +726,7 @@ void procmsg_copy_messages(GSList *mlist)
 		} else if (dest == msginfo->to_folder) {
 			copylist = g_slist_append(copylist, msginfo);
 		} else {
-			folder_item_copy_msgs_with_dest(dest, copylist);
+			folder_item_copy_msgs(dest, copylist);
 			g_slist_free(copylist);
 			copylist = NULL;
 			dest = msginfo->to_folder;
@@ -672,7 +735,7 @@ void procmsg_copy_messages(GSList *mlist)
 	}
 
 	if (copylist) {
-		folder_item_copy_msgs_with_dest(dest, copylist);
+		folder_item_copy_msgs(dest, copylist);
 		g_slist_free(copylist);
 	}
 }
@@ -708,6 +771,48 @@ gchar *procmsg_get_message_file(MsgInfo *msginfo)
 	return filename;
 }
 
+GSList *procmsg_get_message_file_list(GSList *mlist)
+{
+	GSList *file_list = NULL;
+	MsgInfo *msginfo;
+	MsgFileInfo *fileinfo;
+	gchar *file;
+
+	while (mlist != NULL) {
+		msginfo = (MsgInfo *)mlist->data;
+		file = procmsg_get_message_file(msginfo);
+		if (!file) {
+			procmsg_message_file_list_free(file_list);
+			return NULL;
+		}
+		fileinfo = g_new(MsgFileInfo, 1);
+		fileinfo->file = file;
+		fileinfo->flags = g_new(MsgFlags, 1);
+		*fileinfo->flags = msginfo->flags;
+		file_list = g_slist_prepend(file_list, fileinfo);
+		mlist = mlist->next;
+	}
+
+	file_list = g_slist_reverse(file_list);
+
+	return file_list;
+}
+
+void procmsg_message_file_list_free(GSList *file_list)
+{
+	GSList *cur;
+	MsgFileInfo *fileinfo;
+
+	for (cur = file_list; cur != NULL; cur = cur->next) {
+		fileinfo = (MsgFileInfo *)cur->data;
+		g_free(fileinfo->file);
+		g_free(fileinfo->flags);
+		g_free(fileinfo);
+	}
+
+	g_slist_free(file_list);
+}
+
 FILE *procmsg_open_message(MsgInfo *msginfo)
 {
 	FILE *fp;
@@ -747,6 +852,7 @@ FILE *procmsg_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
 {
 	FILE *fp;
 	MimeInfo *mimeinfo_;
+	glong fpos;
 
 	g_return_val_if_fail(msginfo != NULL, NULL);
 
@@ -768,6 +874,7 @@ FILE *procmsg_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
 	if (MSG_IS_ENCRYPTED(msginfo->flags) &&
 	    !msginfo->plaintext_file &&
 	    !msginfo->decryption_failed) {
+		fpos = ftell(fp);
 		rfc2015_decrypt_message(msginfo, mimeinfo_, fp);
 		if (msginfo->plaintext_file &&
 		    !msginfo->decryption_failed) {
@@ -780,6 +887,9 @@ FILE *procmsg_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
 				fclose(fp);
 				return NULL;
 			}
+		} else {
+			if (fseek(fp, fpos, SEEK_SET) < 0)
+				perror("fseek");
 		}
 	}
 
@@ -803,6 +913,106 @@ gboolean procmsg_msg_exist(MsgInfo *msginfo)
 	return ret;
 }
 
+void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
+				PrefsFilterType type)
+{
+	static HeaderEntry hentry[] = {{"List-Id:",	   NULL, TRUE},
+				       {"X-ML-Name:",	   NULL, TRUE},
+				       {"X-List:",	   NULL, TRUE},
+				       {"X-Mailing-list:", NULL, TRUE},
+				       {"X-Sequence:",	   NULL, TRUE},
+				       {NULL,		   NULL, FALSE}};
+	enum
+	{
+		H_LIST_ID	 = 0,
+		H_X_ML_NAME	 = 1,
+		H_X_LIST	 = 2,
+		H_X_MAILING_LIST = 3,
+		H_X_SEQUENCE	 = 4
+	};
+
+	FILE *fp;
+
+	g_return_if_fail(msginfo != NULL);
+	g_return_if_fail(header != NULL);
+	g_return_if_fail(key != NULL);
+
+	*header = NULL;
+	*key = NULL;
+
+	switch (type) {
+	case FILTER_BY_NONE:
+		return;
+	case FILTER_BY_AUTO:
+		if ((fp = procmsg_open_message(msginfo)) == NULL)
+			return;
+		procheader_get_header_fields(fp, hentry);
+		fclose(fp);
+
+#define SET_FILTER_KEY(hstr, idx)	\
+{					\
+	*header = g_strdup(hstr);	\
+	*key = hentry[idx].body;	\
+	hentry[idx].body = NULL;	\
+}
+
+		if (hentry[H_LIST_ID].body != NULL) {
+			SET_FILTER_KEY("List-Id", H_LIST_ID);
+			extract_list_id_str(*key);
+		} else if (hentry[H_X_ML_NAME].body != NULL) {
+			SET_FILTER_KEY("X-ML-Name", H_X_ML_NAME);
+		} else if (hentry[H_X_LIST].body != NULL) {
+			SET_FILTER_KEY("X-List", H_X_LIST);
+		} else if (hentry[H_X_MAILING_LIST].body != NULL) {
+			SET_FILTER_KEY("X-Mailing-list", H_X_MAILING_LIST);
+		} else if (hentry[H_X_SEQUENCE].body != NULL) {
+			gchar *p;
+
+			SET_FILTER_KEY("X-Sequence", H_X_SEQUENCE);
+			p = *key;
+			while (*p != '\0') {
+				while (*p != '\0' && !isspace(*p)) p++;
+				while (isspace(*p)) p++;
+				if (isdigit(*p)) {
+					*p = '\0';
+					break;
+				}
+			}
+			g_strstrip(*key);
+		} else if (msginfo->subject) {
+			*header = g_strdup("Subject");
+			*key = g_strdup(msginfo->subject);
+		}
+
+#undef SET_FILTER_KEY
+
+		g_free(hentry[H_LIST_ID].body);
+		hentry[H_LIST_ID].body = NULL;
+		g_free(hentry[H_X_ML_NAME].body);
+		hentry[H_X_ML_NAME].body = NULL;
+		g_free(hentry[H_X_LIST].body);
+		hentry[H_X_LIST].body = NULL;
+		g_free(hentry[H_X_MAILING_LIST].body);
+		hentry[H_X_MAILING_LIST].body = NULL;
+
+		break;
+	case FILTER_BY_FROM:
+		*header = g_strdup("From");
+		*key = g_strdup(msginfo->from);
+		break;
+	case FILTER_BY_TO:
+		*header = g_strdup("To");
+		*key = g_strdup(msginfo->to);
+		break;
+	case FILTER_BY_SUBJECT:
+		*header = g_strdup("Subject");
+		*key = g_strdup(msginfo->subject);
+		break;
+	default:
+		break;
+	}
+}
+
 void procmsg_empty_trash(void)
 {
 	FolderItem *trash;
@@ -817,8 +1027,9 @@ void procmsg_empty_trash(void)
 
 gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 {
-	gint i;
 	gint ret = 0;
+	GSList *mlist = NULL;
+	GSList *cur;
 
 	if (!queue)
 		queue = folder_get_default_queue();
@@ -826,27 +1037,34 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 
 	folder_item_scan(queue);
 	if (queue->last_num < 0) return -1;
-	else if (queue->last_num == 0) return 0;
 
-	for (i = 1; i <= queue->last_num; i++) {
+	mlist = folder_item_get_msg_list(queue, FALSE);
+	mlist = procmsg_sort_msg_list(mlist, SORT_BY_NUMBER, SORT_ASCENDING);
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
 		gchar *file;
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
 
-		file = folder_item_fetch_msg(queue, i);
+		file = procmsg_get_message_file(msginfo);
 		if (file) {
 			if (send_message_queue(file) < 0) {
-				g_warning(_("Sending queued message %d failed.\n"), i);
-				ret = -1;
+				g_warning(_("Sending queued message %d failed.\n"), msginfo->msgnum);
 			} else {
+				ret++;
 				if (save_msgs)
 					procmsg_save_to_outbox
 						(queue->folder->outbox,
 						 file, TRUE);
-				folder_item_remove_msg(queue, i);
+				folder_item_remove_msg(queue, msginfo);
 			}
 			g_free(file);
 		}
+
+		procmsg_msginfo_free(msginfo);
 	}
 
+	g_slist_free(mlist);
+
 	return ret;
 }
 
@@ -888,21 +1106,20 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
 		fclose(fp);
 
 		folder_item_scan(outbox);
-		if ((num = folder_item_add_msg(outbox, tmp, TRUE)) < 0) {
+		if ((num = folder_item_add_msg(outbox, tmp, &flag, TRUE)) < 0) {
 			g_warning("can't save message\n");
 			unlink(tmp);
 			return -1;
 		}
 	} else {
 		folder_item_scan(outbox);
-		if ((num = folder_item_add_msg(outbox, file, FALSE)) < 0) {
+		if ((num = folder_item_add_msg
+			(outbox, file, &flag, FALSE)) < 0) {
 			g_warning("can't save message\n");
 			return -1;
 		}
 	}
 
-	procmsg_add_flags(outbox, num, flag);
-
 	return 0;
 }
 
@@ -1050,17 +1267,6 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	g_free(msginfo);
 }
 
-static gint procmsg_cmp_msgnum(gconstpointer a, gconstpointer b)
-{
-	const MsgInfo *msginfo = a;
-	const guint msgnum = GPOINTER_TO_UINT(b);
-
-	if (!msginfo)
-		return -1;
-
-	return msginfo->msgnum - msgnum;
-}
-
 gint procmsg_cmp_msgnum_for_sort(gconstpointer a, gconstpointer b)
 {
 	const MsgInfo *msginfo1 = a;
@@ -1074,13 +1280,64 @@ gint procmsg_cmp_msgnum_for_sort(gconstpointer a, gconstpointer b)
 	return msginfo1->msgnum - msginfo2->msgnum;
 }
 
-static gint procmsg_cmp_flag_msgnum(gconstpointer a, gconstpointer b)
-{
-	const FlagInfo *finfo = a;
-	const guint msgnum = GPOINTER_TO_UINT(b);
+#define CMP_FUNC_DEF(func_name, val)					\
+static gint func_name(gconstpointer a, gconstpointer b)			\
+{									\
+	const MsgInfo *msginfo1 = a;					\
+	const MsgInfo *msginfo2 = b;					\
+									\
+	if (!msginfo1 || !msginfo2)					\
+		return -1;						\
+									\
+	return (val) * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+}
 
-	if (!finfo)
-		return -1;
+CMP_FUNC_DEF(procmsg_cmp_by_mark,
+	     MSG_IS_MARKED(msginfo1->flags) - MSG_IS_MARKED(msginfo2->flags))
+CMP_FUNC_DEF(procmsg_cmp_by_unread,
+	     MSG_IS_UNREAD(msginfo1->flags) - MSG_IS_UNREAD(msginfo2->flags))
+CMP_FUNC_DEF(procmsg_cmp_by_mime,
+	     MSG_IS_MIME(msginfo1->flags) - MSG_IS_MIME(msginfo2->flags))
+CMP_FUNC_DEF(procmsg_cmp_by_label,
+	     MSG_GET_COLORLABEL(msginfo1->flags) -
+	     MSG_GET_COLORLABEL(msginfo2->flags))
 
-	return finfo->msgnum - msgnum;
+CMP_FUNC_DEF(procmsg_cmp_by_number, msginfo1->msgnum - msginfo2->msgnum)
+CMP_FUNC_DEF(procmsg_cmp_by_size, msginfo1->size - msginfo2->size)
+CMP_FUNC_DEF(procmsg_cmp_by_date, msginfo1->date_t - msginfo2->date_t)
+
+#undef CMP_FUNC_DEF
+#define CMP_FUNC_DEF(func_name, var_name)				\
+static gint func_name(gconstpointer a, gconstpointer b)			\
+{									\
+	const MsgInfo *msginfo1 = a;					\
+	const MsgInfo *msginfo2 = b;					\
+									\
+	if (!msginfo1->var_name)					\
+		return (msginfo2->var_name != NULL);			\
+	if (!msginfo2->var_name)					\
+		return -1;						\
+									\
+	return strcasecmp(msginfo1->var_name, msginfo2->var_name) *	\
+		(cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+}
+
+CMP_FUNC_DEF(procmsg_cmp_by_from, fromname)
+CMP_FUNC_DEF(procmsg_cmp_by_to, to)
+
+#undef CMP_FUNC_DEF
+
+static gint procmsg_cmp_by_subject(gconstpointer a, gconstpointer b)	\
+{									\
+	const MsgInfo *msginfo1 = a;					\
+	const MsgInfo *msginfo2 = b;					\
+									\
+	if (!msginfo1->subject)						\
+		return (msginfo2->subject != NULL);			\
+	if (!msginfo2->subject)						\
+		return -1;						\
+									\
+	return subject_compare_for_sort					\
+		(msginfo1->subject, msginfo2->subject) *		\
+		(cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
 }
