@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,32 +32,32 @@
 
 typedef struct _MsgInfo		MsgInfo;
 typedef struct _MsgFlags	MsgFlags;
+typedef struct _MsgFileInfo	MsgFileInfo;
 
 #include "folder.h"
 #include "procmime.h"
+#include "prefs_filter.h"
 
-typedef enum
-{
-	MSG_NEW		= 1 << 0,
-	MSG_UNREAD	= 1 << 1,
-	MSG_MARKED	= 1 << 2,
-	MSG_DELETED	= 1 << 3,
-	MSG_REPLIED	= 1 << 4,
-	MSG_FORWARDED	= 1 << 5,
+#define MSG_NEW			(1U << 0)
+#define MSG_UNREAD		(1U << 1)
+#define MSG_MARKED		(1U << 2)
+#define MSG_DELETED		(1U << 3)
+#define MSG_REPLIED		(1U << 4)
+#define MSG_FORWARDED		(1U << 5)
 
 #define MSG_CLABEL_SBIT	(7)		/* start bit of color label */
 #define MAKE_MSG_CLABEL(h, m, l)	(((h) << (MSG_CLABEL_SBIT + 2)) | \
 					 ((m) << (MSG_CLABEL_SBIT + 1)) | \
 					 ((l) << (MSG_CLABEL_SBIT + 0)))
 
-	MSG_CLABEL_NONE = MAKE_MSG_CLABEL(0, 0, 0),
-	MSG_CLABEL_1    = MAKE_MSG_CLABEL(0, 0, 1),
-	MSG_CLABEL_2    = MAKE_MSG_CLABEL(0, 1, 0),
-	MSG_CLABEL_3    = MAKE_MSG_CLABEL(0, 1, 1),
-	MSG_CLABEL_4    = MAKE_MSG_CLABEL(1, 0, 0),
-	MSG_CLABEL_5    = MAKE_MSG_CLABEL(1, 0, 1),
-	MSG_CLABEL_6    = MAKE_MSG_CLABEL(1, 1, 0),
-	MSG_CLABEL_7    = MAKE_MSG_CLABEL(1, 1, 1),
+#define MSG_CLABEL_NONE		MAKE_MSG_CLABEL(0U, 0U, 0U)
+#define MSG_CLABEL_1		MAKE_MSG_CLABEL(0U, 0U, 1U)
+#define MSG_CLABEL_2		MAKE_MSG_CLABEL(0U, 1U, 0U)
+#define MSG_CLABEL_3		MAKE_MSG_CLABEL(0U, 1U, 1U)
+#define MSG_CLABEL_4		MAKE_MSG_CLABEL(1U, 0U, 0U)
+#define MSG_CLABEL_5		MAKE_MSG_CLABEL(1U, 0U, 1U)
+#define MSG_CLABEL_6		MAKE_MSG_CLABEL(1U, 1U, 0U)
+#define MSG_CLABEL_7		MAKE_MSG_CLABEL(1U, 1U, 1U)
 
 #define MSG_CLABEL_ORANGE	MSG_CLABEL_1
 #define MSG_CLABEL_RED		MSG_CLABEL_2
@@ -67,27 +67,25 @@ typedef enum
 #define MSG_CLABEL_GREEN	MSG_CLABEL_6
 #define MSG_CLABEL_BROWN	MSG_CLABEL_7
 
-	/* RESERVED */
-	MSG_RESERVED	= 1 << 31
-} MsgPermFlags;
+/* RESERVED */
+#define	MSG_RESERVED		(1U << 31)
+
+typedef guint32 MsgPermFlags;
 
 #define MSG_CLABEL_FLAG_MASK	(MSG_CLABEL_7)
 
-typedef enum
-{
-	MSG_MOVE	= 1 << 0,
-	MSG_COPY	= 1 << 1,
+#define MSG_MOVE		(1U << 0)
+#define MSG_COPY		(1U << 1)
+#define MSG_QUEUED		(1U << 16)
+#define MSG_DRAFT		(1U << 17)
+#define MSG_ENCRYPTED		(1U << 18)
+#define MSG_IMAP		(1U << 19)
+#define MSG_NEWS		(1U << 20)
+#define MSG_SIGNED		(1U << 21)
+#define MSG_MIME		(1U << 29)
+#define MSG_CACHED		(1U << 31)
 
-	MSG_QUEUED	= 1 << 16,
-	MSG_DRAFT	= 1 << 17,
-	MSG_ENCRYPTED   = 1 << 18,
-	MSG_IMAP	= 1 << 19,
-	MSG_NEWS	= 1 << 20,
-
-	MSG_MIME	= 1 << 29,
-
-	MSG_CACHED	= 1 << 31
-} MsgTmpFlags;
+typedef guint32 MsgTmpFlags;
 
 #define MSG_CACHED_FLAG_MASK	(MSG_MIME)
 
@@ -128,18 +126,19 @@ typedef enum
 #define WRITE_CACHE_DATA_INT(n, fp) \
 	fwrite(&n, sizeof(n), 1, fp)
 
-#define WRITE_CACHE_DATA(data, fp) \
-{ \
-	gint len; \
- \
-	if (data == NULL || (len = strlen(data)) == 0) { \
-		len = 0; \
-		WRITE_CACHE_DATA_INT(len, fp); \
-	} else { \
-		len = strlen(data); \
-		WRITE_CACHE_DATA_INT(len, fp); \
-		fwrite(data, len, 1, fp); \
-	} \
+#define WRITE_CACHE_DATA(data, fp)			\
+{							\
+	gint len;					\
+							\
+	if (data == NULL) {				\
+		len = 0;				\
+		WRITE_CACHE_DATA_INT(len, fp);		\
+	} else {					\
+		len = strlen(data);			\
+		WRITE_CACHE_DATA_INT(len, fp);		\
+		if (len > 0)				\
+			fwrite(data, len, 1, fp);	\
+	}						\
 }
 
 struct _MsgFlags
@@ -178,6 +177,12 @@ struct _MsgInfo
 	guint decryption_failed : 1;
 };
 
+struct _MsgFileInfo
+{
+	gchar *file;
+	MsgFlags *flags;
+};
+
 GHashTable *procmsg_msg_hash_table_create	(GSList		*mlist);
 void procmsg_msg_hash_table_append		(GHashTable	*msg_table,
 						 GSList		*mlist);
@@ -187,6 +192,9 @@ GSList *procmsg_read_cache		(FolderItem	*item,
 					 gboolean	 scan_file);
 void	procmsg_set_flags		(GSList		*mlist,
 					 FolderItem	*item);
+GSList *procmsg_sort_msg_list		(GSList		*mlist,
+					 FolderSortKey	 sort_key,
+					 FolderSortType	 sort_type);
 gint	procmsg_get_last_num_in_msg_list(GSList		*mlist);
 void	procmsg_msg_list_free		(GSList		*mlist);
 void	procmsg_write_cache		(MsgInfo	*msginfo,
@@ -198,14 +206,14 @@ void	procmsg_flush_mark_queue	(FolderItem	*item,
 void	procmsg_add_flags		(FolderItem	*item,
 					 gint		 num,
 					 MsgFlags	 flags);
-void	procmsg_get_mark_sum		(const gchar	*folder,
+void	procmsg_get_mark_sum		(FolderItem	*item,
 					 gint		*new,
 					 gint		*unread,
 					 gint		*total,
 					 gint		*min,
 					 gint		*max,
 					 gint		 first);
-FILE   *procmsg_open_mark_file		(const gchar	*folder,
+FILE   *procmsg_open_mark_file		(FolderItem	*item,
 					 gboolean	 append);
 
 GNode  *procmsg_get_thread_tree		(GSList		*mlist);
@@ -215,6 +223,8 @@ void	procmsg_copy_messages		(GSList		*mlist);
 
 gchar  *procmsg_get_message_file_path	(MsgInfo	*msginfo);
 gchar  *procmsg_get_message_file	(MsgInfo	*msginfo);
+GSList *procmsg_get_message_file_list	(GSList		*mlist);
+void	procmsg_message_file_list_free	(GSList		*file_list);
 FILE   *procmsg_open_message		(MsgInfo	*msginfo);
 #if USE_GPGME
 FILE   *procmsg_open_message_decrypted	(MsgInfo	*msginfo,
@@ -222,6 +232,11 @@ FILE   *procmsg_open_message_decrypted	(MsgInfo	*msginfo,
 #endif
 gboolean procmsg_msg_exist		(MsgInfo	*msginfo);
 
+void	procmsg_get_filter_keyword	(MsgInfo	  *msginfo,
+					 gchar	         **header,
+					 gchar	         **key,
+					 PrefsFilterType   type);
+
 void	procmsg_empty_trash		(void);
 gint	procmsg_send_queue		(FolderItem	*queue,
 					 gboolean	 save_msgs);
