@@ -61,8 +61,6 @@ static char *mime_version_name[] = {
     NULL
 };
 
-static char *create_boundary (void);
-
 #if 0
 static void dump_mimeinfo (const char *text, MimeInfo *x)
 {
@@ -510,7 +508,7 @@ void rfc2015_check_signature (MimeInfo *mimeinfo, FILE *fp)
 
 int rfc2015_is_encrypted (MimeInfo *mimeinfo)
 {
-    if (!mimeinfo)
+    if (!mimeinfo || mimeinfo->mime_type != MIME_MULTIPART)
         return 0;
     if (g_strcasecmp (mimeinfo->content_type, "multipart/encrypted"))
         return 0;
@@ -596,6 +594,7 @@ void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
     FILE *dstfp;
     size_t nread;
     char buf[BUFFSIZE];
+    int in_cline;
     GpgmeError err;
 
     g_return_if_fail (msginfo != NULL);
@@ -654,9 +653,17 @@ void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
     if (fseek(fp, tmpinfo->fpos, SEEK_SET) < 0)
         perror("fseek");
 
+    in_cline = 0;
     while (fgets(buf, sizeof(buf), fp)) {
-        if (headerp (buf, content_names))
+        if (headerp (buf, content_names)) {
+            in_cline = 1;
             continue;
+        }
+        if (in_cline) {
+            if (buf[0] == ' ' || buf[0] == '\t')
+                continue;
+            in_cline = 0;
+        }
         if (buf[0] == '\r' || buf[0] == '\n')
             break;
         fputs (buf, dstfp);
@@ -775,7 +782,9 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
     GpgmeRecipients rset = NULL;
     size_t nread;
     int mime_version_seen = 0;
-    char *boundary = create_boundary ();
+    char *boundary;
+
+    boundary = generate_mime_boundary ("Encrypt");
 
     /* Create the list of recipients */
     rset = gpgmegtk_recipient_selection (recp_list);
@@ -799,7 +808,7 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
     }
 
     /* get the content header lines from the source */
-    clineidx=0;
+    clineidx = 0;
     saved_last = 0;
     while (!err && fgets(buf, sizeof(buf), fp)) {
         /* fixme: check for overlong lines */
@@ -813,13 +822,13 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
             continue;
         }
         if (saved_last) {
-            saved_last = 0;
             if (*buf == ' ' || *buf == '\t') {
-                char *last = clines[clineidx-1];
-                clines[clineidx-1] = g_strconcat (last, buf, NULL);
+                char *last = clines[clineidx - 1];
+                clines[clineidx - 1] = g_strconcat (last, buf, NULL);
                 g_free (last);
                 continue;
             }
+            saved_last = 0;
         }
 
         if (headerp (buf, mime_version_name)) 
@@ -1107,10 +1116,12 @@ rfc2015_sign (const char *file, GSList *key_list)
     GpgmeData sigdata = NULL;
     size_t nread;
     int mime_version_seen = 0;
-    char *boundary = create_boundary ();
+    char *boundary;
     char *micalg = NULL;
     char *siginfo;
 
+    boundary = generate_mime_boundary ("Signature");
+
     /* Open the source file */
     if ((fp = fopen(file, "rb")) == NULL) {
         FILE_OP_ERROR(file, "fopen");
@@ -1140,13 +1151,13 @@ rfc2015_sign (const char *file, GSList *key_list)
             continue;
         }
         if (saved_last) {
-            saved_last = 0;
             if (*buf == ' ' || *buf == '\t') {
                 char *last = clines[clineidx - 1];
                 clines[clineidx - 1] = g_strconcat (last, buf, NULL);
                 g_free (last);
                 continue;
             }
+            saved_last = 0;
         }
 
         if (headerp (buf, mime_version_name)) 
@@ -1223,7 +1234,7 @@ rfc2015_sign (const char *file, GSList *key_list)
     fprintf (fp, "Content-Type: multipart/signed; "
              "protocol=\"application/pgp-signature\";\r\n");
     if (micalg)
-        fprintf (fp, " micalg=\"%s\";", micalg);
+        fprintf (fp, " micalg=\"%s\";\r\n", micalg);
     fprintf (fp, " boundary=\"%s\"\r\n", boundary);
 
     /* Part 1: signed material */
@@ -1381,59 +1392,4 @@ failure:
     return -1;
 }
 
-
-/****************
- * Create a new boundary in a way that it is very unlikely that this
- * will occur in the following text.  It would be easy to ensure
- * uniqueness if everything is either quoted-printable or base64
- * encoded (note that conversion is allowed), but because MIME bodies
- * may be nested, it may happen that the same boundary has already
- * been used. We avoid scanning the message for conflicts and hope the
- * best.
- *
- *   boundary := 0*69<bchars> bcharsnospace
- *   bchars := bcharsnospace / " "
- *   bcharsnospace := DIGIT / ALPHA / "'" / "(" / ")" /
- *                    "+" / "_" / "," / "-" / "." /
- *                    "/" / ":" / "=" / "?"  
- */
-
-static char *
-create_boundary (void)
-{
-    static char tbl[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-			"abcdefghijklmnopqrstuvwxyz"
-			"1234567890'()+_,./:=?";
-    char buf[17];
-    int i, equal;
-    int pid;
-
-    pid = getpid();
-
-    /* We make the boundary depend on the pid, so that all running
-     * processed generate different values even when they have been
-     * started within the same second and srand48(time(NULL)) has been
-     * used.  I can't see whether this is really an advantage but it
-     * doesn't do any harm.
-     */
-    equal = -1;
-    for(i = 0; i < sizeof(buf) - 1; i++) {
-	buf[i] = tbl[(lrand48() ^ pid) % (sizeof(tbl) - 1)]; /* fill with random */
-	if(buf[i] == '=' && equal == -1)
-	    equal = i;
-    }
-    buf[i] = 0;
-
-    /* now make sure that we do have the sequence "=." in it which cannot
-     * be matched by quoted-printable or base64 encoding */
-    if(equal != -1 && (equal+1) < i)
-	buf[equal+1] = '.';
-    else {
-	buf[0] = '=';
-	buf[1] = '.';
-    }
-
-    return g_strdup(buf);
-}
-
 #endif /* USE_GPGME */
