@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,17 +48,6 @@
 #include "compose.h"
 #include "displayheader.h"
 
-#define FONT_LOAD(font, s) \
-{ \
-	gchar *fontstr, *p; \
- \
-	Xstrdup_a(fontstr, s, ); \
-	if ((p = strchr(fontstr, ',')) != NULL) *p = '\0'; \
-	font = gdk_font_load(fontstr); \
-	if (!font) \
-		g_warning("Couldn't load the font '%s'\n", fontstr); \
-}
-
 typedef struct _RemoteURI	RemoteURI;
 
 struct _RemoteURI
@@ -98,6 +87,29 @@ static GdkColor error_color = {
 };
 #endif
 
+#if USE_GPGME
+static GdkColor good_sig_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0xbfff,
+	(gushort)0
+};
+
+static GdkColor nocheck_sig_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0,
+	(gushort)0xcfff
+};
+
+static GdkColor bad_sig_color = {
+	(gulong)0,
+	(gushort)0xefff,
+	(gushort)0,
+	(gushort)0
+};
+#endif
+
 static GdkFont *text_sb_font;
 static GdkFont *text_mb_font;
 static gint text_sb_font_orig_ascent;
@@ -122,6 +134,10 @@ static void textview_show_html		(TextView	*textview,
 static void textview_write_line		(TextView	*textview,
 					 const gchar	*str,
 					 CodeConverter	*conv);
+static void textview_write_link		(TextView	*textview,
+					 const gchar	*str,
+					 const gchar	*uri,
+					 CodeConverter	*conv);
 static GPtrArray *textview_scan_header	(TextView	*textview,
 					 FILE		*fp);
 static void textview_show_header	(TextView	*textview,
@@ -182,7 +198,7 @@ TextView *textview_create(void)
 		GtkStyle *style;
 		GdkFont *font;
 
-		FONT_LOAD(font, prefs_common.normalfont);
+		font = gtkut_font_load_from_fontset(prefs_common.normalfont);
 		if (font) {
 			style = gtk_style_copy(text_sb->style);
 			gdk_font_unref(style->font);
@@ -286,10 +302,13 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		return;
 	}
 
-	if (prefs_common.force_charset)
+	if (textview->messageview->forced_charset)
+		charset = textview->messageview->forced_charset;
+	else if (prefs_common.force_charset)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+
 	textview_set_font(textview, charset);
 	textview_clear(textview);
 
@@ -380,10 +399,13 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		is_rfc822_part = TRUE;
 	}
 
-	if (prefs_common.force_charset)
+	if (textview->messageview->forced_charset)
+		charset = textview->messageview->forced_charset;
+	else if (prefs_common.force_charset)
 		charset = prefs_common.force_charset;
 	else if (mimeinfo->charset)
 		charset = mimeinfo->charset;
+
 	textview_set_font(textview, charset);
 
 	text = GTK_STEXT(textview->text);
@@ -421,11 +443,6 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 	if (mimeinfo->mime_type == MIME_MULTIPART) return;
 
-	if (!mimeinfo->parent &&
-	    mimeinfo->mime_type != MIME_TEXT &&
-	    mimeinfo->mime_type != MIME_TEXT_HTML)
-		return;
-
 	if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0) {
 		perror("fseek");
 		return;
@@ -468,6 +485,18 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		g_snprintf(buf, sizeof(buf), "\n[%s (%d bytes)]\n",
 			   mimeinfo->content_type, mimeinfo->size);
 
+#if USE_GPGME
+	if (mimeinfo->sigstatus) {
+		GdkColor *color;
+		if (!strcmp(mimeinfo->sigstatus, _("Good signature")))
+			color = &good_sig_color;
+		else if (!strcmp(mimeinfo->sigstatus, _("BAD signature")))
+			color = &bad_sig_color;
+		else
+			color = &nocheck_sig_color;
+		gtk_stext_insert(text, NULL, color, NULL, buf, -1);
+	} else
+#endif
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
 		gtk_stext_insert(text, NULL, NULL, NULL, buf, -1);
@@ -478,7 +507,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 			gtk_stext_insert(text, NULL, NULL, NULL, buf, -1);
 		else
 			gtk_stext_insert(text, NULL, NULL, NULL, "\n", 1);
-		if (prefs_common.force_charset)
+		if (textview->messageview->forced_charset)
+			charset = textview->messageview->forced_charset;
+		else if (prefs_common.force_charset)
 			charset = prefs_common.force_charset;
 		else if (mimeinfo->charset)
 			charset = mimeinfo->charset;
@@ -599,7 +630,10 @@ static void textview_show_html(TextView *textview, FILE *fp,
 	g_return_if_fail(parser != NULL);
 
 	while ((str = html_parse(parser)) != NULL) {
-		textview_write_line(textview, str, NULL);
+		if (parser->href != NULL)
+			textview_write_link(textview, str, parser->href, NULL);
+		else
+			textview_write_line(textview, str, NULL);
 	}
 	html_parser_destroy(parser);
 }
@@ -758,6 +792,7 @@ static void textview_make_clickable_parts(TextView *textview,
 		{"http://",  strcasestr, get_uri_part,   make_uri_string},
 		{"https://", strcasestr, get_uri_part,   make_uri_string},
 		{"ftp://",   strcasestr, get_uri_part,   make_uri_string},
+		{"www.",     strcasestr, get_uri_part,   make_uri_string},
 		{"mailto:",  strcasestr, get_uri_part,   make_uri_string},
 		{"@",        strcasestr, get_email_part, make_email_string}
 	};
@@ -794,7 +829,7 @@ static void textview_make_clickable_parts(TextView *textview,
 
 		if (scanpos) {
 			/* check if URI can be parsed */
-			if (parser[last_index].parse(linebuf, scanpos, &bp, &ep)
+			if (parser[last_index].parse(walk, scanpos, &bp, &ep)
 			    && (ep - bp - 1) > strlen(parser[last_index].needle)) {
 					ADD_TXT_POS(bp, ep, last_index);
 					walk = ep;
@@ -847,20 +882,15 @@ static void textview_write_line(TextView *textview, const gchar *str,
 	GdkColor *fg_color;
 	gint quotelevel = -1;
 
-#if 0
-	if (!conv)
-		strncpy2(buf, str, sizeof(buf));
-	else if (conv_convert(conv, buf, sizeof(buf), str) < 0) {
-		gtk_stext_insert(text, textview->msgfont,
-				prefs_common.enable_color
-				? &error_color : NULL, NULL,
-				"*** Warning: code conversion failed ***\n",
-				-1);
-		return;
-	}
-#endif
-	if (!conv || conv_convert(conv, buf, sizeof(buf), str) < 0)
-		strncpy2(buf, str, sizeof(buf));
+	if (!conv) {
+		if (textview->text_is_mb)
+			conv_localetodisp(buf, sizeof(buf), str);
+		else
+			strncpy2(buf, str, sizeof(buf));
+	} else if (conv_convert(conv, buf, sizeof(buf), str) < 0)
+		conv_localetodisp(buf, sizeof(buf), str);
+	else if (textview->text_is_mb)
+		conv_unreadable_locale(buf);
 
 	strcrchomp(buf);
 	if (prefs_common.conv_mb_alnum) conv_mb_alnum(buf);
@@ -899,6 +929,40 @@ static void textview_write_line(TextView *textview, const gchar *str,
 					      fg_color, NULL, buf);
 }
 
+void textview_write_link(TextView *textview, const gchar *str,
+			 const gchar *uri, CodeConverter *conv)
+{
+	GtkSText *text = GTK_STEXT(textview->text);
+	gchar buf[BUFFSIZE];
+	gchar *bufp;
+	RemoteURI *r_uri;
+
+	if (*str == '\0')
+		return;
+
+	if (!conv) {
+		if (textview->text_is_mb)
+			conv_localetodisp(buf, sizeof(buf), str);
+		else
+			strncpy2(buf, str, sizeof(buf));
+	} else if (conv_convert(conv, buf, sizeof(buf), str) < 0)
+		conv_localetodisp(buf, sizeof(buf), str);
+	else if (textview->text_is_mb)
+		conv_unreadable_locale(buf);
+
+	strcrchomp(buf);
+
+	for (bufp = buf; isspace(*bufp); bufp++)
+		gtk_stext_insert(text, textview->msgfont, NULL, NULL, bufp, 1);
+
+	r_uri = g_new(RemoteURI, 1);
+	r_uri->uri = g_strdup(uri);
+	r_uri->start = gtk_stext_get_point(text);
+	gtk_stext_insert(text, textview->msgfont, &uri_color, NULL, bufp, -1);
+	r_uri->end = gtk_stext_get_point(text);
+	textview->uri_list = g_slist_append(textview->uri_list, r_uri);
+}
+
 void textview_clear(TextView *textview)
 {
 	GtkSText *text = GTK_STEXT(textview->text);
@@ -945,7 +1009,7 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 	/* In multi-byte mode, GtkSText can't display 8bit characters
 	   correctly, so it must be single-byte mode. */
 	if (MB_CUR_MAX > 1) {
-		if (codeset) {
+		if (codeset && conv_get_current_charset() != C_UTF_8) {
 			if (!g_strncasecmp(codeset, "ISO-8859-", 9) ||
 			    !g_strcasecmp(codeset, "BALTIC"))
 				use_fontset = FALSE;
@@ -1003,11 +1067,11 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 				text_sb_font->ascent = text_sb_font_orig_ascent;
 				text_sb_font->descent = text_sb_font_orig_descent;
 			}
-			if (MB_CUR_MAX > 1) {
-				FONT_LOAD(font, "-*-courier-medium-r-normal--14-*-*-*-*-*-iso8859-1");
-			} else {
-				FONT_LOAD(font, prefs_common.textfont);
-			}
+			if (MB_CUR_MAX > 1)
+				font = gdk_font_load("-*-courier-medium-r-normal--14-*-*-*-*-*-iso8859-1");
+			else
+				font = gtkut_font_load_from_fontset
+					(prefs_common.textfont);
 			if (font && text_sb_font != font) {
 				if (text_sb_font)
 					gdk_font_unref(text_sb_font);
@@ -1033,7 +1097,7 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 	}
 
 	if (!textview->boldfont && prefs_common.boldfont)
-		FONT_LOAD(textview->boldfont, prefs_common.boldfont);
+		textview->boldfont = gtkut_font_load(prefs_common.boldfont);
 	if (!spacingfont)
 		spacingfont = gdk_font_load("-*-*-medium-r-normal--6-*");
 }
@@ -1128,6 +1192,9 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		    !g_strcasecmp(header->name, "Cc"))
 			unfold_line(header->body);
 
+		if (textview->text_is_mb == TRUE)
+			conv_unreadable_locale(header->body);
+
 		if (prefs_common.enable_color &&
 		    (!strncmp(header->name, "X-Mailer", 8) ||
 		     !strncmp(header->name, "X-Newsreader", 12)) &&
@@ -1154,42 +1221,25 @@ gboolean textview_search_string(TextView *textview, const gchar *str,
 {
 	GtkSText *text = GTK_STEXT(textview->text);
 	gint pos;
-	wchar_t *wcs;
 	gint len;
-	gint text_len;
-	gboolean found = FALSE;
 
 	g_return_val_if_fail(str != NULL, FALSE);
 
-	wcs = strdup_mbstowcs(str);
-	g_return_val_if_fail(wcs != NULL, FALSE);
-	len = wcslen(wcs);
+	len = get_mbs_len(str);
+	g_return_val_if_fail(len >= 0, FALSE);
+
 	pos = textview->cur_pos;
 	if (pos < textview->body_pos)
 		pos = textview->body_pos;
-	text_len = gtk_stext_get_length(text);
-	if (text_len - pos < len) {
-		g_free(wcs);
-		return FALSE;
+
+	if ((pos = gtkut_stext_find(text, pos, str, case_sens)) != -1) {
+		gtk_editable_set_position(GTK_EDITABLE(text), pos + len);
+		gtk_editable_select_region(GTK_EDITABLE(text), pos, pos + len);
+		textview_set_position(textview, pos + len);
+		return TRUE;
 	}
 
-	for (; pos < text_len; pos++) {
-		if (text_len - pos < len) break;
-		if (gtkut_stext_match_string(text, pos, wcs, len, case_sens)
-		    == TRUE) {
-			gtk_editable_set_position(GTK_EDITABLE(text),
-						  pos + len);
-			gtk_editable_select_region(GTK_EDITABLE(text),
-						   pos, pos + len);
-			textview_set_position(textview, pos + len);
-			found = TRUE;
-			break;
-		}
-		if (text_len - pos == len) break;
-	}
-
-	g_free(wcs);
-	return found;
+	return FALSE;
 }
 
 gboolean textview_search_string_backward(TextView *textview, const gchar *str,
@@ -1499,9 +1549,10 @@ static gint textview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 			RemoteURI *uri = (RemoteURI *)cur->data;
 
 			if (textview->cur_pos >= uri->start &&
-			    textview->cur_pos <  uri->end) {
+			    textview->cur_pos <= uri->end) {
 				if (!g_strncasecmp(uri->uri, "mailto:", 7))
-					compose_new(NULL, uri->uri + 7, NULL);
+					compose_new(NULL, NULL, uri->uri + 7,
+						    NULL);
 				else
 					open_uri(uri->uri,
 						 prefs_common.uri_cmd);
