@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -145,6 +145,10 @@ void slist_free_strings		(GSList		*list);
 void hash_free_strings		(GHashTable	*table);
 void hash_free_value_mem	(GHashTable	*table);
 
+gint str_case_equal		(gconstpointer	 v,
+				 gconstpointer	 v2);
+guint str_case_hash		(gconstpointer	 key);
+
 void ptr_array_free_strings	(GPtrArray	*array);
 
 /* number-string conversion */
@@ -203,14 +207,18 @@ gint wcsncasecmp		(const wchar_t *s1,
 				 size_t		n);
 wchar_t *wcscasestr		(const wchar_t *haystack,
 				 const wchar_t *needle);
+gint get_mbs_len		(const gchar	*s);
 
-gboolean is_next_nonascii	(const wchar_t *s);
-gboolean is_next_mbs		(const wchar_t *s);
-wchar_t *find_wspace		(const wchar_t *s);
+gboolean is_next_nonascii	(const guchar *s);
+gint get_next_word_len		(const gchar *s);
 
 /* functions for string parsing */
 gint subject_compare			(const gchar	*s1,
 					 const gchar	*s2);
+gint subject_compare_for_sort		(const gchar	*s1,
+					 const gchar	*s2);
+void trim_subject_for_compare		(gchar		*str);
+void trim_subject_for_sort		(gchar		*str);
 void trim_subject			(gchar		*str);
 void eliminate_parenthesis		(gchar		*str,
 					 gchar		 op,
@@ -219,10 +227,6 @@ void extract_parenthesis		(gchar		*str,
 					 gchar		 op,
 					 gchar		 cl);
 
-void extract_one_parenthesis_with_skip_quote	(gchar		*str,
-						 gchar		 quote_chr,
-						 gchar		 op,
-						 gchar		 cl);
 void extract_parenthesis_with_skip_quote	(gchar		*str,
 						 gchar		 quote_chr,
 						 gchar		 op,
@@ -240,6 +244,7 @@ gchar *strrchr_with_skip_quote		(const gchar	*str,
 					 gint		 quote_chr,
 					 gint		 c);
 void extract_address			(gchar		*str);
+void extract_list_id_str		(gchar		*str);
 
 GSList *address_list_append		(GSList		*addr_list,
 					 const gchar	*str);
@@ -279,7 +284,8 @@ gchar **strsplit_with_quote		(const gchar	*str,
 					 const gchar	*delim,
 					 gint		 max_tokens);
 
-gchar *get_abbrev_newsgroup_name	(const gchar	*group);
+gchar *get_abbrev_newsgroup_name	(const gchar	*group,
+					 gint		 len);
 gchar *trim_string			(const gchar	*str,
 					 gint		 len);
 
@@ -338,9 +344,20 @@ gint copy_file_part		(FILE		*fp,
 				 off_t		 offset,
 				 size_t		 length,
 				 const gchar	*dest);
+
+gchar *canonicalize_str		(const gchar	*str);
 gint canonicalize_file		(const gchar	*src,
 				 const gchar	*dest);
 gint canonicalize_file_replace	(const gchar	*file);
+gint uncanonicalize_file	(const gchar	*src,
+				 const gchar	*dest);
+gint uncanonicalize_file_replace(const gchar	*file);
+
+gchar *normalize_newlines	(const gchar	*str);
+
+gchar *get_outgoing_rfc2822_str	(FILE		*fp);
+gchar *generate_mime_boundary	(const gchar	*prefix);
+
 gint change_file_mode_rw	(FILE		*fp,
 				 const gchar	*file);
 FILE *my_tmpfile		(void);
@@ -348,12 +365,14 @@ FILE *str_open_as_stream	(const gchar	*str);
 gint str_write_to_file		(const gchar	*str,
 				 const gchar	*file);
 gchar *file_read_to_str		(const gchar	*file);
+gchar *file_read_stream_to_str	(FILE		*fp);
 
 /* process execution */
 gint execute_async		(gchar *const	 argv[]);
 gint execute_sync		(gchar *const	 argv[]);
 gint execute_command_line	(const gchar	*cmdline,
 				 gboolean	 async);
+gchar *get_command_output	(const gchar	*cmdline);
 
 /* open URI with external browser */
 gint open_uri(const gchar *uri, const gchar *cmdline);
